@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -35,15 +35,8 @@
 #include "DFGPhase.h"
 #include "DFGSSACalculator.h"
 #include "DFGVariableAccessDataDump.h"
-#include "JSCInlines.h"
-
-#undef RELEASE_ASSERT
-#define RELEASE_ASSERT(assertion) do { \
-    if (!(assertion)) { \
-        WTFReportAssertionFailure(__FILE__, __LINE__, WTF_PRETTY_FUNCTION, #assertion); \
-        CRASH(); \
-    } \
-} while (0)
+#include "JSCJSValueInlines.h"
+#include "OperandsInlines.h"
 
 namespace JSC { namespace DFG {
 
@@ -52,7 +45,7 @@ class SSAConversionPhase : public Phase {
     
 public:
     SSAConversionPhase(Graph& graph)
-        : Phase(graph, "SSA conversion")
+        : Phase(graph, "SSA conversion"_s)
         , m_insertionSet(graph)
     {
     }
@@ -66,7 +59,7 @@ public:
         m_graph.clearReplacements();
         m_graph.clearCPSCFGData();
 
-        HashMap<unsigned, BasicBlock*, WTF::IntHash<unsigned>, WTF::UnsignedWithZeroKeyHashTraits<unsigned>> entrypointIndexToArgumentsBlock;
+        UncheckedKeyHashMap<unsigned, BasicBlock*, WTF::IntHash<unsigned>, WTF::UnsignedWithZeroKeyHashTraits<unsigned>> entrypointIndexToArgumentsBlock;
 
         m_graph.m_numberOfEntrypoints = m_graph.m_roots.size();
         m_graph.m_argumentFormats.resize(m_graph.m_numberOfEntrypoints);
@@ -118,10 +111,7 @@ public:
 
         m_graph.ensureSSADominators();
         
-        if (verbose) {
-            dataLog("Graph before SSA transformation:\n");
-            m_graph.dump();
-        }
+        dataLogIf(verbose, "Graph before SSA transformation:\n", m_graph);
 
         // Create a SSACalculator::Variable for every root VariableAccessData.
         for (VariableAccessData& variable : m_graph.m_variableAccessData) {
@@ -158,8 +148,8 @@ public:
                     childNode = m_insertionSet.insertNode(
                         nodeIndex, node->variableAccessData()->prediction(),
                         GetStack, node->origin,
-                        OpInfo(m_graph.m_stackAccessData.add(variable->local(), variable->flushFormat())));
-                    if (!ASSERT_DISABLED)
+                        OpInfo(m_graph.m_stackAccessData.add(variable->operand(), variable->flushFormat())));
+                    if (ASSERT_ENABLED)
                         m_argumentGetters.add(childNode);
                     m_argumentMapping.add(node, childNode);
                 }
@@ -178,7 +168,7 @@ public:
                 VariableAccessData* variable = m_variableForSSAIndex[ssaVariable->index()];
                 
                 // Prune by liveness. This doesn't buy us much other than compile times.
-                Node* headNode = block->variablesAtHead.operand(variable->local());
+                Node* headNode = block->variablesAtHead.operand(variable->operand());
                 if (!headNode)
                     return nullptr;
 
@@ -219,16 +209,18 @@ public:
             });
         
         if (verbose) {
-            dataLog("Computed Phis, about to transform the graph.\n");
-            dataLog("\n");
-            dataLog("Graph:\n");
-            m_graph.dump();
-            dataLog("\n");
-            dataLog("Mappings:\n");
-            for (unsigned i = 0; i < m_variableForSSAIndex.size(); ++i)
-                dataLog("    ", i, ": ", VariableAccessDataDump(m_graph, m_variableForSSAIndex[i]), "\n");
-            dataLog("\n");
-            dataLog("SSA calculator: ", calculator, "\n");
+            WTF::dataFile().atomically([&](auto&) {
+                dataLogLn("Computed Phis, about to transform the graph.");
+                dataLogLn();
+                dataLogLn("Graph:");
+                dataLog(m_graph);
+                dataLogLn();
+                dataLogLn("Mappings:");
+                for (unsigned i = 0; i < m_variableForSSAIndex.size(); ++i)
+                    dataLogLn("    ", i, ": ", VariableAccessDataDump(m_graph, m_variableForSSAIndex[i]));
+                dataLogLn();
+                dataLogLn("SSA calculator: ", calculator);
+            });
         }
         
         // Do the bulk of the SSA conversion. For each block, this tracks the operand->Node
@@ -279,8 +271,7 @@ public:
                     
                     VariableAccessData* variable = nodeAtHead->variableAccessData();
                     
-                    if (verbose)
-                        dataLog("Considering live variable ", VariableAccessDataDump(m_graph, variable), " at head of block ", *block, "\n");
+                    dataLogLnIf(verbose, "Considering live variable ", VariableAccessDataDump(m_graph, variable), " at head of block ", *block);
                     
                     SSACalculator::Variable* ssaVariable = m_ssaVariableForVariable.get(variable);
                     SSACalculator::Def* def = calculator.reachingDefAtHead(block, ssaVariable);
@@ -299,25 +290,28 @@ public:
                         node = node->replacement();
                         ASSERT(!node->replacement());
                     }
-                    if (verbose)
-                        dataLog("Mapping: ", VirtualRegister(valueForOperand.operandForIndex(i)), " -> ", node, "\n");
+                    dataLogLnIf(verbose, "Mapping: ", valueForOperand.operandForIndex(i), " -> ", node);
                     valueForOperand[i] = node;
                 }
             }
             
             // Insert Phis by asking the calculator what phis there are in this block. Also update
-            // valueForOperand with those Phis. For Phis associated with variables that are not
-            // flushed, we also insert a MovHint.
+            // valueForOperand with those Phis. We also insert a MovHint/PutStack pair to communicate
+            // the phi value to future phases like PutStack sinking.
             size_t phiInsertionPoint = 0;
             for (SSACalculator::Def* phiDef : calculator.phisForBlock(block)) {
                 VariableAccessData* variable = m_variableForSSAIndex[phiDef->variable()->index()];
                 
                 m_insertionSet.insert(phiInsertionPoint, phiDef->value());
-                valueForOperand.operand(variable->local()) = phiDef->value();
+                valueForOperand.operand(variable->operand()) = phiDef->value();
                 
                 m_insertionSet.insertNode(
                     phiInsertionPoint, SpecNone, MovHint, block->at(0)->origin.withInvalidExit(),
-                    OpInfo(variable->local().offset()), phiDef->value()->defaultEdge());
+                    OpInfo(variable->operand()), phiDef->value()->defaultEdge());
+                auto format = variable->flushFormat();
+                m_insertionSet.insertNode(
+                    phiInsertionPoint, SpecNone, PutStack, block->at(0)->origin.withInvalidExit(),
+                    OpInfo(m_graph.m_stackAccessData.add(variable->operand(), format)), Edge(phiDef->value(), uncheckedUseKindFor(format)));
             }
 
             if (block->at(0)->origin.exitOK)
@@ -327,8 +321,10 @@ public:
                 Node* node = block->at(nodeIndex);
                 
                 if (verbose) {
-                    dataLog("Processing node ", node, ":\n");
-                    m_graph.dump(WTF::dataFile(), "    ", node);
+                    WTF::dataFile().atomically([&](auto& out) {
+                        dataLogLn("Processing node ", node, ":");
+                        m_graph.dump(out, "    ", node);
+                    });
                 }
                 
                 m_graph.performSubstitution(node);
@@ -337,7 +333,7 @@ public:
                 case MovHint: {
                     m_insertionSet.insertNode(
                         nodeIndex, SpecNone, KillStack, node->origin,
-                        OpInfo(node->unlinkedLocal().offset()));
+                        OpInfo(node->unlinkedOperand()));
                     node->origin.exitOK = false; // KillStack clobbers exit.
                     break;
                 }
@@ -349,19 +345,18 @@ public:
                     if (!!(node->flags() & NodeIsFlushed)) {
                         node->convertToPutStack(
                             m_graph.m_stackAccessData.add(
-                                variable->local(), variable->flushFormat()));
+                                variable->operand(), variable->flushFormat()));
                     } else
                         node->remove(m_graph);
                     
-                    if (verbose)
-                        dataLog("Mapping: ", variable->local(), " -> ", child, "\n");
-                    valueForOperand.operand(variable->local()) = child;
+                    dataLogLnIf(verbose, "Mapping: ", variable->operand(), " -> ", child);
+                    valueForOperand.operand(variable->operand()) = child;
                     break;
                 }
                     
                 case GetStack: {
                     ASSERT(m_argumentGetters.contains(node));
-                    valueForOperand.operand(node->stackAccessData()->local) = node;
+                    valueForOperand.operand(node->stackAccessData()->operand) = node;
                     break;
                 }
                     
@@ -370,9 +365,8 @@ public:
                     node->children.reset();
                     
                     node->remove(m_graph);
-                    if (verbose)
-                        dataLog("Replacing node ", node, " with ", valueForOperand.operand(variable->local()), "\n");
-                    node->setReplacement(valueForOperand.operand(variable->local()));
+                    dataLogLnIf(verbose, "Replacing node ", node, " with ", valueForOperand.operand(variable->operand()));
+                    node->setReplacement(valueForOperand.operand(variable->operand()));
                     break;
                 }
                     
@@ -385,7 +379,7 @@ public:
                 case PhantomLocal: {
                     ASSERT(node->child1().useKind() == UntypedUse);
                     VariableAccessData* variable = node->variableAccessData();
-                    node->child1() = valueForOperand.operand(variable->local())->defaultEdge();
+                    node->child1() = valueForOperand.operand(variable->operand())->defaultEdge();
                     node->remove(m_graph);
                     break;
                 }
@@ -425,12 +419,12 @@ public:
                     // is not exitOK.
                     UseKind useKind = uncheckedUseKindFor(format);
 
-                    dataLogLnIf(verbose, "Inserting Upsilon for ", variable->local(), " propagating ", valueForOperand.operand(variable->local()), " to ", phiNode);
+                    dataLogLnIf(verbose, "Inserting Upsilon for ", variable->operand(), " propagating ", valueForOperand.operand(variable->operand()), " to ", phiNode);
                     
                     m_insertionSet.insertNode(
                         upsilonInsertionPoint, SpecNone, Upsilon, upsilonOrigin,
                         OpInfo(phiNode), Edge(
-                            valueForOperand.operand(variable->local()),
+                            valueForOperand.operand(variable->operand()),
                             useKind));
                 }
             }
@@ -457,13 +451,12 @@ public:
             unsigned entrypointIndex = pair.key;
             BasicBlock* oldRoot = pair.value;
             ArgumentsVector& arguments = m_graph.m_rootToArguments.find(oldRoot)->value;
-            Vector<FlushFormat> argumentFormats;
-            argumentFormats.reserveInitialCapacity(arguments.size());
-            for (unsigned i = 0; i < arguments.size(); ++i) {
-                Node* node = m_argumentMapping.get(arguments[i]);
+            auto argumentFormats = arguments.map([&](auto& argument) {
+                Node* node = m_argumentMapping.get(argument);
                 RELEASE_ASSERT(node);
-                argumentFormats.uncheckedAppend(node->stackAccessData()->format);
-            }
+                return node->stackAccessData()->format;
+            });
+
             m_graph.m_argumentFormats[entrypointIndex] = WTFMove(argumentFormats);
         }
 
@@ -474,19 +467,16 @@ public:
 
         m_graph.m_form = SSA;
 
-        if (verbose) {
-            dataLog("Graph after SSA transformation:\n");
-            m_graph.dump();
-        }
+        dataLogIf(verbose, "Graph after SSA transformation:\n", m_graph);
 
         return true;
     }
 
 private:
     InsertionSet m_insertionSet;
-    HashMap<VariableAccessData*, SSACalculator::Variable*> m_ssaVariableForVariable;
-    HashMap<Node*, Node*> m_argumentMapping;
-    HashSet<Node*> m_argumentGetters;
+    UncheckedKeyHashMap<VariableAccessData*, SSACalculator::Variable*> m_ssaVariableForVariable;
+    UncheckedKeyHashMap<Node*, Node*> m_argumentMapping;
+    UncheckedKeyHashSet<Node*> m_argumentGetters;
     Vector<VariableAccessData*> m_variableForSSAIndex;
 };
 
