@@ -37,6 +37,7 @@
 #include "DFGInsertionSet.h"
 #include "DFGPhase.h"
 #include "JSCInlines.h"
+#include "StructureID.h"
 #include <wtf/CommaPrinter.h>
 #include <wtf/HashSet.h>
 
@@ -71,17 +72,14 @@ template<PhaseMode mode>
 class StoreBarrierInsertionPhase : public Phase {
 public:
     StoreBarrierInsertionPhase(Graph& graph)
-        : Phase(graph, mode == PhaseMode::Fast ? "fast store barrier insertion" : "global store barrier insertion")
+        : Phase(graph, mode == PhaseMode::Fast ? "fast store barrier insertion"_s : "global store barrier insertion"_s)
         , m_insertionSet(graph)
     {
     }
     
     bool run()
     {
-        if (DFGStoreBarrierInsertionPhaseInternal::verbose) {
-            dataLog("Starting store barrier insertion:\n");
-            m_graph.dump();
-        }
+        dataLogIf(DFGStoreBarrierInsertionPhaseInternal::verbose, "Starting store barrier insertion:\n", m_graph);
         
         switch (mode) {
         case PhaseMode::Fast: {
@@ -108,8 +106,8 @@ public:
             // towards believing that all nodes need barriers. "Needing a barrier" is like
             // saying that the node is in a past epoch. "Not needing a barrier" is like saying
             // that the node is in the current epoch.
-            m_stateAtHead = makeUnique<BlockMap<HashSet<Node*>>>(m_graph);
-            m_stateAtTail = makeUnique<BlockMap<HashSet<Node*>>>(m_graph);
+            m_stateAtHead = makeUnique<BlockMap<UncheckedKeyHashSet<Node*>>>(m_graph);
+            m_stateAtTail = makeUnique<BlockMap<UncheckedKeyHashSet<Node*>>>(m_graph);
             
             BlockList postOrder = m_graph.blocksInPostOrder();
             
@@ -171,9 +169,8 @@ private:
     bool handleBlock(BasicBlock* block)
     {
         if (DFGStoreBarrierInsertionPhaseInternal::verbose) {
-            dataLog("Dealing with block ", pointerDump(block), "\n");
-            if (reallyInsertBarriers())
-                dataLog("    Really inserting barriers.\n");
+            dataLogLn("Dealing with block ", pointerDump(block));
+            dataLogLnIf(reallyInsertBarriers(), "    Really inserting barriers.");
         }
         
         m_currentEpoch = Epoch::first();
@@ -206,21 +203,23 @@ private:
 
         bool result = true;
 
-        HashMap<AbstractHeap, Node*> potentialStackEscapes;
+        UncheckedKeyHashMap<AbstractHeap, Node*> potentialStackEscapes;
         
         for (m_nodeIndex = 0; m_nodeIndex < block->size(); ++m_nodeIndex) {
             m_node = block->at(m_nodeIndex);
             
             if (DFGStoreBarrierInsertionPhaseInternal::verbose) {
-                dataLog(
-                    "    ", m_currentEpoch, ": Looking at node ", m_node, " with children: ");
-                CommaPrinter comma;
-                m_graph.doToChildren(
-                    m_node,
-                    [&] (Edge edge) {
-                        dataLog(comma, edge, " (", edge->epoch(), ")");
-                    });
-                dataLog("\n");
+                WTF::dataFile().atomically([&](auto&) {
+                    dataLog(
+                        "    ", m_currentEpoch, ": Looking at node ", m_node, " with children: ");
+                    CommaPrinter comma;
+                    m_graph.doToChildren(
+                        m_node,
+                        [&] (Edge edge) {
+                            dataLog(comma, edge, " (", edge->epoch(), ")");
+                        });
+                    dataLogLn();
+                });
             }
             
             if (mode == PhaseMode::Global) {
@@ -234,10 +233,10 @@ private:
             switch (m_node->op()) {
             case PutByValDirect:
             case PutByVal:
-            case PutByValAlias:
-            case PutByValMegamorphic: {
+            case PutByValAlias: {
                 switch (m_node->arrayMode().modeForPut().type()) {
                 case Array::Generic:
+                case Array::Float16Array:
                 case Array::BigInt64Array:
                 case Array::BigUint64Array: {
                     Edge child1 = m_graph.varArgChild(m_node, 0);
@@ -333,15 +332,16 @@ private:
                 break;
             }
 
-            case EnumeratorPutByVal: {
+            case EnumeratorPutByVal:
+            case PutByValMegamorphic: {
                 Edge child1 = m_graph.varArgChild(m_node, 0);
-                Edge child3 = m_graph.varArgChild(m_node, 2);
-                considerBarrier(child1, child3);
+                considerBarrier(child1);
                 break;
             }
                 
             case MultiPutByOffset:
             case MultiDeleteByOffset: {
+                // These nodes may cause transition too.
                 considerBarrier(m_node->child1());
                 break;
             }
@@ -382,13 +382,17 @@ private:
             case NewArray:
             case NewArrayWithSize:
             case NewArrayWithConstantSize:
+            case NewArrayWithSizeAndStructure:
             case NewArrayBuffer:
             case NewInternalFieldObject:
             case NewTypedArray:
             case NewRegexp:
             case NewStringObject:
+            case NewMap:
+            case NewSet:
             case NewSymbol:
             case MaterializeNewObject:
+            case MaterializeNewArrayWithConstantSize:
             case MaterializeCreateActivation:
             case MakeRope:
             case MakeAtomString:
@@ -518,16 +522,18 @@ private:
             }
             
             if (DFGStoreBarrierInsertionPhaseInternal::verbose) {
-                dataLog(
-                    "    ", m_currentEpoch, ": Done with node ", m_node, " (", m_node->epoch(),
-                    ") with children: ");
-                CommaPrinter comma;
-                m_graph.doToChildren(
-                    m_node,
-                    [&] (Edge edge) {
-                        dataLog(comma, edge, " (", edge->epoch(), ")");
-                    });
-                dataLog("\n");
+                WTF::dataFile().atomically([&](auto&) {
+                    dataLog(
+                        "    ", m_currentEpoch, ": Done with node ", m_node, " (", m_node->epoch(),
+                        ") with children: ");
+                    CommaPrinter comma;
+                    m_graph.doToChildren(
+                        m_node,
+                        [&] (Edge edge) {
+                            dataLog(comma, edge, " (", edge->epoch(), ")");
+                        });
+                    dataLogLn();
+                });
             }
             
             if (mode == PhaseMode::Global) {
@@ -555,8 +561,7 @@ private:
     
     void considerBarrier(Edge base, Edge child)
     {
-        if (DFGStoreBarrierInsertionPhaseInternal::verbose)
-            dataLog("        Considering adding barrier ", base, " => ", child, "\n");
+        dataLogLnIf(DFGStoreBarrierInsertionPhaseInternal::verbose, "        Considering adding barrier ", base, " => ", child);
         
         // We don't need a store barrier if the child is guaranteed to not be a cell.
         switch (mode) {
@@ -564,8 +569,7 @@ private:
             // Don't try too hard because it's too expensive to run AI.
             if (child->hasConstant()) {
                 if (!child->asJSValue().isCell()) {
-                    if (DFGStoreBarrierInsertionPhaseInternal::verbose)
-                        dataLog("            Rejecting because of constant type.\n");
+                    dataLogLnIf(DFGStoreBarrierInsertionPhaseInternal::verbose, "            Rejecting because of constant type.");
                     return;
                 }
             } else {
@@ -575,8 +579,7 @@ private:
                 case NodeResultInt32:
                 case NodeResultInt52:
                 case NodeResultBoolean:
-                    if (DFGStoreBarrierInsertionPhaseInternal::verbose)
-                        dataLog("            Rejecting because of result type.\n");
+                    dataLogLnIf(DFGStoreBarrierInsertionPhaseInternal::verbose, "            Rejecting because of result type.");
                     return;
                 default:
                     break;
@@ -589,8 +592,7 @@ private:
             // Go into rage mode to eliminate any chance of a barrier with a non-cell child. We
             // can afford to keep around AI in Global mode.
             if (!m_interpreter->needsTypeCheck(child, ~SpecCell)) {
-                if (DFGStoreBarrierInsertionPhaseInternal::verbose)
-                    dataLog("            Rejecting because of AI type.\n");
+                dataLogLnIf(DFGStoreBarrierInsertionPhaseInternal::verbose, "            Rejecting because of AI type.");
                 return;
             }
             break;
@@ -601,21 +603,18 @@ private:
     
     void considerBarrier(Edge base)
     {
-        if (DFGStoreBarrierInsertionPhaseInternal::verbose)
-            dataLog("        Considering adding barrier on ", base, "\n");
+        dataLogLnIf(DFGStoreBarrierInsertionPhaseInternal::verbose, "        Considering adding barrier on ", base);
         
         // We don't need a store barrier if the epoch of the base is identical to the current
         // epoch. That means that we either just allocated the object and so it's guaranteed to
         // be in newgen, or we just ran a barrier on it so it's guaranteed to be remembered
         // already.
         if (base->epoch() == m_currentEpoch) {
-            if (DFGStoreBarrierInsertionPhaseInternal::verbose)
-                dataLog("            Rejecting because it's in the current epoch.\n");
+            dataLogLnIf(DFGStoreBarrierInsertionPhaseInternal::verbose, "            Rejecting because it's in the current epoch.");
             return;
         }
         
-        if (DFGStoreBarrierInsertionPhaseInternal::verbose)
-            dataLog("            Inserting barrier.\n");
+        dataLogLnIf(DFGStoreBarrierInsertionPhaseInternal::verbose, "            Inserting barrier.");
         insertBarrier(m_nodeIndex + 1, base);
     }
 
@@ -660,8 +659,8 @@ private:
     // Things we only use in Global mode.
     std::unique_ptr<InPlaceAbstractState> m_state;
     std::unique_ptr<AbstractInterpreter<InPlaceAbstractState>> m_interpreter;
-    std::unique_ptr<BlockMap<HashSet<Node*>>> m_stateAtHead;
-    std::unique_ptr<BlockMap<HashSet<Node*>>> m_stateAtTail;
+    std::unique_ptr<BlockMap<UncheckedKeyHashSet<Node*>>> m_stateAtHead;
+    std::unique_ptr<BlockMap<UncheckedKeyHashSet<Node*>>> m_stateAtTail;
     bool m_isConverged;
 };
 
