@@ -33,12 +33,14 @@
 #include "B3MemoryValue.h"
 #include "B3PatchpointValue.h"
 #include "B3ValueInlines.h"
+#include "ClonedArguments.h"
 #include "DateInstance.h"
 #include "DirectArguments.h"
 #include "FTLState.h"
 #include "GetterSetter.h"
 #include "JSPropertyNameEnumerator.h"
 #include "JSWebAssemblyInstance.h"
+#include "JSWrapperObject.h"
 #include "RegExpObject.h"
 #include "ScopedArguments.h"
 #include "ShadowChicken.h"
@@ -97,9 +99,7 @@ AbstractHeapRepository::AbstractHeapRepository()
     RELEASE_ASSERT(!JSCell_freeListNext.offset());
 }
 
-AbstractHeapRepository::~AbstractHeapRepository()
-{
-}
+AbstractHeapRepository::~AbstractHeapRepository() = default;
 
 void AbstractHeapRepository::decorateMemory(const AbstractHeap* heap, B3::Value* value)
 {
@@ -146,9 +146,11 @@ void AbstractHeapRepository::computeRangesAndDecorateInstructions()
     using namespace B3;
     root.compute();
 
-    if (verboseCompilationEnabled()) {
-        dataLog("Abstract Heap Repository:\n");
-        root.deepDump(WTF::dataFile());
+    if (UNLIKELY(verboseCompilationEnabled())) {
+        WTF::dataFile().atomically([&](auto&) {
+            dataLogLn("Abstract Heap Repository:");
+            root.deepDump(WTF::dataFile());
+        });
     }
     
     auto rangeFor = [&] (const AbstractHeap* heap) -> HeapRange {
