@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2024 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,6 +27,7 @@
 #include "DeferredWorkTimer.h"
 
 #include "CatchScope.h"
+#include "DeferredWorkTimerInlines.h"
 #include "GlobalObjectMethodTable.h"
 #include "JSGlobalObject.h"
 #include "VM.h"
@@ -35,23 +36,25 @@
 
 namespace JSC {
 
-WTF_MAKE_TZONE_ALLOCATED_IMPL_NESTED(DeferredWorkTimerTicketData, DeferredWorkTimer::TicketData);
+WTF_MAKE_TZONE_ALLOCATED_IMPL(DeferredWorkTimer::TicketData);
 
 namespace DeferredWorkTimerInternal {
 static constexpr bool verbose = false;
 }
 
-inline DeferredWorkTimer::TicketData::TicketData(JSGlobalObject* globalObject, JSObject* scriptExecutionOwner, Vector<Weak<JSCell>>&& dependencies)
-    : m_dependencies(WTFMove(dependencies))
+inline DeferredWorkTimer::TicketData::TicketData(WorkType type, JSObject* scriptExecutionOwner, Vector<JSCell*>&& dependencies)
+    : m_type(type)
+    , m_dependencies(WTFMove(dependencies))
     , m_scriptExecutionOwner(scriptExecutionOwner)
-    , m_globalObject(globalObject)
 {
-    globalObject->addWeakTicket(this);
+    ASSERT_WITH_MESSAGE(!m_dependencies.isEmpty(), "dependencies shouldn't be empty since it should contain the target");
+    ASSERT_WITH_MESSAGE(isTargetObject(), "target must be a JSObject");
+    target()->globalObject()->addWeakTicket(this);
 }
 
-inline Ref<DeferredWorkTimer::TicketData> DeferredWorkTimer::TicketData::create(JSGlobalObject* globalObject, JSObject* scriptExecutionOwner, Vector<Weak<JSCell>>&& dependencies)
+inline Ref<DeferredWorkTimer::TicketData> DeferredWorkTimer::TicketData::create(WorkType type, JSObject* scriptExecutionOwner, Vector<JSCell*>&& dependencies)
 {
-    return adoptRef(*new TicketData(globalObject, scriptExecutionOwner, WTFMove(dependencies)));
+    return adoptRef(*new TicketData(type, scriptExecutionOwner, WTFMove(dependencies)));
 }
 
 inline VM& DeferredWorkTimer::TicketData::vm()
@@ -62,9 +65,15 @@ inline VM& DeferredWorkTimer::TicketData::vm()
 
 inline void DeferredWorkTimer::TicketData::cancel()
 {
-    m_scriptExecutionOwner.clear();
+    dataLogLnIf(DeferredWorkTimerInternal::verbose, "Canceling ticket: ", RawPointer(this));
+    m_isCancelled = true;
+}
+
+inline void DeferredWorkTimer::TicketData::cancelAndClear()
+{
+    cancel();
     m_dependencies.clear();
-    m_globalObject.clear();
+    m_scriptExecutionOwner = nullptr;
 }
 
 DeferredWorkTimer::DeferredWorkTimer(VM& vm)
@@ -158,21 +167,21 @@ void DeferredWorkTimer::runRunLoop()
     ASSERT(!m_apiLock->vm()->currentThreadIsHoldingAPILock());
     ASSERT(&RunLoop::current() == &m_apiLock->vm()->runLoop());
     m_shouldStopRunLoopWhenAllTicketsFinish = true;
-    if (m_pendingTickets.size())
+    if (!m_pendingTickets.isEmpty())
         RunLoop::run();
 }
 
-DeferredWorkTimer::Ticket DeferredWorkTimer::addPendingWork(VM& vm, JSObject* target, Vector<Weak<JSCell>>&& dependencies)
+DeferredWorkTimer::Ticket DeferredWorkTimer::addPendingWork(WorkType type, VM& vm, JSObject* target, Vector<JSCell*>&& dependencies)
 {
     ASSERT_UNUSED(vm, vm.currentThreadIsHoldingAPILock() || (Thread::mayBeGCThread() && vm.heap.worldIsStopped()));
     for (unsigned i = 0; i < dependencies.size(); ++i)
-        ASSERT(dependencies[i].get() != target && dependencies[i].get());
+        ASSERT(dependencies[i] != target && dependencies[i]);
 
     auto* globalObject = target->globalObject();
     JSObject* scriptExecutionOwner = globalObject->globalObjectMethodTable()->currentScriptExecutionOwner(globalObject);
-    dependencies.append(Weak<JSCell>(target));
+    dependencies.append(target);
 
-    auto ticketData = TicketData::create(globalObject, scriptExecutionOwner, WTFMove(dependencies));
+    auto ticketData = TicketData::create(type, scriptExecutionOwner, WTFMove(dependencies));
     Ticket ticket = ticketData.ptr();
 
     dataLogLnIf(DeferredWorkTimerInternal::verbose, "Adding new pending ticket: ", RawPointer(ticket));
@@ -208,6 +217,9 @@ void DeferredWorkTimer::scheduleWorkSoon(Ticket ticket, Task&& task)
         setTimeUntilFire(0_s);
 }
 
+// Since TicketData is ThreadSafeWeakPtr now, we should optimize the DeferredWorkTimer's
+// workflow, e.g. directly clear the TicketData from cancelPendingWork.
+// https://bugs.webkit.org/show_bug.cgi?id=276538
 bool DeferredWorkTimer::cancelPendingWork(Ticket ticket)
 {
     ASSERT(m_pendingTickets.contains(ticket));
@@ -215,7 +227,6 @@ bool DeferredWorkTimer::cancelPendingWork(Ticket ticket)
 
     bool result = false;
     if (!ticket->isCancelled()) {
-        dataLogLnIf(DeferredWorkTimerInternal::verbose, "Canceling ticket: ", RawPointer(ticket));
         ticket->cancel();
         result = true;
     }
@@ -226,15 +237,50 @@ bool DeferredWorkTimer::cancelPendingWork(Ticket ticket)
 void DeferredWorkTimer::cancelPendingWorkSafe(JSGlobalObject* globalObject)
 {
     Locker locker { m_taskLock };
+
+    dataLogLnIf(DeferredWorkTimerInternal::verbose, "Cancel pending work for globalObject ", RawPointer(globalObject));
     for (Ref<TicketData> ticket : *globalObject->m_weakTickets) {
         if (!ticket->isCancelled())
             cancelPendingWork(ticket.ptr());
-        m_tasks.append(std::make_tuple(ticket.ptr(), [](DeferredWorkTimer::Ticket) mutable { }));
+        m_tasks.append(std::make_tuple(ticket.ptr(), [](DeferredWorkTimer::Ticket) { }));
     }
     if (!isScheduled() && !m_currentlyRunningTask)
         setTimeUntilFire(0_s);
 }
 
+void DeferredWorkTimer::cancelPendingWork(VM& vm)
+{
+    ASSERT(vm.heap.isInPhase(CollectorPhase::End));
+    Locker locker { m_taskLock };
+
+    dataLogLnIf(DeferredWorkTimerInternal::verbose, "Cancel pending work for vm ", RawPointer(&vm));
+    auto isValid = [&](auto& ticket) {
+        bool isTargetGlobalObjectLive = vm.heap.isMarked(ticket->target()->globalObject());
+#if ASSERT_ENABLED
+        if (isTargetGlobalObjectLive) {
+            for (JSCell* dependency : ticket->dependencies())
+                ASSERT(vm.heap.isMarked(dependency));
+        }
+#endif
+        return isTargetGlobalObjectLive && vm.heap.isMarked(ticket->scriptExecutionOwner());
+    };
+
+    bool needToFire = false;
+    for (auto& ticket : m_pendingTickets) {
+        if (ticket->isCancelled() || !isValid(ticket)) {
+            // At this point, no one can visit or need the dependencies.
+            // So, they are safe to clear here for better debugging and testing.
+            ticket->cancelAndClear();
+            needToFire = true;
+        }
+    }
+    // GC can be triggered before an invalid and scheduled ticket is fired. In that case,
+    // we also need to remove the corresponding pending task. Since doWork handles all cases
+    // for removal, we should let it handle that for consistency.
+    if (needToFire && !isScheduled() && !m_currentlyRunningTask)
+        setTimeUntilFire(0_s);
+}
+
 void DeferredWorkTimer::didResumeScriptExecutionOwner()
 {
     ASSERT(!m_currentlyRunningTask);
@@ -249,4 +295,16 @@ bool DeferredWorkTimer::hasAnyPendingWork() const
     return !m_pendingTickets.isEmpty();
 }
 
+bool DeferredWorkTimer::hasImminentlyScheduledWork() const
+{
+    ASSERT(m_apiLock->vm()->currentThreadIsHoldingAPILock() || (Thread::mayBeGCThread() && m_apiLock->vm()->heap.worldIsStopped()));
+    for (auto& ticket : m_pendingTickets) {
+        if (ticket->isCancelled())
+            continue;
+        if (ticket->type() == WorkType::ImminentlyScheduled)
+            return true;
+    }
+    return false;
+}
+
 } // namespace JSC
