@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2023 Apple Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,25 +31,21 @@
 #include "IteratorOperations.h"
 #include "JSCInlines.h"
 #include "ObjectConstructor.h"
+#include <wtf/text/MakeString.h>
 
 // While UListFormatter APIs are draft in ICU 67, they are stable in ICU 68 with the same function signatures.
 // So we can assume that these signatures of draft APIs are stable.
 // If UListFormatter is available, UNumberFormatter is also available.
-#if HAVE(ICU_U_LIST_FORMATTER)
 #ifdef U_HIDE_DRAFT_API
 #undef U_HIDE_DRAFT_API
 #endif
-#endif
 #include <unicode/ulistformatter.h>
 #include <unicode/unumberformatter.h>
 #include <unicode/ures.h>
-#if HAVE(ICU_U_LIST_FORMATTER)
 #define U_HIDE_DRAFT_API 1
-#endif
-
-#if HAVE(ICU_U_LIST_FORMATTER)
 #include <unicode/uformattedvalue.h>
-#endif
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
 
 namespace JSC {
 namespace IntlDurationFormatInternal {
@@ -244,7 +240,6 @@ void IntlDurationFormat::initializeDurationFormat(JSGlobalObject* globalObject,
     m_fractionalDigits = intlNumberOption(globalObject, options, vm.propertyNames->fractionalDigits, 0, 9, fractionalDigitsUndefinedValue);
     RETURN_IF_EXCEPTION(scope, void());
 
-#if HAVE(ICU_U_LIST_FORMATTER)
     {
         auto toUListFormatterWidth = [](Style style) {
             // 6. Let listStyle be durationFormat.[[Style]].
@@ -271,15 +266,8 @@ void IntlDurationFormat::initializeDurationFormat(JSGlobalObject* globalObject,
             return;
         }
     }
-#else
-    UNUSED_PARAM(IntlDurationFormatInternal::verbose);
-    throwTypeError(globalObject, scope, "Failed to initialize Intl.DurationFormat since this feature is not supported in the linked ICU version"_s);
-    return;
-#endif
 }
 
-#if HAVE(ICU_U_LIST_FORMATTER)
-
 static String retrieveSeparator(const CString& locale, const String& numberingSystem)
 {
     ASCIILiteral fallbackTimeSeparator = ":"_s;
@@ -316,9 +304,9 @@ enum class ElementType : uint8_t {
 
 struct Element {
     ElementType m_type;
+    bool m_valueSignBit;
     TemporalUnit m_unit;
     String m_string;
-    double m_value;
     std::unique_ptr<UFormattedNumber, ICUDeleter<unumf_closeResult>> m_formattedNumber;
 };
 
@@ -340,10 +328,69 @@ static DurationSignType getDurationSign(ISO8601::Duration duration)
     return DurationSignType::Zero;
 }
 
+static String int128ToString(Int128 value)
+{
+    Vector<LChar> resultString;
+    bool isNegative = value < 0;
+    if (isNegative)
+        value = -value;
+
+    while (value) {
+        Int128 digit = value % 10;
+        resultString.append(static_cast<char>('0' + digit));
+        value /= 10;
+    }
+
+    if (isNegative)
+        resultString.append('-');
+
+    std::reverse(resultString.begin(), resultString.end());
+
+    return StringImpl::adopt(WTFMove(resultString));
+}
+
+static String buildDecimalFormat(TemporalUnit unit, Int128 ns)
+{
+    ASSERT(unit == TemporalUnit::Second || unit == TemporalUnit::Millisecond || unit == TemporalUnit::Microsecond);
+
+    int flactionalDigits = 0;
+    Int128 exponent = 0;
+    if (unit == TemporalUnit::Second) {
+        flactionalDigits = 9;
+        exponent = Int128(1000000000);
+    } else if (unit == TemporalUnit::Millisecond) {
+        flactionalDigits = 6;
+        exponent = Int128(1000000);
+    } else {
+        ASSERT(unit == TemporalUnit::Microsecond);
+        flactionalDigits = 3;
+        exponent = Int128(1000);
+    }
+
+    Int128 integerPart = ns / exponent;
+    ASSERT(ns % exponent >= std::numeric_limits<int64_t>::min() && ns % exponent <= std::numeric_limits<int64_t>::max());
+    int64_t fractionalPart = std::abs(static_cast<int64_t>(ns % exponent));
+
+    StringBuilder builder;
+
+    builder.append(int128ToString(integerPart));
+    builder.append("."_s);
+
+    String fractionalString = String::number(fractionalPart);
+    int zeroLength = flactionalDigits - fractionalString.length();
+    for (int i = 0; i < zeroLength; ++i)
+        builder.append("0"_s);
+    builder.append(fractionalString);
+
+    return builder.toString();
+}
+
 static Vector<Element> collectElements(JSGlobalObject* globalObject, const IntlDurationFormat* durationFormat, ISO8601::Duration duration)
 {
     // https://tc39.es/proposal-intl-duration-format/#sec-partitiondurationformatpattern
 
+    ASSERT(isValidDuration(duration));
+
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
@@ -356,15 +403,16 @@ static Vector<Element> collectElements(JSGlobalObject* globalObject, const IntlD
         TemporalUnit unit = static_cast<TemporalUnit>(index);
         auto unitData = durationFormat->units()[index];
         double value = duration[unit];
+        std::optional<Int128> totalNanosecondsValue;
 
         StringBuilder skeletonBuilder;
-        skeletonBuilder.append("rounding-mode-half-up"_s);
 
         switch (unit) {
         // 3.j. If unit is "seconds", "milliseconds", or "microseconds", then
         case TemporalUnit::Second:
         case TemporalUnit::Millisecond:
         case TemporalUnit::Microsecond: {
+            skeletonBuilder.append("rounding-mode-down"_s);
             IntlDurationFormat::UnitStyle nextStyle = IntlDurationFormat::UnitStyle::Long;
             if (unit == TemporalUnit::Second)
                 nextStyle = durationFormat->units()[static_cast<unsigned>(TemporalUnit::Millisecond)].style();
@@ -376,13 +424,15 @@ static Vector<Element> collectElements(JSGlobalObject* globalObject, const IntlD
             }
             if (nextStyle == IntlDurationFormat::UnitStyle::Numeric) {
                 if (unit == TemporalUnit::Second)
-                    value = value + duration[TemporalUnit::Millisecond] / 1000.0 + duration[TemporalUnit::Microsecond] / 1000000.0 + duration[TemporalUnit::Nanosecond] / 1000000000.0;
+                    totalNanosecondsValue = duration.totalNanoseconds<TemporalUnit::Second>();
                 else if (unit == TemporalUnit::Millisecond)
-                    value = value + duration[TemporalUnit::Microsecond] / 1000.0 + duration[TemporalUnit::Nanosecond] / 1000000.0;
+                    totalNanosecondsValue = duration.totalNanoseconds<TemporalUnit::Millisecond>();
                 else {
                     ASSERT(unit == TemporalUnit::Microsecond);
-                    value = value + duration[TemporalUnit::Nanosecond] / 1000.0;
+                    totalNanosecondsValue = duration.totalNanoseconds<TemporalUnit::Microsecond>();
                 }
+                ASSERT(totalNanosecondsValue);
+
                 // https://github.com/unicode-org/icu/blob/master/docs/userguide/format_parse/numbers/skeletons.md#fraction-precision
                 skeletonBuilder.append(" ."_s);
 
@@ -397,21 +447,29 @@ static Vector<Element> collectElements(JSGlobalObject* globalObject, const IntlD
             }
             break;
         }
-        default:
+        default: {
+            skeletonBuilder.append("rounding-mode-half-up"_s);
             break;
         }
+        }
+
+        auto style = unitData.style();
 
         // 3.k. If style is "2-digit", then
         //     i. Perform ! CreateDataPropertyOrThrow(nfOpts, "minimumIntegerDigits", 2F).
-        skeletonBuilder.append(" integer-width/"_s, WTF::ICU::majorVersion() >= 67 ? '*' : '+'); // Prior to ICU 67, use the symbol + instead of *.
-        if (unitData.style() == IntlDurationFormat::UnitStyle::TwoDigit)
+        skeletonBuilder.append(" integer-width/*"_s);
+        if (style == IntlDurationFormat::UnitStyle::TwoDigit)
             skeletonBuilder.append("00"_s);
         else
             skeletonBuilder.append('0');
 
+        // 9. Perform ! CreateDataPropertyOrThrow(nfOpts, "useGrouping", false).
+        if (style == IntlDurationFormat::UnitStyle::TwoDigit || style ==  IntlDurationFormat::UnitStyle::Numeric)
+            skeletonBuilder.append(" group-off"_s);
+
         // 3.l. If value is not 0 or display is not "auto", then
         value = purifyNaN(value);
-        if (value != 0 || unitData.display() != IntlDurationFormat::Display::Auto) {
+        if (value || unitData.display() != IntlDurationFormat::Display::Auto || style == IntlDurationFormat::UnitStyle::TwoDigit || style ==  IntlDurationFormat::UnitStyle::Numeric) {
             auto formatToString = [&](UFormattedNumber* formattedNumber) -> String {
                 auto scope = DECLARE_THROW_SCOPE(vm);
 
@@ -429,14 +487,17 @@ static Vector<Element> collectElements(JSGlobalObject* globalObject, const IntlD
             // https://github.com/unicode-org/icu/blob/main/docs/userguide/format_parse/numbers/skeletons.md#sign-display
             if (needsSignDisplay)
                 skeletonBuilder.append(" +_"_s);
-            else if (!value) {
-                if (!durationSign)
-                    durationSign = getDurationSign(duration);
-                if (durationSign == DurationSignType::Negative) {
-                    value = -0.0;
-                    needsSignDisplay = true;
+
+            auto adjustSignDisplay = [&]() -> void {
+                if (!needsSignDisplay && !value) {
+                    if (!durationSign)
+                        durationSign = getDurationSign(duration);
+                    if (durationSign == DurationSignType::Negative) {
+                        value = -0.0;
+                        needsSignDisplay = true;
+                    }
                 }
-            }
+            };
 
             auto formatDouble = [&](const String& skeleton) -> std::unique_ptr<UFormattedNumber, ICUDeleter<unumf_closeResult>> {
                 auto scope = DECLARE_THROW_SCOPE(vm);
@@ -466,61 +527,103 @@ static Vector<Element> collectElements(JSGlobalObject* globalObject, const IntlD
                 return formattedNumber;
             };
 
-            switch (unitData.style()) {
+            auto formatIntl128AsDecimal = [&](const String& skeleton) -> std::unique_ptr<UFormattedNumber, ICUDeleter<unumf_closeResult>> {
+                ASSERT(totalNanosecondsValue);
+                ASSERT(unit == TemporalUnit::Second || unit == TemporalUnit::Millisecond || unit == TemporalUnit::Microsecond);
+
+                auto scope = DECLARE_THROW_SCOPE(vm);
+
+                dataLogLnIf(IntlDurationFormatInternal::verbose, skeleton);
+                StringView skeletonView(skeleton);
+                auto upconverted = skeletonView.upconvertedCharacters();
+
+                UErrorCode status = U_ZERO_ERROR;
+                auto numberFormatter = std::unique_ptr<UNumberFormatter, UNumberFormatterDeleter>(unumf_openForSkeletonAndLocale(upconverted.get(), skeletonView.length(), durationFormat->dataLocaleWithExtensions().data(), &status));
+                if (U_FAILURE(status)) {
+                    throwTypeError(globalObject, scope, "Failed to initialize NumberFormat"_s);
+                    return { };
+                }
+
+                auto formattedNumber = std::unique_ptr<UFormattedNumber, ICUDeleter<unumf_closeResult>>(unumf_openResult(&status));
+                if (U_FAILURE(status)) {
+                    throwTypeError(globalObject, scope, "Failed to format a number."_s);
+                    return { };
+                }
+
+                // We need to keep string alive while strSpan is in use.
+                auto string = buildDecimalFormat(unit, totalNanosecondsValue.value());
+                auto strSpan = string.impl()->span8();
+                unumf_formatDecimal(numberFormatter.get(), reinterpret_cast<const char*>(strSpan.data()), strSpan.size(), formattedNumber.get(), &status);
+                if (U_FAILURE(status)) {
+                    throwTypeError(globalObject, scope, "Failed to format a number."_s);
+                    return { };
+                }
+
+                return formattedNumber;
+            };
+
+            switch (style) {
             // 3.l.i. If style is "2-digit" or "numeric", then
             case IntlDurationFormat::UnitStyle::TwoDigit:
             case IntlDurationFormat::UnitStyle::Numeric: {
-                auto formattedNumber = formatDouble(skeletonBuilder.toString());
-                RETURN_IF_EXCEPTION(scope, { });
+                // https://tc39.es/proposal-intl-duration-format/#sec-formatnumericunits
+                ASSERT(unit == TemporalUnit::Hour || unit == TemporalUnit::Minute || unit == TemporalUnit::Second);
 
-                auto formatted = formatToString(formattedNumber.get());
-                RETURN_IF_EXCEPTION(scope, { });
+                double secondsValue = duration[TemporalUnit::Second];
+                if (durationFormat->units()[static_cast<unsigned>(TemporalUnit::Millisecond)].style() == IntlDurationFormat::UnitStyle::Numeric)
+                    secondsValue = secondsValue + duration[TemporalUnit::Millisecond] / 1000.0 + duration[TemporalUnit::Microsecond] / 1000000.0 + duration[TemporalUnit::Nanosecond] / 1000000000.0;
 
-                elements.append({ ElementType::Element, unit, WTFMove(formatted), value, WTFMove(formattedNumber) });
+                bool needsFormatHours = duration[TemporalUnit::Hour] || durationFormat->units()[static_cast<unsigned>(TemporalUnit::Hour)].display() != IntlDurationFormat::Display::Auto;
+                bool needsFormatSeconds = secondsValue || durationFormat->units()[static_cast<unsigned>(TemporalUnit::Second)].display() != IntlDurationFormat::Display::Auto;
+                bool needsFormatMinutes = (needsFormatHours && needsFormatSeconds) || duration[TemporalUnit::Minute] || durationFormat->units()[static_cast<unsigned>(TemporalUnit::Minute)].display() != IntlDurationFormat::Display::Auto;
 
-                if (unit == TemporalUnit::Hour || unit == TemporalUnit::Minute) {
-                    IntlDurationFormat::UnitData nextUnit;
-                    double nextValue = 0;
-                    if (unit == TemporalUnit::Hour) {
-                        nextUnit = durationFormat->units()[static_cast<unsigned>(TemporalUnit::Minute)];
-                        nextValue = duration[TemporalUnit::Minute];
-                    } else {
-                        nextUnit = durationFormat->units()[static_cast<unsigned>(TemporalUnit::Second)];
-                        nextValue = duration[TemporalUnit::Second];
-                        if (durationFormat->units()[static_cast<unsigned>(TemporalUnit::Millisecond)].style() == IntlDurationFormat::UnitStyle::Numeric)
-                            nextValue = nextValue + duration[TemporalUnit::Millisecond] / 1000.0 + duration[TemporalUnit::Microsecond] / 1000000.0 + duration[TemporalUnit::Nanosecond] / 1000000000.0;
-                    }
+                bool needsFormat = (unit == TemporalUnit::Hour && needsFormatHours) || (unit == TemporalUnit::Minute && needsFormatMinutes) || (unit == TemporalUnit::Second && needsFormatSeconds);
+                bool needsSeparator = (unit == TemporalUnit::Hour && needsFormatHours && needsFormatMinutes) || (unit == TemporalUnit::Minute && needsFormatSeconds);
 
-                    if (nextValue != 0 || nextUnit.display() != IntlDurationFormat::Display::Auto) {
-                        if (separator.isNull())
-                            separator = retrieveSeparator(durationFormat->dataLocaleWithExtensions(), durationFormat->numberingSystem());
-                        elements.append({ ElementType::Literal, unit, separator, value, nullptr });
-                    }
+                if (needsFormat) {
+                    adjustSignDisplay();
+
+                    auto formattedNumber = totalNanosecondsValue ? formatIntl128AsDecimal(skeletonBuilder.toString()) : formatDouble(skeletonBuilder.toString());
+                    RETURN_IF_EXCEPTION(scope, { });
+
+                    auto formatted = formatToString(formattedNumber.get());
+                    RETURN_IF_EXCEPTION(scope, { });
+
+                    elements.append({ ElementType::Element, std::signbit(value), unit, WTFMove(formatted), WTFMove(formattedNumber) });
                 }
+
+                if (needsSeparator) {
+                    if (separator.isNull())
+                        separator = retrieveSeparator(durationFormat->dataLocaleWithExtensions(), durationFormat->numberingSystem());
+                    elements.append({ ElementType::Literal, std::signbit(value), unit, separator, nullptr });
+                }
+
                 break;
             }
             // 3.l.ii. Else
             case IntlDurationFormat::UnitStyle::Long:
             case IntlDurationFormat::UnitStyle::Short:
             case IntlDurationFormat::UnitStyle::Narrow: {
+                adjustSignDisplay();
+
                 skeletonBuilder.append(" measure-unit/duration-"_s);
                 skeletonBuilder.append(String(temporalUnitSingularPropertyName(vm, unit).uid()));
-                if (unitData.style() == IntlDurationFormat::UnitStyle::Long)
+                if (style == IntlDurationFormat::UnitStyle::Long)
                     skeletonBuilder.append(" unit-width-full-name"_s);
-                else if (unitData.style() == IntlDurationFormat::UnitStyle::Short)
+                else if (style == IntlDurationFormat::UnitStyle::Short)
                     skeletonBuilder.append(" unit-width-short"_s);
                 else {
-                    ASSERT(unitData.style() == IntlDurationFormat::UnitStyle::Narrow);
+                    ASSERT(style == IntlDurationFormat::UnitStyle::Narrow);
                     skeletonBuilder.append(" unit-width-narrow"_s);
                 }
 
-                auto formattedNumber = formatDouble(skeletonBuilder.toString());
+                auto formattedNumber = totalNanosecondsValue ? formatIntl128AsDecimal(skeletonBuilder.toString()) : formatDouble(skeletonBuilder.toString());
                 RETURN_IF_EXCEPTION(scope, { });
 
                 auto formatted = formatToString(formattedNumber.get());
                 RETURN_IF_EXCEPTION(scope, { });
 
-                elements.append({ ElementType::Element, unit, WTFMove(formatted), value, WTFMove(formattedNumber) });
+                elements.append({ ElementType::Element, std::signbit(value), unit, WTFMove(formatted), WTFMove(formattedNumber) });
                 break;
             }
             }
@@ -532,15 +635,12 @@ static Vector<Element> collectElements(JSGlobalObject* globalObject, const IntlD
     return elements;
 }
 
-#endif
-
 // https://tc39.es/proposal-intl-duration-format/#sec-Intl.DurationFormat.prototype.format
 JSValue IntlDurationFormat::format(JSGlobalObject* globalObject, ISO8601::Duration duration) const
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
-#if HAVE(ICU_U_LIST_FORMATTER)
     auto elements = collectElements(globalObject, this, WTFMove(duration));
     RETURN_IF_EXCEPTION(scope, { });
 
@@ -576,10 +676,6 @@ JSValue IntlDurationFormat::format(JSGlobalObject* globalObject, ISO8601::Durati
         return throwTypeError(globalObject, scope, "failed to format list of strings"_s);
 
     return jsString(vm, String(WTFMove(result)));
-#else
-    UNUSED_PARAM(duration);
-    return throwTypeError(globalObject, scope, "failed to format list of strings"_s);
-#endif
 }
 
 // https://tc39.es/proposal-intl-duration-format/#sec-Intl.DurationFormat.prototype.formatToParts
@@ -588,7 +684,6 @@ JSValue IntlDurationFormat::formatToParts(JSGlobalObject* globalObject, ISO8601:
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
-#if HAVE(ICU_U_LIST_FORMATTER)
     auto elements = collectElements(globalObject, this, WTFMove(duration));
     RETURN_IF_EXCEPTION(scope, { });
 
@@ -690,7 +785,7 @@ JSValue IntlDurationFormat::formatToParts(JSGlobalObject* globalObject, ISO8601:
                     }
                     IntlFieldIterator iterator(*fieldItr.get());
                     JSString* type = jsString(vm, String(temporalUnitSingularPropertyName(vm, element.m_unit).uid()));
-                    IntlNumberFormat::formatToPartsInternal(globalObject, IntlNumberFormat::Style::Unit, std::signbit(element.m_value), IntlMathematicalValue::numberTypeFromDouble(element.m_value), element.m_string, iterator, parts, nullptr, type);
+                    IntlNumberFormat::formatToPartsInternal(globalObject, IntlNumberFormat::Style::Unit, element.m_valueSignBit, IntlMathematicalValue::NumberType::Integer, element.m_string, iterator, parts, nullptr, type);
                     RETURN_IF_EXCEPTION(scope, void());
                     break;
                 }
@@ -741,10 +836,6 @@ JSValue IntlDurationFormat::formatToParts(JSGlobalObject* globalObject, ISO8601:
     }
 
     return parts;
-#else
-    UNUSED_PARAM(duration);
-    return throwTypeError(globalObject, scope, "failed to format list of strings"_s);
-#endif
 }
 
 // https://tc39.es/proposal-intl-duration-format/#sec-Intl.DurationFormat.prototype.resolvedOptions
@@ -753,6 +844,7 @@ JSObject* IntlDurationFormat::resolvedOptions(JSGlobalObject* globalObject) cons
     VM& vm = globalObject->vm();
     JSObject* options = constructEmptyObject(globalObject);
     options->putDirect(vm, vm.propertyNames->locale, jsString(vm, m_locale));
+    options->putDirect(vm, vm.propertyNames->numberingSystem, jsString(vm, m_numberingSystem));
     options->putDirect(vm, vm.propertyNames->style, jsNontrivialString(vm, styleString(m_style)));
 
     for (unsigned index = 0; index < numberOfTemporalUnits; ++index) {
@@ -762,8 +854,8 @@ JSObject* IntlDurationFormat::resolvedOptions(JSGlobalObject* globalObject) cons
         options->putDirect(vm, displayName(vm, unit), jsNontrivialString(vm, displayString(unitData.display())));
     }
 
-    options->putDirect(vm, vm.propertyNames->fractionalDigits, m_fractionalDigits == fractionalDigitsUndefinedValue ? jsUndefined() : jsNumber(m_fractionalDigits));
-    options->putDirect(vm, vm.propertyNames->numberingSystem, jsString(vm, m_numberingSystem));
+    if (m_fractionalDigits != fractionalDigitsUndefinedValue)
+        options->putDirect(vm, vm.propertyNames->fractionalDigits, jsNumber(m_fractionalDigits));
     return options;
 }
 
@@ -814,3 +906,5 @@ ASCIILiteral IntlDurationFormat::displayString(Display display)
 }
 
 } // namespace JSC
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
