@@ -11,8 +11,10 @@
 
 #if defined(ANGLE_PLATFORM_ANDROID)
 #    include <android/log.h>
+#    include <dlfcn.h>
 #endif
 #include "ANGLEPerfTestArgs.h"
+#include "common/base/anglebase/trace_event/trace_event.h"
 #include "common/debug.h"
 #include "common/gl_enum_utils.h"
 #include "common/mathutil.h"
@@ -22,10 +24,13 @@
 #include "common/utilities.h"
 #include "test_utils/runner/TestSuite.h"
 #include "third_party/perf/perf_test.h"
-#include "third_party/trace_event/trace_event.h"
 #include "util/shader_utils.h"
 #include "util/test_utils.h"
 
+#if defined(ANGLE_PLATFORM_ANDROID)
+#    include "util/android/AndroidWindow.h"
+#endif
+
 #include <cassert>
 #include <cmath>
 #include <fstream>
@@ -236,18 +241,48 @@ void FinishAndCheckForContextLoss()
     }
 }
 
-int EstimateStepsToRun(double trialTime, int stepsPerformed, int stepAlignment)
+void DumpFpsValues(const char *test, double mean_time)
 {
-    double scale   = gTrialTimeSeconds / trialTime;
-    int stepsToRun = static_cast<int>(static_cast<double>(stepsPerformed) * scale);
-    stepsToRun     = std::max(1, stepsToRun);
-    if (stepAlignment != 1)
-    {
-        stepsToRun = rx::roundUp(stepsToRun, stepAlignment);
-    }
-
-    return stepsToRun;
+#if defined(ANGLE_PLATFORM_ANDROID)
+    std::ofstream fp(AndroidWindow::GetExternalStorageDirectory() + "/traces_fps.txt",
+                     std::ios::app);
+#else
+    std::ofstream fp("traces_fps.txt", std::ios::app);
+#endif
+    double fps_value = 1000 / mean_time;
+    fp << test << " " << fps_value << std::endl;
+    fp.close();
 }
+
+#if defined(ANGLE_PLATFORM_ANDROID)
+constexpr bool kHasATrace = true;
+
+void *gLibAndroid = nullptr;
+bool (*gATraceIsEnabled)(void);
+bool (*gATraceSetCounter)(const char *counterName, int64_t counterValue);
+
+void SetupATrace()
+{
+    if (gLibAndroid == nullptr)
+    {
+        gLibAndroid       = dlopen("libandroid.so", RTLD_NOW | RTLD_LOCAL);
+        gATraceIsEnabled  = (decltype(gATraceIsEnabled))dlsym(gLibAndroid, "ATrace_isEnabled");
+        gATraceSetCounter = (decltype(gATraceSetCounter))dlsym(gLibAndroid, "ATrace_setCounter");
+    }
+}
+
+bool ATraceEnabled()
+{
+    return gATraceIsEnabled();
+}
+#else
+constexpr bool kHasATrace = false;
+void SetupATrace() {}
+bool ATraceEnabled()
+{
+    return false;
+}
+#endif
 }  // anonymous namespace
 
 TraceEvent::TraceEvent(char phaseIn,
@@ -272,11 +307,11 @@ ANGLEPerfTest::ANGLEPerfTest(const std::string &name,
       mGPUTimeNs(0),
       mSkipTest(false),
       mStepsToRun(std::max(gStepsPerTrial, gMaxStepsPerformed)),
-      mTrialTimeLimitSeconds(gTrialTimeSeconds),
       mTrialNumStepsPerformed(0),
       mTotalNumStepsPerformed(0),
       mIterationsPerStep(iterationsPerStep),
-      mRunning(true)
+      mRunning(true),
+      mPerfMonitor(0)
 {
     if (mStory == "")
     {
@@ -292,7 +327,11 @@ ANGLEPerfTest::ANGLEPerfTest(const std::string &name,
     mReporter->RegisterImportantMetric(".gpu_time", units);
     mReporter->RegisterFyiMetric(".trial_steps", "count");
     mReporter->RegisterFyiMetric(".total_steps", "count");
-    mReporter->RegisterFyiMetric(".steps_to_run", "count");
+
+    if (kHasATrace)
+    {
+        SetupATrace();
+    }
 }
 
 ANGLEPerfTest::~ANGLEPerfTest() {}
@@ -312,23 +351,17 @@ void ANGLEPerfTest::run()
         // GTEST_SKIP returns.
     }
 
-    if (mStepsToRun <= 0)
-    {
-        // We don't call finish between calibration steps when calibrating non-Render tests. The
-        // Render tests will have already calibrated when this code is run.
-        calibrateStepsToRun();
-        ASSERT(mStepsToRun > 0);
-    }
-
     uint32_t numTrials = OneFrame() ? 1 : gTestTrials;
     if (gVerboseLogging)
     {
         printf("Test Trials: %d\n", static_cast<int>(numTrials));
     }
 
+    atraceCounter("TraceStage", 3);
+
     for (uint32_t trial = 0; trial < numTrials; ++trial)
     {
-        runTrial(mTrialTimeLimitSeconds, mStepsToRun, RunTrialPolicy::RunContinuously);
+        runTrial(gTrialTimeSeconds, mStepsToRun, RunTrialPolicy::RunContinuously);
         processResults();
         if (gVerboseLogging)
         {
@@ -341,6 +374,8 @@ void ANGLEPerfTest::run()
         }
     }
 
+    atraceCounter("TraceStage", 0);
+
     if (gVerboseLogging && !mTestTrialResults.empty())
     {
         double numResults = static_cast<double>(mTestTrialResults.size());
@@ -365,6 +400,12 @@ void ANGLEPerfTest::run()
         {
             printf("Mean result time: %.4lf ms.\n", mean);
         }
+
+        if (kStandaloneBenchmark)
+        {
+            DumpFpsValues(mStory.c_str(), mean);
+        }
+
         printf("Coefficient of variation: %.2lf%%\n", coefficientOfVariation * 100.0);
     }
 }
@@ -378,10 +419,27 @@ void ANGLEPerfTest::runTrial(double maxRunTime, int maxStepsToRun, RunTrialPolic
     mTrialTimer.start();
     startTest();
 
+    int loopStepsPerformed  = 0;
+    double lastLoopWallTime = 0;
     while (mRunning)
     {
-        // Only stop on aligned steps or in --smoke-test-mode (single step per trial).
-        if (mTrialNumStepsPerformed % stepAlignment == 0 || gStepsPerTrial == 1)
+        // When ATrace enabled, track average frame time before the first frame of each trace loop.
+        if (ATraceEnabled() && stepAlignment > 1 && runPolicy == RunTrialPolicy::RunContinuously &&
+            mTrialNumStepsPerformed % stepAlignment == 0)
+        {
+            double wallTime = mTrialTimer.getElapsedWallClockTime();
+            if (loopStepsPerformed > 0)  // 0 at the first frame of the first loop
+            {
+                int frameTimeAvgUs = int(1e6 * (wallTime - lastLoopWallTime) / loopStepsPerformed);
+                atraceCounter("TraceLoopFrameTimeAvgUs", frameTimeAvgUs);
+                loopStepsPerformed = 0;
+            }
+            lastLoopWallTime = wallTime;
+        }
+
+        // Only stop on aligned steps or in a few special case modes
+        if (mTrialNumStepsPerformed % stepAlignment == 0 || gStepsPerTrial == 1 || gRunToKeyFrame ||
+            gMaxStepsPerformed != kDefaultMaxStepsPerformed)
         {
             if (gMaxStepsPerformed > 0 && mTotalNumStepsPerformed >= gMaxStepsPerformed)
             {
@@ -413,6 +471,15 @@ void ANGLEPerfTest::runTrial(double maxRunTime, int maxStepsToRun, RunTrialPolic
             }
         }
 
+        if (gFpsLimit)
+        {
+            double wantTime    = mTrialNumStepsPerformed / double(gFpsLimit);
+            double currentTime = mTrialTimer.getElapsedWallClockTime();
+            if (currentTime < wantTime)
+            {
+                std::this_thread::sleep_for(std::chrono::duration<double>(wantTime - currentTime));
+            }
+        }
         step();
 
         if (runPolicy == RunTrialPolicy::FinishEveryStep)
@@ -424,6 +491,7 @@ void ANGLEPerfTest::runTrial(double maxRunTime, int maxStepsToRun, RunTrialPolic
         {
             mTrialNumStepsPerformed++;
             mTotalNumStepsPerformed++;
+            loopStepsPerformed++;
         }
 
         if ((mTotalNumStepsPerformed % kNumberOfStepsPerformedToComputeGPUTime) == 0)
@@ -431,15 +499,86 @@ void ANGLEPerfTest::runTrial(double maxRunTime, int maxStepsToRun, RunTrialPolic
             computeGPUTime();
         }
     }
+
+    if (runPolicy == RunTrialPolicy::RunContinuously)
+    {
+        atraceCounter("TraceLoopFrameTimeAvgUs", 0);
+    }
     finishTest();
     mTrialTimer.stop();
     computeGPUTime();
 }
 
-void ANGLEPerfTest::SetUp() {}
+void ANGLEPerfTest::SetUp()
+{
+    if (gWarmup)
+    {
+        atraceCounter("TraceStage", 1);
+
+        // Trace tests run with glFinish for a loop (getStepAlignment == frameCount).
+        int warmupSteps = getStepAlignment();
+        if (gVerboseLogging)
+        {
+            printf("Warmup: %d steps\n", warmupSteps);
+        }
+
+        Timer warmupTimer;
+        warmupTimer.start();
+
+        runTrial(gTrialTimeSeconds, warmupSteps, RunTrialPolicy::FinishEveryStep);
+
+        if (warmupSteps > 1)  // trace tests only: getStepAlignment() is 1 otherwise
+        {
+            atraceCounter("TraceStage", 2);
+
+            // Short traces (e.g. 10 frames) have some spikes after the first loop b/308975999
+            const double kMinWarmupTime = 1.5;
+            double remainingTime        = kMinWarmupTime - warmupTimer.getElapsedWallClockTime();
+            if (remainingTime > 0)
+            {
+                printf("Warmup: Looping for remaining warmup time (%.2f seconds).\n",
+                       remainingTime);
+                runTrial(remainingTime, std::numeric_limits<int>::max(),
+                         RunTrialPolicy::RunContinuouslyWarmup);
+            }
+        }
+
+        if (gVerboseLogging)
+        {
+            printf("Warmup took %.2lf seconds.\n", warmupTimer.getElapsedWallClockTime());
+        }
+    }
+}
 
 void ANGLEPerfTest::TearDown() {}
 
+void ANGLEPerfTest::recordIntegerMetric(const char *metric, size_t value, const std::string &units)
+{
+    // Prints "RESULT ..." to stdout
+    mReporter->AddResult(metric, value);
+
+    // Saves results to file if enabled
+    TestSuite::GetMetricWriter().writeInfo(mName, mBackend, mStory, metric, units);
+    TestSuite::GetMetricWriter().writeIntegerValue(value);
+}
+
+void ANGLEPerfTest::recordDoubleMetric(const char *metric, double value, const std::string &units)
+{
+    // Prints "RESULT ..." to stdout
+    mReporter->AddResult(metric, value);
+
+    // Saves results to file if enabled
+    TestSuite::GetMetricWriter().writeInfo(mName, mBackend, mStory, metric, units);
+    TestSuite::GetMetricWriter().writeDoubleValue(value);
+}
+
+void ANGLEPerfTest::addHistogramSample(const char *metric, double value, const std::string &units)
+{
+    std::string measurement = mName + mBackend + metric;
+    // Output histogram JSON set format if enabled.
+    TestSuite::GetInstance()->addHistogramSample(measurement, mStory, value, units);
+}
+
 void ANGLEPerfTest::processResults()
 {
     processClockResult(".cpu_time", mTrialTimer.getElapsedCpuTime());
@@ -457,15 +596,8 @@ void ANGLEPerfTest::processResults()
         printf("Ran %0.2lf iterations per second\n", fps);
     }
 
-    if (gCalibration)
-    {
-        mReporter->AddResult(".steps_to_run", static_cast<size_t>(mStepsToRun));
-    }
-    else
-    {
-        mReporter->AddResult(".trial_steps", static_cast<size_t>(mTrialNumStepsPerformed));
-        mReporter->AddResult(".total_steps", static_cast<size_t>(mTotalNumStepsPerformed));
-    }
+    mReporter->AddResult(".trial_steps", static_cast<size_t>(mTrialNumStepsPerformed));
+    mReporter->AddResult(".total_steps", static_cast<size_t>(mTotalNumStepsPerformed));
 
     if (!mProcessMemoryUsageKBSamples.empty())
     {
@@ -489,32 +621,22 @@ void ANGLEPerfTest::processResults()
 
         // Median
         {
-            size_t midpoint = samples.size() >> 1;
+            size_t midpoint = samples.size() / 2;
             std::nth_element(samples.begin(), samples.begin() + midpoint, samples.end());
 
-            std::stringstream medianStr;
-            medianStr << "." << counterName << "_median";
-            std::string medianName = medianStr.str();
-
-            mReporter->AddResult(medianName, static_cast<size_t>(samples[midpoint]));
-
-            std::string measurement = mName + mBackend + "." + counterName + "_median";
-            TestSuite::GetInstance()->addHistogramSample(
-                measurement, mStory, static_cast<double>(samples[midpoint]), "count");
+            std::string medianName = "." + counterName + "_median";
+            recordIntegerMetric(medianName.c_str(), static_cast<size_t>(samples[midpoint]),
+                                "count");
+            addHistogramSample(medianName.c_str(), static_cast<double>(samples[midpoint]), "count");
         }
 
         // Maximum
         {
             const auto &maxIt = std::max_element(samples.begin(), samples.end());
 
-            std::stringstream maxStr;
-            maxStr << "." << counterName << "_max";
-            std::string maxName = maxStr.str();
-            mReporter->AddResult(maxName, static_cast<size_t>(*maxIt));
-
-            std::string measurement = mName + mBackend + "." + counterName + "_max";
-            TestSuite::GetInstance()->addHistogramSample(measurement, mStory,
-                                                         static_cast<double>(*maxIt), "count");
+            std::string maxName = "." + counterName + "_max";
+            recordIntegerMetric(maxName.c_str(), static_cast<size_t>(*maxIt), "count");
+            addHistogramSample(maxName.c_str(), static_cast<double>(*maxIt), "count");
         }
 
         // Sum
@@ -522,14 +644,9 @@ void ANGLEPerfTest::processResults()
             GLuint64 sum =
                 std::accumulate(samples.begin(), samples.end(), static_cast<GLuint64>(0));
 
-            std::stringstream sumStr;
-            sumStr << "." << counterName << "_sum";
-            std::string sumName = sumStr.str();
-            mReporter->AddResult(sumName, static_cast<size_t>(sum));
-
-            std::string measurement = mName + mBackend + "." + counterName + "_sum";
-            TestSuite::GetInstance()->addHistogramSample(measurement, mStory,
-                                                         static_cast<double>(sum), "count");
+            std::string sumName = "." + counterName + "_max";
+            recordIntegerMetric(sumName.c_str(), static_cast<size_t>(sum), "count");
+            addHistogramSample(sumName.c_str(), static_cast<double>(sum), "count");
         }
     }
 }
@@ -563,12 +680,9 @@ void ANGLEPerfTest::processClockResult(const char *metric, double resultSeconds)
     {
         result = secondsPerIteration * kNanoSecondsPerSecond;
     }
-    mReporter->AddResult(metric, result);
-
-    // Output histogram JSON set format if enabled.
-    TestSuite::GetInstance()->addHistogramSample(mName + mBackend + metric, mStory,
-                                                 secondsPerIteration * kMilliSecondsPerSecond,
-                                                 "msBestFitFormat_smallerIsBetter");
+    recordDoubleMetric(metric, result, units);
+    addHistogramSample(metric, secondsPerIteration * kMilliSecondsPerSecond,
+                       "msBestFitFormat_smallerIsBetter");
 }
 
 void ANGLEPerfTest::processMemoryResult(const char *metric, uint64_t resultKB)
@@ -579,11 +693,9 @@ void ANGLEPerfTest::processMemoryResult(const char *metric, uint64_t resultKB)
         mReporter->RegisterImportantMetric(metric, "sizeInBytes");
     }
 
-    mReporter->AddResult(metric, static_cast<size_t>(resultKB * 1000));
-
-    TestSuite::GetInstance()->addHistogramSample(mName + mBackend + metric, mStory,
-                                                 static_cast<double>(resultKB) * 1000.0,
-                                                 "sizeInBytes_smallerIsBetter");
+    recordIntegerMetric(metric, static_cast<size_t>(resultKB * 1000), "sizeInBytes");
+    addHistogramSample(metric, static_cast<double>(resultKB) * 1000.0,
+                       "sizeInBytes_smallerIsBetter");
 }
 
 double ANGLEPerfTest::normalizedTime(size_t value) const
@@ -591,82 +703,22 @@ double ANGLEPerfTest::normalizedTime(size_t value) const
     return static_cast<double>(value) / static_cast<double>(mTrialNumStepsPerformed);
 }
 
-void ANGLEPerfTest::calibrateStepsToRun()
-{
-    ASSERT(mTrialTimeLimitSeconds > 0);
-    ASSERT(gWarmupTrials > 0);
-
-    Timer calibrationTimer;
-    calibrationTimer.start();
-
-    if (isRenderTest())
-    {
-        // To compute an accurate value for "mStepsToRun" we do a two-pass calibration. The
-        // first pass runs for "gCalibrationTimeSeconds" and calls glFinish every step. The
-        // estimated steps to run using this method is very inaccurate but is guaranteed to
-        // complete in a fixed amount of time. Using that estimate we then run a second pass
-        // and call Finish after running the estimated number of steps. We can then iterate
-        // on the number of steps to improve on our estimate for "mStepsToRun".
-
-        // Run initially for "gMaxTrialTime" using the run trial policy.
-        runTrial(mTrialTimeLimitSeconds, std::numeric_limits<int>::max(),
-                 RunTrialPolicy::FinishEveryStep);
-
-        if (gVerboseLogging)
-        {
-            printf("Calibration trial 0: %d steps in %.2lf seconds.\n", mTrialNumStepsPerformed,
-                   mTrialTimer.getElapsedWallClockTime());
-        }
-
-        for (int trial = 1; trial < gWarmupTrials; ++trial)
-        {
-            // Compute an intermediate steps to run for the next warm-up trial.
-            int stepsToRun = EstimateStepsToRun(mTrialTimer.getElapsedWallClockTime(),
-                                                mTrialNumStepsPerformed, getStepAlignment());
-            runTrial(mTrialTimeLimitSeconds, stepsToRun, RunTrialPolicy::RunContinuously);
-
-            if (gVerboseLogging)
-            {
-                printf("Calibration trial %d: %d steps in %.2lf seconds.\n", trial,
-                       mTrialNumStepsPerformed, mTrialTimer.getElapsedWallClockTime());
-            }
-        }
-    }
-    else
-    {
-        while (calibrationTimer.getElapsedWallClockTime() + mTrialTimeLimitSeconds <
-               gCalibrationTimeSeconds)
-        {
-            runTrial(gCalibrationTimeSeconds, std::numeric_limits<int>::max(),
-                     RunTrialPolicy::RunContinuously);
-        }
-    }
-
-    // Compute a final steps to run with the data from the final warm-up trial.
-    mStepsToRun = EstimateStepsToRun(mTrialTimer.getElapsedWallClockTime(), mTrialNumStepsPerformed,
-                                     getStepAlignment());
-
-    if (gVerboseLogging)
-    {
-        printf("Running %d steps after calibration.\n", mStepsToRun);
-    }
-
-    // Calibration allows the perf test runner script to save some time.
-    if (gCalibration)
-    {
-        processResults();
-    }
-
-    // Increase the time limit slightly to ensure we run the calibrated step count.
-    mTrialTimeLimitSeconds++;
-}
-
 int ANGLEPerfTest::getStepAlignment() const
 {
     // Default: No special alignment rules.
     return 1;
 }
 
+void ANGLEPerfTest::atraceCounter(const char *counterName, int64_t counterValue)
+{
+#if defined(ANGLE_PLATFORM_ANDROID)
+    if (ATraceEnabled())
+    {
+        gATraceSetCounter(counterName, counterValue);
+    }
+#endif
+}
+
 RenderTestParams::RenderTestParams()
 {
 #if defined(ANGLE_DEBUG_LAYERS_ENABLED)
@@ -684,6 +736,9 @@ std::string RenderTestParams::backend() const
     {
         case GLESDriverType::AngleEGL:
             break;
+        case GLESDriverType::AngleVulkanSecondariesEGL:
+            strstr << "_vulkan_secondaries";
+            break;
         case GLESDriverType::SystemWGL:
         case GLESDriverType::SystemEGL:
             strstr << "_native";
@@ -796,14 +851,17 @@ ANGLERenderTest::ANGLERenderTest(const std::string &name,
     switch (testParams.driver)
     {
         case GLESDriverType::AngleEGL:
-            mGLWindow = EGLWindow::New(testParams.clientType, testParams.majorVersion,
-                                       testParams.minorVersion, testParams.profileMask);
+            mGLWindow = EGLWindow::New(testParams.majorVersion, testParams.minorVersion);
             mEntryPointsLib.reset(OpenSharedLibrary(ANGLE_EGL_LIBRARY_NAME, SearchType::ModuleDir));
             break;
+        case GLESDriverType::AngleVulkanSecondariesEGL:
+            mGLWindow = EGLWindow::New(testParams.majorVersion, testParams.minorVersion);
+            mEntryPointsLib.reset(OpenSharedLibrary(ANGLE_VULKAN_SECONDARIES_EGL_LIBRARY_NAME,
+                                                    SearchType::ModuleDir));
+            break;
         case GLESDriverType::SystemEGL:
 #if defined(ANGLE_USE_UTIL_LOADER) && !defined(ANGLE_PLATFORM_WINDOWS)
-            mGLWindow = EGLWindow::New(testParams.clientType, testParams.majorVersion,
-                                       testParams.minorVersion, testParams.profileMask);
+            mGLWindow = EGLWindow::New(testParams.majorVersion, testParams.minorVersion);
             mEntryPointsLib.reset(OpenSharedLibraryWithExtension(
                 GetNativeEGLLibraryNameWithExtension(), SearchType::SystemDir));
 #else
@@ -812,16 +870,14 @@ ANGLERenderTest::ANGLERenderTest(const std::string &name,
             break;
         case GLESDriverType::SystemWGL:
 #if defined(ANGLE_USE_UTIL_LOADER) && defined(ANGLE_PLATFORM_WINDOWS)
-            mGLWindow = WGLWindow::New(testParams.clientType, testParams.majorVersion,
-                                       testParams.minorVersion, testParams.profileMask);
+            mGLWindow = WGLWindow::New(testParams.majorVersion, testParams.minorVersion);
             mEntryPointsLib.reset(OpenSharedLibrary("opengl32", SearchType::SystemDir));
 #else
             skipTest("WGL driver not available.");
 #endif  // defined(ANGLE_USE_UTIL_LOADER) && defined(ANGLE_PLATFORM_WINDOWS)
             break;
         case GLESDriverType::ZinkEGL:
-            mGLWindow = EGLWindow::New(testParams.clientType, testParams.majorVersion,
-                                       testParams.minorVersion, testParams.profileMask);
+            mGLWindow = EGLWindow::New(testParams.majorVersion, testParams.minorVersion);
             mEntryPointsLib.reset(
                 OpenSharedLibrary(ANGLE_MESA_EGL_LIBRARY_NAME, SearchType::ModuleDir));
             break;
@@ -854,8 +910,6 @@ void ANGLERenderTest::SetUp()
         return;
     }
 
-    ANGLEPerfTest::SetUp();
-
     // Set a consistent CPU core affinity and high priority.
     StabilizeCPUForBenchmarking();
 
@@ -1008,27 +1062,8 @@ void ANGLERenderTest::SetUp()
 
     mTestTrialResults.reserve(gTestTrials);
 
-    if (mStepsToRun <= 0)
-    {
-        calibrateStepsToRun();
-    }
-    else
-    {
-        if (gVerboseLogging)
-        {
-            printf("Warmup: %d trials, %d steps per trial\n", gWarmupTrials, mStepsToRun);
-        }
-
-        for (int warmupTrial = 0; warmupTrial < gWarmupTrials; ++warmupTrial)
-        {
-            runTrial(mTrialTimeLimitSeconds, mStepsToRun, RunTrialPolicy::RunContinuously);
-            if (gVerboseLogging)
-            {
-                printf("Warm-up trial took %.2lf seconds.\n",
-                       mTrialTimer.getElapsedWallClockTime());
-            }
-        }
-    }
+    // Runs warmup if enabled
+    ANGLEPerfTest::SetUp();
 
     initPerfCounters();
 }
@@ -1037,6 +1072,12 @@ void ANGLERenderTest::TearDown()
 {
     ASSERT(mTimestampQueries.empty());
 
+    if (!mPerfCounterInfo.empty())
+    {
+        glDeletePerfMonitorsAMD(1, &mPerfMonitor);
+        mPerfMonitor = 0;
+    }
+
     if (!mSkipTest)
     {
         destroyBenchmark();
@@ -1124,6 +1165,13 @@ void ANGLERenderTest::initPerfCounters()
             fprintf(stderr, "'%s' does not match any available perf counters.\n", counter.c_str());
         }
     }
+
+    if (!mPerfCounterInfo.empty())
+    {
+        glGenPerfMonitorsAMD(1, &mPerfMonitor);
+        // Note: technically, glSelectPerfMonitorCountersAMD should be used to select the counters,
+        // but currently ANGLE always captures all counters.
+    }
 }
 
 void ANGLERenderTest::updatePerfCounters()
@@ -1289,7 +1337,13 @@ void ANGLERenderTest::computeGPUTime()
     }
 }
 
-void ANGLERenderTest::startTest() {}
+void ANGLERenderTest::startTest()
+{
+    if (!mPerfCounterInfo.empty())
+    {
+        glBeginPerfMonitorAMD(mPerfMonitor);
+    }
+}
 
 void ANGLERenderTest::finishTest()
 {
@@ -1298,6 +1352,11 @@ void ANGLERenderTest::finishTest()
     {
         FinishAndCheckForContextLoss();
     }
+
+    if (!mPerfCounterInfo.empty())
+    {
+        glEndPerfMonitorAMD(mPerfMonitor);
+    }
 }
 
 bool ANGLERenderTest::popEvent(Event *event)
