@@ -21,6 +21,7 @@
 #include "include/private/base/SkTArray.h"
 #include "include/private/base/SkTPin.h"
 #include "include/private/base/SkTo.h"
+#include "modules/skcms/skcms.h"
 #include "src/base/SkArenaAlloc.h"
 #include "src/base/SkFloatBits.h"
 #include "src/base/SkVx.h"
@@ -201,9 +202,10 @@ SkGradientBaseShader::SkGradientBaseShader(const Descriptor& desc, const SkMatri
             fColorCount = 4
      */
     fColorCount = desc.fColorCount;
-    // check if we need to add in start and/or end position/colors
+
+    // Check if we need to add in start and/or end position/colors
     if (desc.fPositions) {
-        fFirstStopIsImplicit = desc.fPositions[0] != 0;
+        fFirstStopIsImplicit = desc.fPositions[0] > 0;
         fLastStopIsImplicit = desc.fPositions[desc.fColorCount - 1] != SK_Scalar1;
         fColorCount += fFirstStopIsImplicit + fLastStopIsImplicit;
     }
@@ -239,15 +241,62 @@ SkGradientBaseShader::SkGradientBaseShader(const Descriptor& desc, const SkMatri
         const SkScalar uniformStep = desc.fPositions[startIndex] - prev;
         for (int i = startIndex; i < count; i++) {
             // Pin the last value to 1.0, and make sure pos is monotonic.
-            auto curr = (i == desc.fColorCount) ? 1 : SkTPin(desc.fPositions[i], prev, 1.0f);
+            float curr = 1.0f;
+            if (i != desc.fColorCount) {
+                curr = SkTPin(desc.fPositions[i], prev, 1.0f);
+
+                // If a value is clamped to 1.0 before the last stop, the last stop
+                // actually isn't implicit if we thought it was.
+                if (curr == 1.0f && fLastStopIsImplicit) {
+                    fLastStopIsImplicit = false;
+                }
+            }
+
             uniformStops &= SkScalarNearlyEqual(uniformStep, curr - prev);
 
             *positions++ = prev = curr;
         }
 
-        // If the stops are uniform, treat them as implicit.
         if (uniformStops) {
+            // If the stops are uniform, treat them as implicit.
             fPositions = nullptr;
+        } else {
+            // Remove duplicate stops with more than two of the same stop,
+            // keeping the leftmost and rightmost stop colors.
+            // i.e.       0, 0, 0,   0.2, 0.2, 0.3, 0.3, 0.3, 1, 1
+            // w/  clamp  0,    0,   0.2, 0.2, 0.3,      0.3, 1, 1
+            // w/o clamp        0,   0.2, 0.2, 0.3,      0.3, 1
+            int i = 0;
+            int dedupedColorCount = 0;
+            for (int j = 1; j <= fColorCount; j++) {
+                // We can compare the current positions at i and j since once these fPosition
+                // values are overwritten, our i and j pointers will be past the overwritten values.
+                if (j == fColorCount || fPositions[i] != fPositions[j]) {
+                    bool dupStop = j - i > 1;
+
+                    // Ignore the leftmost stop (i) if it is a non-clamp tilemode with
+                    // a duplicate stop on t = 0.
+                    bool ignoreLeftmost = dupStop && fTileMode != SkTileMode::kClamp
+                                                    && fPositions[i] == 0;
+                    if (!ignoreLeftmost) {
+                        fPositions[dedupedColorCount] = fPositions[i];
+                        fColors[dedupedColorCount] =  fColors[i];
+                        dedupedColorCount++;
+                    }
+
+                    // Include the rightmost stop (j-1) only if the stop has a duplicate,
+                    // ignoring the rightmost stop if it is a non-clamp tilemode with t = 1.
+                    bool ignoreRightmost = fTileMode != SkTileMode::kClamp
+                                                    && fPositions[j - 1] == 1;
+                    if (dupStop && !ignoreRightmost) {
+                        fPositions[dedupedColorCount] = fPositions[j - 1];
+                        fColors[dedupedColorCount] = fColors[j - 1];
+                        dedupedColorCount++;
+                    }
+                    i = j;
+                }
+            }
+            fColorCount = dedupedColorCount;
         }
     }
 }
@@ -593,6 +642,21 @@ static sk_sp<SkColorSpace> intermediate_color_space(SkGradientShader::Interpolat
             // part of the conversion is a matrix multiply, which could be absorbed into the
             // color space xform.
             return SkColorSpace::MakeSRGBLinear();
+
+        // These rectangular color spaces have their own transfer curves.
+        case ColorSpace::kDisplayP3:
+            return SkColorSpace::MakeRGB(SkNamedTransferFn::kSRGB, SkNamedGamut::kDisplayP3);
+
+        case ColorSpace::kRec2020:
+            return SkColorSpace::MakeRGB(SkNamedTransferFn::kRec2020, SkNamedGamut::kRec2020);
+
+        case ColorSpace::kProphotoRGB:
+            static skcms_Matrix3x3 lin_proPhoto_to_XYZ_D50;
+            SkNamedPrimaries::kProPhotoRGB.toXYZD50(&lin_proPhoto_to_XYZ_D50);
+            return SkColorSpace::MakeRGB(SkNamedTransferFn::kProPhotoRGB, lin_proPhoto_to_XYZ_D50);
+
+        case ColorSpace::kA98RGB:
+            return SkColorSpace::MakeRGB(SkNamedTransferFn::kA98RGB, SkNamedGamut::kAdobeRGB);
     }
     SkUNREACHABLE;
 }
