@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2018 Apple Inc. All rights reserved.
+ *  Copyright (C) 2005-2024 Apple Inc. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -23,21 +23,64 @@
 #include <new>
 #include <stdlib.h>
 #include <wtf/DebugHeap.h>
+#include <wtf/MallocCommon.h>
 #include <wtf/StdLibExtras.h>
 
+#if !USE(SYSTEM_MALLOC)
+#include <bmalloc/BPlatform.h>
+// Enable USE(LIBPAS)
+// FIXME: Replaces uses of `#if !USE(SYSTEM_MALLOC) \n #if BUSE(LIBPAS)` with `#if USE(LIBPAS)`
+#if BUSE(LIBPAS)
+#define USE_LIBPAS 1
+#endif
+#endif
+
 namespace WTF {
 
 // There are several malloc-related macros to annotate class / struct. If these annotations are attached,
-// allocation is handled by bmalloc if bmalloc is available.
+// allocation is handled by libpas or bmalloc, depending on which if any of these memory allocators is available.
 //
 // TLDR; Here is a quick guidance.
+//   1. If your class / struct /template is derived from a base class which uses:
+//          WTF_MAKE_TZONE_ALLOCATED
+//          WTF_MAKE_TZONE_ALLOCATED_EXPORT
+//          WTF_MAKE_STRUCT_TZONE_ALLOCATED
+//          WTF_MAKE_TZONE_ALLOCATED_TEMPLATE
+//          WTF_MAKE_TZONE_ALLOCATED_TEMPLATE_EXPORT
+//          WTF_MAKE_COMPACT_TZONE_ALLOCATED
+//          WTF_MAKE_COMPACT_TZONE_ALLOCATED_EXPORT
+//      then you must use the same TZONE_ALLOCATED annotation as well. If your class is a template, use one of
+//      the template forms (WTF_MAKE_TZONE_ALLOCATED_TEMPLATE or WTF_MAKE_TZONE_ALLOCATED_TEMPLATE_EXPORT).
+//      If your class is not a template, use one of the non-template forms.
 //
-//   1. If your class / struct is derived from a base class which uses WTF_MAKE_ISO_ALLOCATED / WTF_MAKE_ISO_ALLOCATED_EXPORT,
-//      you must use WTF_MAKE_ISO_ALLOCATED / WTF_MAKE_ISO_ALLOCATED_EXPORT.
-//   2. If your class / struct is a DOM object, use WTF_MAKE_ISO_ALLOCATED.
-//   3. If your class / struct is particularly memory consuming and if you think tracking footprint of your class is helpful for memory-reduction work,
-//      use WTF_MAKE_FAST_ALLOCATED_WITH_HEAP_IDENTIFIER / WTF_MAKE_FAST_ALLOCATED_WITH_HEAP_IDENTIFIER.
-//   4. Otherwise, use WTF_MAKE_FAST_ALLOCATED / WTF_MAKE_STRUCT_FAST_ALLOCATED.
+//   2. If your class / struct is derived from a base class which uses:
+//          WTF_MAKE_TZONE_OR_ISO_ALLOCATED
+//          WTF_MAKE_TZONE_OR_ISO_ALLOCATED_EXPORT
+//          WTF_MAKE_TZONE_OR_ISO_ALLOCATED_TEMPLATE
+//          WTF_MAKE_COMPACT_TZONE_OR_ISO_ALLOCATED
+//          WTF_MAKE_COMPACT_TZONE_OR_ISO_ALLOCATED_EXPORT
+//      then you must use the same TZONE_OR_ISO_ALLOCATED annotation as well. If your class is a template, use the
+//      template forms (WTF_MAKE_TZONE_OR_ISO_ALLOCATED_TEMPLATE). If your class is not a template, use one of the
+//      non-template forms.
+//
+//   3. If your class / struct / template is a DOM object, use a TZONE_OR_ISO_ALLOCATED annotation.
+//
+//   4. If your class / struct is particularly memory consuming and if you think tracking footprint of your class
+//      is helpful for memory-reduction work, use WTF_MAKE_FAST_ALLOCATED_WITH_HEAP_IDENTIFIER /
+//      WTF_MAKE_FAST_ALLOCATED_WITH_HEAP_IDENTIFIER.
+//
+//   5. For classes / structs that are fixed sized, use a TZONE_ALLOCATED annotation
+//
+//   6. Otherwise, use WTF_MAKE_FAST_ALLOCATED / WTF_MAKE_STRUCT_FAST_ALLOCATED.
+//
+//   7. If your template uses a WTF_MAKE_TZONE_ALLOCATED_TEMPLATE or WTF_MAKE_TZONE_ALLOCATED_TEMPLATE_EXPORT
+//      annotation, then you must use WTF_MAKE_TZONE_ALLOCATED_TEMPLATE_IMPL or
+//      WTF_MAKE_TZONE_ALLOCATED_TEMPLATE_IMPL_WITH_MULTIPLE_OR_SPECIALIZED_PARAMETERS after the template definition
+//      (in the header file as well if that's there the template is defined).
+//
+//   8. If your template uses a WTF_MAKE_COMPACT_TZONE_OR_ISO_ALLOCATED or WTF_MAKE_COMPACT_TZONE_OR_ISO_ALLOCATED_EXPORT
+//      annotation, then you must use WTF_MAKE_TZONE_OR_ISO_ALLOCATED_TEMPLATE_IMPL after the template definition
+//      (in the header file as well if that's there the template is defined).
 //
 // Let's explain the differences in detail.
 //
@@ -59,8 +102,41 @@ namespace WTF {
 //     To enable MallocHeapBreakdown, set ENABLE_MALLOC_HEAP_BREAKDOWN in WTF and BENABLE_MALLOC_HEAP_BREAKDOWN in bmalloc.
 //     For the details of MallocHeapBreakdown, please look at ChangeLog of https://trac.webkit.org/changeset/253987/webkit.
 //
+// - WTF_MAKE_TZONE_ALLOCATED(ClassName)
+// - WTF_MAKE_TZONE_ALLOCATED_EXPORT(ClassName, exportMacro)
+// - WTF_MAKE_STRUCT_TZONE_ALLOCATED(ClassName)
+// - WTF_MAKE_TZONE_ALLOCATED_TEMPLATE(ClassName)
+// - WTF_MAKE_TZONE_ALLOCATED_TEMPLATE_EXPORT(ClassName, exportMacro)
+// - WTF_MAKE_COMPACT_TZONE_ALLOCATED(ClassName)
+// - WTF_MAKE_COMPACT_TZONE_ALLOCATED_EXPORT(ClassName, exportMacro)
+//     class / struct / template is allocated from a fixed set of shared libpas heaps with the same size and alignment. Each set of shared heaps is
+//     known as a TZoneTypeBuckets. The particular bucket is selected using SHA 256 hashing on a process startup value along with particulars of the
+//     class / struct being allocated. This sharing provides protections from type confusion and use-after-free bugs, but with less memory overhead of
+//     IsoHeap allocated objects. This is the preferred allocation method.
+//     For example, if Event is annotated with WTF_MAKE_TZONE_ALLOCATED(Event), all derived classes of Event must be annotated with
+//     WTF_MAKE_TZONE_ALLOCATED(XXX). When you annotate a class with WTF_MAKE_TZONE_ALLOCATED(XXX), you need to add WTF_MAKE_TZONE_ALLOCATED_IMPL(XXX)
+//     in cpp file side.
+//     Because WTF_MAKE_TZONE_ALLOCATED_IMPL defines functions on the cpp side, you sometimes need to annotate these functions with export macros when your
+//     class is used outside of the component defining your class (e.g. your class is in WebCore and it is also used in WebKit). In this case,
+//     you can use WTF_MAKE_TZONE_ALLOCATED_EXPORT(XXX). to annotate these functions with appropriate export macros:
+//        e.g. WTF_MAKE_TZONE_ALLOCATED_EXPORT(Special, JS_EXPORT_PRIVATE)).
+//
+//     WTF_MAKE_STRUCT_TZONE_ALLOCATED is the same as WTF_MAKE_TZONE_ALLOCATED, but for structs.
+//
+//     WTF_MAKE_TZONE_ALLOCATED_TEMPLATE and WTF_MAKE_TZONE_ALLOCATED_TEMPLATE_EXPORT are the template analogs of WTF_MAKE_TZONE_ALLOCATED and
+//     WTF_MAKE_TZONE_ALLOCATED_EXPORT. Templates definitions annotated with these macros must be followed by WTF_MAKE_TZONE_ALLOCATED_TEMPLATE_IMPL
+//     or WTF_MAKE_TZONE_ALLOCATED_TEMPLATE_IMPL_WITH_MULTIPLE_OR_SPECIALIZED_PARAMETERS to provide the instantiation of statics used by the allocator.
+//
+//     WTF_MAKE_COMPACT_TZONE_ALLOCATED and WTF_MAKE_COMPACT_TZONE_ALLOCATED_EXPORT are analogs of WTF_MAKE_TZONE_ALLOCATED and
+//     WTF_MAKE_TZONE_ALLOCATED_EXPORT, but or special classes whose object pointers need to be stored in some custom scheme e.g. in CompactPtrs.
+//     An example of this is the StringImpl class.
+//
+//     The WTF_MAKE_TZONE_OR_ISO_ALLOCATED family of macros is just like WTF_MAKE_TZONE_ALLOCATED, except they will fall back to WTF_MAKE_ISO_ALLOCATED,
+//     when TZone allocation is disabled, but IsoHeap allocation is enabled.
+//
 // - WTF_MAKE_ISO_ALLOCATED(ClassName)
 // - WTF_MAKE_ISO_ALLOCATED_EXPORT(ClassName, exportMacro)
+//     Note, these annotations are legacy and should not be used for new code.
 //     class / struct is allocated from bmalloc IsoHeap. IsoHeap assigns virtual address only for particular type,
 //     so that this avoids use-after-free based type punning. We are adopting IsoHeap mainly for class / struct which is exposed to user JavaScript (e.g. DOM objects).
 //     For example , all the derived classes of ScriptWrappable must be allocated in IsoHeap.
@@ -74,17 +150,6 @@ namespace WTF {
 WTF_EXPORT_PRIVATE void fastSetMaxSingleAllocationSize(size_t);
 #endif
 
-class TryMallocReturnValue {
-public:
-    TryMallocReturnValue(void*);
-    TryMallocReturnValue(TryMallocReturnValue&&);
-    ~TryMallocReturnValue();
-    template<typename T> bool getValue(T*&) WARN_UNUSED_RETURN;
-private:
-    void operator=(TryMallocReturnValue&&) = delete;
-    mutable void* m_data;
-};
-
 WTF_EXPORT_PRIVATE bool isFastMallocEnabled();
 
 // These functions call CRASH() if an allocation fails.
@@ -134,41 +199,12 @@ WTF_EXPORT_PRIVATE void releaseFastMallocFreeMemoryForThisThread();
 WTF_EXPORT_PRIVATE void fastCommitAlignedMemory(void*, size_t);
 WTF_EXPORT_PRIVATE void fastDecommitAlignedMemory(void*, size_t);
 
-WTF_EXPORT_PRIVATE void fastEnableMiniMode();
+WTF_EXPORT_PRIVATE void fastEnableMiniMode(bool forceMiniMode = false);
 
 WTF_EXPORT_PRIVATE void fastDisableScavenger();
 
-class ForbidMallocUseForCurrentThreadScope {
-public:
-#if ASSERT_ENABLED
-    WTF_EXPORT_PRIVATE ForbidMallocUseForCurrentThreadScope();
-    WTF_EXPORT_PRIVATE ~ForbidMallocUseForCurrentThreadScope();
-#else
-    ForbidMallocUseForCurrentThreadScope() = default;
-    ~ForbidMallocUseForCurrentThreadScope() { }
-#endif
-
-    ForbidMallocUseForCurrentThreadScope(const ForbidMallocUseForCurrentThreadScope&) = delete;
-    ForbidMallocUseForCurrentThreadScope(ForbidMallocUseForCurrentThreadScope&&) = delete;
-    ForbidMallocUseForCurrentThreadScope& operator=(const ForbidMallocUseForCurrentThreadScope&) = delete;
-    ForbidMallocUseForCurrentThreadScope& operator=(ForbidMallocUseForCurrentThreadScope&&) = delete;
-};
-
-class DisableMallocRestrictionsForCurrentThreadScope {
-public:
-#if ASSERT_ENABLED
-    WTF_EXPORT_PRIVATE DisableMallocRestrictionsForCurrentThreadScope();
-    WTF_EXPORT_PRIVATE ~DisableMallocRestrictionsForCurrentThreadScope();
-#else
-    DisableMallocRestrictionsForCurrentThreadScope() = default;
-    ~DisableMallocRestrictionsForCurrentThreadScope() { }
-#endif
-
-    DisableMallocRestrictionsForCurrentThreadScope(const DisableMallocRestrictionsForCurrentThreadScope&) = delete;
-    DisableMallocRestrictionsForCurrentThreadScope(DisableMallocRestrictionsForCurrentThreadScope&&) = delete;
-    DisableMallocRestrictionsForCurrentThreadScope& operator=(const DisableMallocRestrictionsForCurrentThreadScope&) = delete;
-    DisableMallocRestrictionsForCurrentThreadScope& operator=(DisableMallocRestrictionsForCurrentThreadScope&&) = delete;
-};
+// allocate with guard pages at a rate of 1/guardMallocRate
+WTF_EXPORT_PRIVATE void forceEnablePGM(uint16_t guardMallocRate);
 
 struct FastMallocStatistics {
     size_t reservedVMBytes;
@@ -183,29 +219,6 @@ WTF_EXPORT_PRIVATE void fastMallocDumpMallocStats();
 // size as the minimally aligned memory allocation.
 typedef unsigned long long AllocAlignmentInteger;
 
-inline TryMallocReturnValue::TryMallocReturnValue(void* data)
-    : m_data(data)
-{
-}
-
-inline TryMallocReturnValue::TryMallocReturnValue(TryMallocReturnValue&& source)
-    : m_data(source.m_data)
-{
-    source.m_data = nullptr;
-}
-
-inline TryMallocReturnValue::~TryMallocReturnValue()
-{
-    ASSERT(!m_data);
-}
-
-template<typename T> inline bool TryMallocReturnValue::getValue(T*& data)
-{
-    data = static_cast<T*>(m_data);
-    m_data = nullptr;
-    return data;
-}
-
 // C++ STL allocator implementation. You can integrate fastMalloc into STL containers.
 // e.g. std::unordered_map<Key, Value, std::hash<Key>, std::equal_to<Key>, FastAllocator<std::pair<const Key, Value>>>.
 template<typename T>
@@ -235,6 +248,8 @@ public:
 
 template<typename T, typename U> inline bool operator==(const FastAllocator<T>&, const FastAllocator<U>&) { return true; }
 
+struct FastCompactMalloc;
+
 struct FastMalloc {
     static void* malloc(size_t size) { return fastMalloc(size); }
     
@@ -257,7 +272,7 @@ struct FastMalloc {
             return realResult;
         return nullptr;
     }
-    
+
     static void* realloc(void* p, size_t size) { return fastRealloc(p, size); }
 
     static void* tryRealloc(void* p, size_t size)
@@ -270,6 +285,21 @@ struct FastMalloc {
     }
     
     static void free(void* p) { fastFree(p); }
+
+    static void fastFree(void* p) { ::WTF::fastFree(p); }
+
+    static constexpr ALWAYS_INLINE size_t nextCapacity(size_t capacity)
+    {
+        return capacity + capacity / 4 + 1;
+    }
+
+    using CompactMalloc = FastCompactMalloc;
+};
+
+struct FastAlignedMalloc {
+    static void* alignedMalloc(size_t alignment, size_t size) { return fastAlignedMalloc(alignment, size); }
+    static void* tryAlignedMalloc(size_t alignment, size_t size) { return tryFastAlignedMalloc(alignment, size); }
+    static void free(void* p) { fastAlignedFree(p); }
 };
 
 struct FastCompactMalloc {
@@ -307,6 +337,13 @@ struct FastCompactMalloc {
     }
 
     static void free(void* p) { fastFree(p); }
+
+    static void fastFree(void* p) { ::WTF::fastFree(p); }
+
+    static constexpr ALWAYS_INLINE size_t nextCapacity(size_t capacity)
+    {
+        return capacity + capacity / 4 + 1;
+    }
 };
 
 template<typename T>
@@ -372,12 +409,11 @@ inline constexpr std::enable_if_t<!WTF::IsTypeComplete<std::remove_pointer_t<T>>
     return AllowCompactPointers<std::remove_const_t<std::remove_pointer_t<T>>*>::value;
 }
 
-using WTF::DisableMallocRestrictionsForCurrentThreadScope;
+using WTF::FastAlignedMalloc;
 using WTF::FastAllocator;
 using WTF::FastMalloc;
 using WTF::FastCompactMalloc;
 using WTF::FastFree;
-using WTF::ForbidMallocUseForCurrentThreadScope;
 using WTF::isFastMallocEnabled;
 using WTF::fastCalloc;
 using WTF::fastFree;
@@ -406,12 +442,6 @@ using WTF::tryFastCompactMalloc;
 using WTF::tryFastCompactZeroedMalloc;
 using WTF::fastCompactAlignedMalloc;
 
-#if OS(DARWIN)
-#define WTF_PRIVATE_INLINE __private_extern__ inline __attribute__((always_inline))
-#else
-#define WTF_PRIVATE_INLINE inline __attribute__((always_inline))
-#endif
-
 #define WTF_MAKE_FAST_ALLOCATED_IMPL \
     void* operator new(size_t, void* p) { return p; } \
     void* operator new[](size_t, void* p) { return p; } \
@@ -444,7 +474,7 @@ using WTF::fastCompactAlignedMalloc;
     { \
         ::WTF::fastFree(p); \
     } \
-    using WTFIsFastAllocated = int; \
+    using WTFIsFastMallocAllocated = int; \
 
 #define WTF_MAKE_FAST_COMPACT_ALLOCATED_IMPL \
     WTF_ALLOW_COMPACT_POINTERS_IMPL; \
@@ -479,7 +509,7 @@ using WTF::fastCompactAlignedMalloc;
     { \
         ::WTF::fastFree(p); \
     } \
-    using WTFIsFastAllocated = int; \
+    using WTFIsFastMallocAllocated = int; \
 
 // FIXME: WTF_MAKE_FAST_ALLOCATED should take class name so that we can create malloc_zone per this macro.
 // https://bugs.webkit.org/show_bug.cgi?id=205702
@@ -537,7 +567,7 @@ using __thisIsHereToForceASemicolonAfterThisMacro UNUSED_TYPE_ALIAS = int
     { \
         classname##Malloc::free(p); \
     } \
-    using WTFIsFastAllocated = int; \
+    using WTFIsFastMallocAllocated = int; \
 
 #define WTF_MAKE_FAST_ALLOCATED_WITH_HEAP_IDENTIFIER(classname) \
 public: \
@@ -596,10 +626,10 @@ using __thisIsHereToForceASemicolonAfterThisMacro UNUSED_TYPE_ALIAS = int
 // Note: WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR must be declared in every subclass.
 #define WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR_IMPL(T) \
 void operator delete(T* object, std::destroying_delete_t, size_t size) { \
-    ASSERT(sizeof(T) == size); \
+    ASSERT_UNUSED(size, sizeof(T) == size); \
     object->T::~T(); \
-    if (UNLIKELY(object->ptrCountWithoutThreadCheck())) { \
-        memset(static_cast<void*>(object), 0, size); \
+    if (UNLIKELY(object->checkedPtrCountWithoutThreadCheck())) { \
+        zeroBytes(object); \
         return; \
     } \
     T::operator delete(object); \
