@@ -27,6 +27,7 @@
 #include <string.h>
 #include <type_traits>
 #include <utility>
+#include <wtf/AlignedStorage.h>
 #include <wtf/Assertions.h>
 #include <wtf/DebugHeap.h>
 #include <wtf/FastMalloc.h>
@@ -650,11 +651,8 @@ DECLARE_ALLOCATOR_WITH_HEAP_IDENTIFIER(HashTable);
         } else if constexpr (HashFunctions::safeToCompareToEmptyOrDeleted) {
             RELEASE_ASSERT(!HashTranslator::equal(KeyTraits::emptyValue(), key));
 
-            ALLOW_DEPRECATED_DECLARATIONS_BEGIN
-            typename std::aligned_storage<sizeof(ValueType), std::alignment_of<ValueType>::value>::type deletedValueBuffer;
-            ALLOW_DEPRECATED_DECLARATIONS_END
-            ValueType* deletedValuePtr = reinterpret_cast_ptr<ValueType*>(&deletedValueBuffer);
-            ValueType& deletedValue = *deletedValuePtr;
+            AlignedStorage<ValueType> deletedValueBuffer;
+            auto& deletedValue = *deletedValueBuffer;
             Traits::constructDeletedValue(deletedValue);
             RELEASE_ASSERT(!HashTranslator::equal(Extractor::extract(deletedValue), key));
         }
