@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016 Canon Inc.
- * Copyright (C) 2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2020-2024 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted, provided that the following conditions
@@ -40,16 +40,21 @@
 #include "ResourceError.h"
 #include "ScriptExecutionContext.h"
 #include <JavaScriptCore/JSONObject.h>
-#include <wtf/text/StringConcatenateNumbers.h>
+#include <wtf/text/MakeString.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(FetchResponseBodyLoader);
+
 // https://fetch.spec.whatwg.org/#null-body-status
 static inline bool isNullBodyStatus(int status)
 {
     return status == 101 || status == 204 || status == 205 || status == 304;
 }
 
+FetchResponse::~FetchResponse() = default;
+
 Ref<FetchResponse> FetchResponse::create(ScriptExecutionContext* context, std::optional<FetchBody>&& body, FetchHeaders::Guard guard, ResourceResponse&& response)
 {
     bool isSynthetic = response.type() == ResourceResponse::Type::Default || response.type() == ResourceResponse::Type::Error;
@@ -270,7 +275,21 @@ void FetchResponse::fetch(ScriptExecutionContext& context, FetchRequest& request
         return;
     }
 
-    auto response = createFetchResponse(context, request, WTFMove(responseCallback));
+    if (request.hasReadableStreamBody()) {
+        request.body().convertReadableStreamToArrayBuffer(request, [context = Ref { context }, weakRequest = WeakPtr { request }, responseCallback = WTFMove(responseCallback), initiator](auto&& exception) mutable {
+            if (!!exception) {
+                responseCallback(WTFMove(*exception));
+                return;
+            }
+
+            Ref protectedRequest = *weakRequest;
+            Ref response = createFetchResponse(context.get(), protectedRequest.get(), WTFMove(responseCallback));
+            response->startLoader(context.get(), protectedRequest.get(), initiator);
+        });
+        return;
+    }
+
+    Ref response = createFetchResponse(context, request, WTFMove(responseCallback));
     response->startLoader(context, request, initiator);
 }
 
@@ -299,30 +318,30 @@ const ResourceResponse& FetchResponse::filteredResponse() const
     return m_internalResponse;
 }
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(FetchResponse::Loader);
+
 void FetchResponse::Loader::didSucceed(const NetworkLoadMetrics& metrics)
 {
-    ASSERT(m_response.hasPendingActivity());
+    Ref response = m_response.get();
+    ASSERT(response->hasPendingActivity());
 
-    m_response.didSucceed(metrics);
+    response->didSucceed(metrics);
 
-    if (m_loader->isStarted()) {
-        Ref<FetchResponse> protector(m_response);
-        m_response.m_loader = nullptr;
-    }
+    if (m_loader->isStarted())
+        response->m_loader = nullptr;
 }
 
 void FetchResponse::Loader::didFail(const ResourceError& error)
 {
-    ASSERT(m_response.hasPendingActivity());
+    Ref response = m_response.get();
+    ASSERT(response->hasPendingActivity());
 
-    m_response.setLoadingError(ResourceError { error });
-    m_response.processReceivedError();
+    response->setLoadingError(ResourceError { error });
+    response->processReceivedError();
 
     // Check whether didFail is called as part of FetchLoader::start.
-    if (m_loader && m_loader->isStarted()) {
-        Ref<FetchResponse> protector(m_response);
-        m_response.m_loader = nullptr;
-    }
+    if (m_loader && m_loader->isStarted())
+        response->m_loader = nullptr;
 }
 
 static std::atomic<uint64_t> nextOpaqueLoadIdentifier;
@@ -347,7 +366,7 @@ void FetchResponse::setReceivedInternalResponse(const ResourceResponse& resource
 FetchResponse::Loader::Loader(FetchResponse& response, NotificationCallback&& responseCallback)
     : m_response(response)
     , m_responseCallback(WTFMove(responseCallback))
-    , m_pendingActivity(m_response.makePendingActivity(m_response))
+    , m_pendingActivity(response.makePendingActivity(response))
 {
 }
 
@@ -355,15 +374,17 @@ FetchResponse::Loader::~Loader() = default;
 
 void FetchResponse::Loader::didReceiveResponse(const ResourceResponse& resourceResponse)
 {
-    m_response.setReceivedInternalResponse(resourceResponse, m_credentials);
+    Ref response = m_response.get();
+    response->setReceivedInternalResponse(resourceResponse, m_credentials);
 
     if (auto responseCallback = WTFMove(m_responseCallback))
-        responseCallback(Ref { m_response });
+        responseCallback(WTFMove(response));
 }
 
 void FetchResponse::Loader::didReceiveData(const SharedBuffer& buffer)
 {
-    ASSERT(m_response.m_readableStreamSource || m_consumeDataCallback);
+    Ref response = m_response.get();
+    ASSERT(response->m_readableStreamSource || m_consumeDataCallback);
 
     if (m_consumeDataCallback) {
         auto chunk = buffer.span();
@@ -371,14 +392,14 @@ void FetchResponse::Loader::didReceiveData(const SharedBuffer& buffer)
         return;
     }
 
-    auto& source = *m_response.m_readableStreamSource;
+    auto& source = *response->m_readableStreamSource;
 
     if (!source.isPulling()) {
-        m_response.body().consumer().append(buffer);
+        response->body().consumer().append(buffer);
         return;
     }
 
-    if (m_response.body().consumer().hasData() && !source.enqueue(m_response.body().consumer().takeAsArrayBuffer())) {
+    if (response->body().consumer().hasData() && !source.enqueue(response->body().consumer().takeAsArrayBuffer())) {
         stop();
         return;
     }
@@ -392,7 +413,7 @@ void FetchResponse::Loader::didReceiveData(const SharedBuffer& buffer)
 bool FetchResponse::Loader::start(ScriptExecutionContext& context, const FetchRequest& request, const String& initiator)
 {
     m_credentials = request.fetchOptions().credentials;
-    m_loader = makeUnique<FetchLoader>(*this, &m_response.m_body->consumer());
+    m_loader = makeUnique<FetchLoader>(*this, &m_response->m_body->consumer());
     m_loader->start(context, request, initiator);
 
     if (!m_loader->isStarted())
