@@ -27,8 +27,8 @@
 
 #include "AudioBufferSourceOptions.h"
 #include "AudioScheduledSourceNode.h"
+#include <wtf/FixedVector.h>
 #include <wtf/Lock.h>
-#include <wtf/UniqueArray.h>
 
 namespace WebCore {
 
@@ -39,7 +39,7 @@ struct AudioBufferSourceOptions;
 // It generally will be used for short sounds which require a high degree of scheduling flexibility (can playback in rhythmically perfect ways).
 
 class AudioBufferSourceNode final : public AudioScheduledSourceNode {
-    WTF_MAKE_ISO_ALLOCATED(AudioBufferSourceNode);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(AudioBufferSourceNode);
 public:
     static ExceptionOr<Ref<AudioBufferSourceNode>> create(BaseAudioContext&, AudioBufferSourceOptions&& = { });
 
@@ -80,8 +80,6 @@ public:
     // If we are no longer playing, propogate silence ahead to downstream nodes.
     bool propagatesSilence() const final;
 
-    const char* activeDOMObjectName() const override { return "AudioBufferSourceNode"; }
-
 private:
     AudioBufferSourceNode(BaseAudioContext&);
 
@@ -104,9 +102,9 @@ private:
     // m_buffer holds the sample data which this node outputs.
     RefPtr<AudioBuffer> m_buffer WTF_GUARDED_BY_LOCK(m_processLock); // Only modified on the main thread but used on the audio thread.
 
-    // Pointers for the buffer and destination.
-    UniqueArray<const float*> m_sourceChannels;
-    UniqueArray<float*> m_destinationChannels;
+    // Spans for the buffer and destination.
+    FixedVector<std::span<const float>> m_sourceChannels;
+    FixedVector<std::span<float>> m_destinationChannels;
 
     Ref<AudioParam> m_detune;
     Ref<AudioParam> m_playbackRate;
