@@ -29,40 +29,39 @@
  */
 
 #include "config.h"
-
-#if ENABLE(WEB_SOCKETS)
-
 #include "WebSocketDeflateFramer.h"
 
+#include "WebSocketExtensionProcessor.h"
+#include "WebSocketFrame.h"
 #include <wtf/HashMap.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/text/StringHash.h>
-#include <wtf/text/WTFString.h>
+#include <wtf/text/StringToIntegerConversion.h>
 
 namespace WebCore {
 
-class WebSocketExtensionDeflateFrame : public WebSocketExtensionProcessor {
-    WTF_MAKE_FAST_ALLOCATED;
+class WebSocketExtensionDeflateFrame final : public WebSocketExtensionProcessor {
+    WTF_MAKE_TZONE_ALLOCATED(WebSocketExtensionDeflateFrame);
 public:
-    explicit WebSocketExtensionDeflateFrame(WebSocketDeflateFramer*);
-    virtual ~WebSocketExtensionDeflateFrame() { }
-
-    virtual String handshakeString() override;
-    virtual bool processResponse(const HashMap<String, String>&) override;
-    virtual String failureReason() override { return m_failureReason; }
+    explicit WebSocketExtensionDeflateFrame(WebSocketDeflateFramer&);
 
 private:
-    WebSocketDeflateFramer* m_framer;
-    bool m_responseProcessed;
+    String handshakeString() final;
+    bool processResponse(const HashMap<String, String>&) final;
+    String failureReason() final { return m_failureReason; }
+
+    WebSocketDeflateFramer& m_framer;
+    bool m_responseProcessed { false };
     String m_failureReason;
 };
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(WebSocketExtensionDeflateFrame);
+
 // FXIME: Remove vendor prefix after the specification matured.
-WebSocketExtensionDeflateFrame::WebSocketExtensionDeflateFrame(WebSocketDeflateFramer* framer)
-    : WebSocketExtensionProcessor("x-webkit-deflate-frame")
+WebSocketExtensionDeflateFrame::WebSocketExtensionDeflateFrame(WebSocketDeflateFramer& framer)
+    : WebSocketExtensionProcessor("x-webkit-deflate-frame"_s)
     , m_framer(framer)
-    , m_responseProcessed(false)
 {
-    ASSERT(m_framer);
 }
 
 String WebSocketExtensionDeflateFrame::handshakeString()
@@ -72,30 +71,29 @@ String WebSocketExtensionDeflateFrame::handshakeString()
 
 bool WebSocketExtensionDeflateFrame::processResponse(const HashMap<String, String>& serverParameters)
 {
-#if USE(ZLIB)
     if (m_responseProcessed) {
-        m_failureReason = "Received duplicate deflate-frame response";
+        m_failureReason = "Received duplicate deflate-frame response"_s;
         return false;
     }
     m_responseProcessed = true;
 
-    int expectedNumParameters = 0;
+    unsigned expectedNumParameters = 0;
     int windowBits = 15;
-    HashMap<String, String>::const_iterator parameter = serverParameters.find("max_window_bits");
+    auto parameter = serverParameters.find<HashTranslatorASCIILiteral>("max_window_bits"_s);
     if (parameter != serverParameters.end()) {
-        windowBits = parameter->value.toInt();
+        windowBits = parseIntegerAllowingTrailingJunk<int>(parameter->value).value_or(0);
         if (windowBits < 8 || windowBits > 15) {
-            m_failureReason = "Received invalid max_window_bits parameter";
+            m_failureReason = "Received invalid max_window_bits parameter"_s;
             return false;
         }
         expectedNumParameters++;
     }
 
     WebSocketDeflater::ContextTakeOverMode mode = WebSocketDeflater::TakeOverContext;
-    parameter = serverParameters.find("no_context_takeover");
+    parameter = serverParameters.find<HashTranslatorASCIILiteral>("no_context_takeover"_s);
     if (parameter != serverParameters.end()) {
         if (!parameter->value.isNull()) {
-            m_failureReason = "Received invalid no_context_takeover parameter";
+            m_failureReason = "Received invalid no_context_takeover parameter"_s;
             return false;
         }
         mode = WebSocketDeflater::DoNotTakeOverContext;
@@ -103,28 +101,24 @@ bool WebSocketExtensionDeflateFrame::processResponse(const HashMap<String, Strin
     }
 
     if (expectedNumParameters != serverParameters.size()) {
-        m_failureReason = "Received unexpected deflate-frame parameter";
+        m_failureReason = "Received unexpected deflate-frame parameter"_s;
         return false;
     }
 
-    m_framer->enableDeflate(windowBits, mode);
+    m_framer.enableDeflate(windowBits, mode);
     return true;
-#else
-    ASSERT_NOT_REACHED();
-    return false;
-#endif
 }
 
-DeflateResultHolder::DeflateResultHolder(WebSocketDeflateFramer* framer)
+WTF_MAKE_TZONE_ALLOCATED_IMPL(DeflateResultHolder);
+
+DeflateResultHolder::DeflateResultHolder(WebSocketDeflateFramer& framer)
     : m_framer(framer)
-    , m_succeeded(true)
 {
-    ASSERT(m_framer);
 }
 
 DeflateResultHolder::~DeflateResultHolder()
 {
-    m_framer->resetDeflateContext();
+    m_framer.resetDeflateContext();
 }
 
 void DeflateResultHolder::fail(const String& failureReason)
@@ -133,16 +127,16 @@ void DeflateResultHolder::fail(const String& failureReason)
     m_failureReason = failureReason;
 }
 
-InflateResultHolder::InflateResultHolder(WebSocketDeflateFramer* framer)
+WTF_MAKE_TZONE_ALLOCATED_IMPL(InflateResultHolder);
+
+InflateResultHolder::InflateResultHolder(WebSocketDeflateFramer& framer)
     : m_framer(framer)
-    , m_succeeded(true)
 {
-    ASSERT(m_framer);
 }
 
 InflateResultHolder::~InflateResultHolder()
 {
-    m_framer->resetInflateContext();
+    m_framer.resetInflateContext();
 }
 
 void InflateResultHolder::fail(const String& failureReason)
@@ -151,30 +145,15 @@ void InflateResultHolder::fail(const String& failureReason)
     m_failureReason = failureReason;
 }
 
-WebSocketDeflateFramer::WebSocketDeflateFramer()
-    : m_enabled(false)
-{
-}
-
 std::unique_ptr<WebSocketExtensionProcessor> WebSocketDeflateFramer::createExtensionProcessor()
 {
-    return std::make_unique<WebSocketExtensionDeflateFrame>(this);
+    return makeUnique<WebSocketExtensionDeflateFrame>(*this);
 }
 
-bool WebSocketDeflateFramer::canDeflate() const
-{
-#if USE(ZLIB)
-    return true;
-#else
-    return false;
-#endif
-}
-
-#if USE(ZLIB)
 void WebSocketDeflateFramer::enableDeflate(int windowBits, WebSocketDeflater::ContextTakeOverMode mode)
 {
-    m_deflater = std::make_unique<WebSocketDeflater>(windowBits, mode);
-    m_inflater = std::make_unique<WebSocketInflater>();
+    m_deflater = makeUnique<WebSocketDeflater>(windowBits, mode);
+    m_inflater = makeUnique<WebSocketInflater>();
     if (!m_deflater->initialize() || !m_inflater->initialize()) {
         m_deflater = nullptr;
         m_inflater = nullptr;
@@ -182,68 +161,53 @@ void WebSocketDeflateFramer::enableDeflate(int windowBits, WebSocketDeflater::Co
     }
     m_enabled = true;
 }
-#endif
 
 std::unique_ptr<DeflateResultHolder> WebSocketDeflateFramer::deflate(WebSocketFrame& frame)
 {
-#if USE(ZLIB)
-    auto result = std::make_unique<DeflateResultHolder>(this);
-    if (!enabled() || !WebSocketFrame::isNonControlOpCode(frame.opCode) || !frame.payloadLength)
+    auto result = makeUnique<DeflateResultHolder>(*this);
+    if (!enabled() || !WebSocketFrame::isNonControlOpCode(frame.opCode) || !frame.payload.size())
         return result;
-    if (!m_deflater->addBytes(frame.payload, frame.payloadLength) || !m_deflater->finish()) {
-        result->fail("Failed to compress frame");
+    if (!m_deflater->addBytes(frame.payload) || !m_deflater->finish()) {
+        result->fail("Failed to compress frame"_s);
         return result;
     }
     frame.compress = true;
-    frame.payload = m_deflater->data();
-    frame.payloadLength = m_deflater->size();
+    frame.payload = m_deflater->span();
     return result;
-#else
-    return std::make_unique<DeflateResultHolder>(this);
-#endif
 }
 
 void WebSocketDeflateFramer::resetDeflateContext()
 {
-#if USE(ZLIB)
     if (m_deflater)
         m_deflater->reset();
-#endif
 }
 
 std::unique_ptr<InflateResultHolder> WebSocketDeflateFramer::inflate(WebSocketFrame& frame)
 {
-    auto result = std::make_unique<InflateResultHolder>(this);
+    auto result = makeUnique<InflateResultHolder>(*this);
     if (!enabled() && frame.compress) {
-        result->fail("Compressed bit must be 0 if no negotiated deflate-frame extension");
+        result->fail("Compressed bit must be 0 if no negotiated deflate-frame extension"_s);
         return result;
     }
-#if USE(ZLIB)
     if (!frame.compress)
         return result;
     if (!WebSocketFrame::isNonControlOpCode(frame.opCode)) {
-        result->fail("Received unexpected compressed frame");
+        result->fail("Received unexpected compressed frame"_s);
         return result;
     }
-    if (!m_inflater->addBytes(frame.payload, frame.payloadLength) || !m_inflater->finish()) {
-        result->fail("Failed to decompress frame");
+    if (!m_inflater->addBytes(frame.payload) || !m_inflater->finish()) {
+        result->fail("Failed to decompress frame"_s);
         return result;
     }
     frame.compress = false;
-    frame.payload = m_inflater->data();
-    frame.payloadLength = m_inflater->size();
+    frame.payload = m_inflater->span();
     return result;
-#else
-    return result;
-#endif
 }
 
 void WebSocketDeflateFramer::resetInflateContext()
 {
-#if USE(ZLIB)
     if (m_inflater)
         m_inflater->reset();
-#endif
 }
 
 void WebSocketDeflateFramer::didFail()
@@ -253,5 +217,3 @@ void WebSocketDeflateFramer::didFail()
 }
 
 } // namespace WebCore
-
-#endif // ENABLE(WEB_SOCKETS)
