@@ -54,11 +54,18 @@ void CSSMediaRule::setMediaQueries(MQ::MediaQueryList&& queries)
 
 String CSSMediaRule::cssText() const
 {
-    StringBuilder result;
-    result.append("@media ", conditionText(), " {\n");
-    appendCSSTextForItems(result);
-    result.append('}');
-    return result.toString();
+    StringBuilder builder;
+    builder.append("@media "_s, conditionText());
+    appendCSSTextForItems(builder);
+    return builder.toString();
+}
+
+String CSSMediaRule::cssText(const CSS::SerializationContext& context) const
+{
+    StringBuilder builder;
+    builder.append("@media "_s, conditionText());
+    appendCSSTextWithReplacementURLsForItems(builder, context);
+    return builder.toString();
 }
 
 String CSSMediaRule::conditionText() const
