@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2022 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2024 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,17 +26,17 @@
 #include "config.h"
 #include "FontFace.h"
 
-#include "AllowedFonts.h"
 #include "CSSFontFaceSource.h"
 #include "CSSFontSelector.h"
 #include "CSSPrimitiveValueMappings.h"
-#include "CSSPropertyParserWorkerSafe.h"
+#include "CSSPropertyParserConsumer+Font.h"
 #include "CSSValueList.h"
 #include "CSSValuePool.h"
 #include "DOMPromiseProxy.h"
 #include "Document.h"
 #include "DocumentInlines.h"
 #include "JSFontFace.h"
+#include "TrustedFonts.h"
 #include <JavaScriptCore/ArrayBuffer.h>
 #include <JavaScriptCore/ArrayBufferView.h>
 #include <JavaScriptCore/JSCInlines.h>
@@ -62,7 +62,7 @@ Ref<FontFace> FontFace::create(ScriptExecutionContext& context, const String& fa
     auto result = adoptRef(*new FontFace(*context.cssFontSelector()));
     result->suspendIfNeeded();
 
-#if COMPILER(GCC) && CPU(ARM64)
+#if COMPILER(GCC) && (CPU(ARM) || CPU(ARM64))
     // FIXME: Workaround for GCC bug https://gcc.gnu.org/bugzilla/show_bug.cgi?id=115033
     // that is related to https://gcc.gnu.org/bugzilla/show_bug.cgi?id=115135 as well.
     volatile
@@ -75,25 +75,25 @@ Ref<FontFace> FontFace::create(ScriptExecutionContext& context, const String& fa
         return result;
     }
 
-    auto fontAllowedTypes = context.settingsValues().downloadableBinaryFontAllowedTypes;
+    auto fontTrustedTypes = context.settingsValues().downloadableBinaryFontTrustedTypes;
     auto sourceConversionResult = WTF::switchOn(source,
         [&] (String& string) -> ExceptionOr<void> {
             auto* document = dynamicDowncast<Document>(context);
-            auto value = CSSPropertyParserWorkerSafe::parseFontFaceSrc(string, document ? CSSParserContext(*document) : HTMLStandardMode);
+            auto value = CSSPropertyParserHelpers::parseFontFaceSrc(string, document ? CSSParserContext(*document) : HTMLStandardMode);
             if (!value)
                 return Exception { ExceptionCode::SyntaxError };
             CSSFontFace::appendSources(result->backing(), *value, &context, false);
             return { };
         },
-        [&, fontAllowedTypes] (RefPtr<ArrayBufferView>& arrayBufferView) -> ExceptionOr<void> {
-            if (!arrayBufferView || !isFontBinaryAllowed(arrayBufferView->data(), arrayBufferView->byteLength(), fontAllowedTypes))
+        [&, fontTrustedTypes] (RefPtr<ArrayBufferView>& arrayBufferView) -> ExceptionOr<void> {
+            if (!arrayBufferView || fontBinaryParsingPolicy(arrayBufferView->span(), fontTrustedTypes) == FontParsingPolicy::Deny)
                 return { };
 
             dataRequiresAsynchronousLoading = populateFontFaceWithArrayBuffer(result->backing(), arrayBufferView.releaseNonNull());
             return { };
         },
-        [&, fontAllowedTypes] (RefPtr<ArrayBuffer>& arrayBuffer) -> ExceptionOr<void> {
-            if (!arrayBuffer || !isFontBinaryAllowed(arrayBuffer->data(), arrayBuffer->byteLength(), fontAllowedTypes))
+        [&, fontTrustedTypes] (RefPtr<ArrayBuffer>& arrayBuffer) -> ExceptionOr<void> {
+            if (!arrayBuffer || fontBinaryParsingPolicy(arrayBuffer->span(), fontTrustedTypes) == FontParsingPolicy::Deny)
                 return { };
 
             unsigned byteLength = arrayBuffer->byteLength();
@@ -119,8 +119,8 @@ Ref<FontFace> FontFace::create(ScriptExecutionContext& context, const String& fa
         result->setErrorState();
         return result;
     }
-    auto setStretchResult = result->setStretch(context, descriptors.stretch.isEmpty() ? "normal"_s : descriptors.stretch);
-    if (setStretchResult.hasException()) {
+    auto setWidthResult = result->setWidth(context, descriptors.width.isEmpty() ? "normal"_s : descriptors.width);
+    if (setWidthResult.hasException()) {
         result->setErrorState();
         return result;
     }
@@ -193,7 +193,7 @@ ExceptionOr<void> FontFace::setFamily(ScriptExecutionContext& context, const Str
 
 ExceptionOr<void> FontFace::setStyle(ScriptExecutionContext& context, const String& style)
 {
-    if (auto value = CSSPropertyParserWorkerSafe::parseFontFaceStyle(style, context)) {
+    if (auto value = CSSPropertyParserHelpers::parseFontFaceFontStyle(style, context)) {
         m_backing->setStyle(*value);
         return { };
     }
@@ -202,17 +202,17 @@ ExceptionOr<void> FontFace::setStyle(ScriptExecutionContext& context, const Stri
 
 ExceptionOr<void> FontFace::setWeight(ScriptExecutionContext& context, const String& weight)
 {
-    if (auto value = CSSPropertyParserWorkerSafe::parseFontFaceWeight(weight, context)) {
+    if (auto value = CSSPropertyParserHelpers::parseFontFaceFontWeight(weight, context)) {
         m_backing->setWeight(*value);
         return { };
     }
     return Exception { ExceptionCode::SyntaxError };
 }
 
-ExceptionOr<void> FontFace::setStretch(ScriptExecutionContext& context, const String& stretch)
+ExceptionOr<void> FontFace::setWidth(ScriptExecutionContext& context, const String& width)
 {
-    if (auto value = CSSPropertyParserWorkerSafe::parseFontFaceStretch(stretch, context)) {
-        m_backing->setStretch(*value);
+    if (auto value = CSSPropertyParserHelpers::parseFontFaceFontWidth(width, context)) {
+        m_backing->setWidth(*value);
         return { };
     }
     return Exception { ExceptionCode::SyntaxError };
@@ -220,7 +220,7 @@ ExceptionOr<void> FontFace::setStretch(ScriptExecutionContext& context, const St
 
 ExceptionOr<void> FontFace::setUnicodeRange(ScriptExecutionContext& context, const String& unicodeRange)
 {
-    if (auto value = CSSPropertyParserWorkerSafe::parseFontFaceUnicodeRange(unicodeRange, context)) {
+    if (auto value = CSSPropertyParserHelpers::parseFontFaceUnicodeRange(unicodeRange, context)) {
         m_backing->setUnicodeRange(*value);
         return { };
     }
@@ -229,7 +229,7 @@ ExceptionOr<void> FontFace::setUnicodeRange(ScriptExecutionContext& context, con
 
 ExceptionOr<void> FontFace::setFeatureSettings(ScriptExecutionContext& context, const String& featureSettings)
 {
-    if (auto value = CSSPropertyParserWorkerSafe::parseFontFaceFeatureSettings(featureSettings, context)) {
+    if (auto value = CSSPropertyParserHelpers::parseFontFaceFeatureSettings(featureSettings, context)) {
         m_backing->setFeatureSettings(*value);
         return { };
     }
@@ -238,7 +238,7 @@ ExceptionOr<void> FontFace::setFeatureSettings(ScriptExecutionContext& context,
 
 ExceptionOr<void> FontFace::setDisplay(ScriptExecutionContext& context, const String& display)
 {
-    if (auto value = CSSPropertyParserWorkerSafe::parseFontFaceDisplay(display, context)) {
+    if (auto value = CSSPropertyParserHelpers::parseFontFaceDisplay(display, context)) {
         m_backing->setDisplay(*value);
         return { };
     }
@@ -247,7 +247,7 @@ ExceptionOr<void> FontFace::setDisplay(ScriptExecutionContext& context, const St
 
 ExceptionOr<void> FontFace::setSizeAdjust(ScriptExecutionContext& context, const String& sizeAdjust)
 {
-    if (auto value = CSSPropertyParserWorkerSafe::parseFontFaceSizeAdjust(sizeAdjust, context)) {
+    if (auto value = CSSPropertyParserHelpers::parseFontFaceSizeAdjust(sizeAdjust, context)) {
         m_backing->setSizeAdjust(*value);
         return { };
     }
@@ -275,9 +275,9 @@ String FontFace::weight() const
     return "normal"_s;
 }
 
-String FontFace::stretch() const
+String FontFace::width() const
 {
-    if (auto value = m_backing->stretch(); !value.isNull())
+    if (auto value = m_backing->width(); !value.isNull())
         return value;
     return "normal"_s;
 }
@@ -380,11 +380,6 @@ FontFace& FontFace::loadedPromiseResolve()
     return *this;
 }
 
-const char* FontFace::activeDOMObjectName() const
-{
-    return "FontFace";
-}
-
 bool FontFace::virtualHasPendingActivity() const
 {
     return m_mayLoadedPromiseBeScriptObservable && !m_loadedPromise->isFulfilled();
