@@ -34,28 +34,30 @@
 
 #include "MutationObserverRegistration.h"
 #include "MutationRecord.h"
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
-std::unique_ptr<MutationObserverInterestGroup> MutationObserverInterestGroup::createIfNeeded(Node& target, MutationObserver::MutationType type, MutationRecordDeliveryOptions oldValueFlag, const QualifiedName* attributeName)
+WTF_MAKE_TZONE_ALLOCATED_IMPL(MutationObserverInterestGroup);
+
+inline MutationObserverInterestGroup::MutationObserverInterestGroup(UncheckedKeyHashMap<Ref<MutationObserver>, MutationRecordDeliveryOptions>&& observers, MutationRecordDeliveryOptions oldValueFlag)
+    : m_observers(WTFMove(observers))
+    , m_oldValueFlag(oldValueFlag)
 {
-    ASSERT((type == MutationObserver::Attributes && attributeName) || !attributeName);
-    HashMap<MutationObserver*, MutationRecordDeliveryOptions> observers;
-    target.getRegisteredMutationObserversOfType(observers, type, attributeName);
+    ASSERT(!m_observers.isEmpty());
+}
+
+std::unique_ptr<MutationObserverInterestGroup> MutationObserverInterestGroup::createIfNeeded(Node& target, MutationObserverOptionType type, MutationRecordDeliveryOptions oldValueFlag, const QualifiedName* attributeName)
+{
+    ASSERT((type == MutationObserverOptionType::Attributes && attributeName) || !attributeName);
+    auto observers = target.registeredMutationObservers(type, attributeName);
     if (observers.isEmpty())
         return nullptr;
 
-    return std::make_unique<MutationObserverInterestGroup>(observers, oldValueFlag);
+    return makeUnique<MutationObserverInterestGroup>(WTFMove(observers), oldValueFlag);
 }
 
-MutationObserverInterestGroup::MutationObserverInterestGroup(HashMap<MutationObserver*, MutationRecordDeliveryOptions>& observers, MutationRecordDeliveryOptions oldValueFlag)
-    : m_oldValueFlag(oldValueFlag)
-{
-    ASSERT(!observers.isEmpty());
-    m_observers.swap(observers);
-}
-
-bool MutationObserverInterestGroup::isOldValueRequested()
+bool MutationObserverInterestGroup::isOldValueRequested() const
 {
     for (auto options : m_observers.values()) {
         if (hasOldValue(options))
@@ -68,7 +70,7 @@ void MutationObserverInterestGroup::enqueueMutationRecord(Ref<MutationRecord>&&
 {
     RefPtr<MutationRecord> mutationWithNullOldValue;
     for (auto& observerOptionsPair : m_observers) {
-        MutationObserver* observer = observerOptionsPair.key;
+        Ref observer = observerOptionsPair.key.get();
         if (hasOldValue(observerOptionsPair.value)) {
             observer->enqueueMutationRecord(mutation.copyRef());
             continue;
