@@ -25,34 +25,34 @@
 
 #include "config.h"
 #include "CreateLinkCommand.h"
-#include "htmlediting.h"
-#include "Text.h"
 
+#include "Editing.h"
 #include "HTMLAnchorElement.h"
+#include "Text.h"
 
 namespace WebCore {
 
-CreateLinkCommand::CreateLinkCommand(Document& document, const String& url)
-    : CompositeEditCommand(document)
+CreateLinkCommand::CreateLinkCommand(Ref<Document>&& document, const String& url)
+    : CompositeEditCommand(WTFMove(document))
+    , m_url(url)
 {
-    m_url = url;
 }
 
 void CreateLinkCommand::doApply()
 {
-    if (endingSelection().isNone())
+    if (endingSelection().isNoneOrOrphaned())
         return;
-        
-    auto anchorElement = HTMLAnchorElement::create(document());
-    anchorElement->setHref(m_url);
+
+    auto document = protectedDocument();
+    auto anchorElement = HTMLAnchorElement::create(document);
+    anchorElement->setHref(AtomString { m_url });
     
     if (endingSelection().isRange())
         applyStyledElement(WTFMove(anchorElement));
     else {
-        insertNodeAt(anchorElement.ptr(), endingSelection().start());
-        auto textNode = Text::create(document(), m_url);
-        appendNode(WTFMove(textNode), anchorElement.ptr());
-        setEndingSelection(VisibleSelection(positionInParentBeforeNode(anchorElement.ptr()), positionInParentAfterNode(anchorElement.ptr()), DOWNSTREAM, endingSelection().isDirectional()));
+        insertNodeAt(anchorElement.copyRef(), endingSelection().start());
+        appendNode(Text::create(document, String { m_url }), anchorElement.copyRef());
+        setEndingSelection(VisibleSelection(positionInParentBeforeNode(anchorElement.ptr()), positionInParentAfterNode(anchorElement.ptr()), Affinity::Downstream, endingSelection().directionality()));
     }
 }
 
