@@ -29,11 +29,14 @@
 #include "SpaceSplitString.h"
 #include <wtf/HashSet.h>
 #include <wtf/SetForScope.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/text/AtomStringHash.h>
 #include <wtf/text/StringBuilder.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(DOMTokenList);
+
 DOMTokenList::DOMTokenList(Element& element, const QualifiedName& attributeName, IsSupportedTokenFunction&& isSupportedToken)
     : m_element(element)
     , m_attributeName(attributeName)
@@ -57,10 +60,10 @@ ExceptionOr<void> DOMTokenList::validateToken(StringView token)
     return { };
 }
 
-ExceptionOr<void> DOMTokenList::validateTokens(const AtomString* tokens, size_t length)
+ExceptionOr<void> DOMTokenList::validateTokens(std::span<const AtomString> tokens)
 {
-    for (size_t i = 0; i < length; ++i) {
-        auto result = validateToken(tokens[i]);
+    for (auto& token : tokens) {
+        auto result = validateToken(token);
         if (result.hasException())
             return result;
     }
@@ -72,20 +75,20 @@ bool DOMTokenList::contains(const AtomString& token) const
     return tokens().contains(token);
 }
 
-inline ExceptionOr<void> DOMTokenList::addInternal(const AtomString* newTokens, size_t length)
+inline ExceptionOr<void> DOMTokenList::addInternal(std::span<const AtomString> newTokens)
 {
     // This is usually called with a single token.
     Vector<AtomString, 1> uniqueNewTokens;
-    uniqueNewTokens.reserveInitialCapacity(length);
+    uniqueNewTokens.reserveInitialCapacity(newTokens.size());
 
     auto& tokens = this->tokens();
 
-    for (size_t i = 0; i < length; ++i) {
-        auto result = validateToken(newTokens[i]);
+    for (auto& newToken : newTokens) {
+        auto result = validateToken(newToken);
         if (result.hasException())
             return result;
-        if (!tokens.contains(newTokens[i]) && !uniqueNewTokens.contains(newTokens[i]))
-            uniqueNewTokens.append(newTokens[i]);
+        if (!tokens.contains(newToken) && !uniqueNewTokens.contains(newToken))
+            uniqueNewTokens.append(newToken);
     }
 
     if (!uniqueNewTokens.isEmpty())
@@ -98,23 +101,23 @@ inline ExceptionOr<void> DOMTokenList::addInternal(const AtomString* newTokens,
 
 ExceptionOr<void> DOMTokenList::add(const FixedVector<AtomString>& tokens)
 {
-    return addInternal(tokens.data(), tokens.size());
+    return addInternal(tokens);
 }
 
 ExceptionOr<void> DOMTokenList::add(const AtomString& token)
 {
-    return addInternal(&token, 1);
+    return addInternal({ &token, 1 });
 }
 
-inline ExceptionOr<void> DOMTokenList::removeInternal(const AtomString* tokensToRemove, size_t length)
+inline ExceptionOr<void> DOMTokenList::removeInternal(std::span<const AtomString> tokensToRemove)
 {
-    auto result = validateTokens(tokensToRemove, length);
+    auto result = validateTokens(tokensToRemove);
     if (result.hasException())
         return result;
 
     auto& tokens = this->tokens();
-    for (size_t i = 0; i < length; ++i)
-        tokens.removeFirst(tokensToRemove[i]);
+    for (auto& tokenToRemove : tokensToRemove)
+        tokens.removeFirst(tokenToRemove);
 
     updateAssociatedAttributeFromTokens();
 
@@ -123,12 +126,12 @@ inline ExceptionOr<void> DOMTokenList::removeInternal(const AtomString* tokensTo
 
 ExceptionOr<void> DOMTokenList::remove(const FixedVector<AtomString>& tokens)
 {
-    return removeInternal(tokens.data(), tokens.size());
+    return removeInternal(tokens);
 }
 
 ExceptionOr<void> DOMTokenList::remove(const AtomString& token)
 {
-    return removeInternal(&token, 1);
+    return removeInternal({ &token, 1 });
 }
 
 ExceptionOr<bool> DOMTokenList::toggle(const AtomString& token, std::optional<bool> force)
@@ -205,18 +208,18 @@ ExceptionOr<bool> DOMTokenList::supports(StringView token)
 {
     if (!m_isSupportedToken)
         return Exception { ExceptionCode::TypeError };
-    return m_isSupportedToken(m_element.document(), token);
+    return m_isSupportedToken(m_element->document(), token);
 }
 
 // https://dom.spec.whatwg.org/#dom-domtokenlist-value
 const AtomString& DOMTokenList::value() const
 {
-    return m_element.getAttribute(m_attributeName);
+    return protectedElement()->getAttribute(m_attributeName);
 }
 
 void DOMTokenList::setValue(const AtomString& value)
 {
-    m_element.setAttribute(m_attributeName, value);
+    protectedElement()->setAttribute(m_attributeName, value);
 }
 
 void DOMTokenList::updateTokensFromAttributeValue(const AtomString& value)
@@ -224,7 +227,7 @@ void DOMTokenList::updateTokensFromAttributeValue(const AtomString& value)
     // Clear tokens but not capacity.
     m_tokens.shrink(0);
 
-    HashSet<AtomString> addedTokens;
+    UncheckedKeyHashSet<AtomString> addedTokens;
     // https://dom.spec.whatwg.org/#ordered%20sets
     for (unsigned start = 0; ; ) {
         while (start < value.length() && isASCIIWhitespace(value[start]))
@@ -254,32 +257,24 @@ void DOMTokenList::updateTokensFromAttributeValue(const AtomString& value)
     m_tokensNeedUpdating = false;
 }
 
-void DOMTokenList::associatedAttributeValueChanged(const AtomString&)
-{
-    // Do not reset the DOMTokenList value if the attribute value was changed by us.
-    if (m_inUpdateAssociatedAttributeFromTokens)
-        return;
-
-    m_tokensNeedUpdating = true;
-}
-
 // https://dom.spec.whatwg.org/#concept-dtl-update
 void DOMTokenList::updateAssociatedAttributeFromTokens()
 {
     ASSERT(!m_tokensNeedUpdating);
 
-    if (m_tokens.isEmpty() && !m_element.hasAttribute(m_attributeName))
+    Ref element = m_element.get();
+    if (m_tokens.isEmpty() && !element->hasAttribute(m_attributeName))
         return;
 
     if (m_tokens.isEmpty()) {
-        m_element.setAttribute(m_attributeName, emptyAtom());
+        element->setAttribute(m_attributeName, emptyAtom());
         return;
     }
 
     bool wholeAttributeIsSingleToken = m_tokens.size() == 1;
     if (wholeAttributeIsSingleToken) {
         SetForScope inAttributeUpdate(m_inUpdateAssociatedAttributeFromTokens, true);
-        m_element.setAttribute(m_attributeName, m_tokens[0]);
+        element->setAttribute(m_attributeName, m_tokens[0]);
         return;
     }
 
@@ -293,13 +288,13 @@ void DOMTokenList::updateAssociatedAttributeFromTokens()
     AtomString serializedValue = builder.toAtomString();
 
     SetForScope inAttributeUpdate(m_inUpdateAssociatedAttributeFromTokens, true);
-    m_element.setAttribute(m_attributeName, serializedValue);
+    element->setAttribute(m_attributeName, serializedValue);
 }
 
 Vector<AtomString, 1>& DOMTokenList::tokens()
 {
     if (m_tokensNeedUpdating)
-        updateTokensFromAttributeValue(m_element.getAttribute(m_attributeName));
+        updateTokensFromAttributeValue(protectedElement()->getAttribute(m_attributeName));
     ASSERT(!m_tokensNeedUpdating);
     return m_tokens;
 }
