@@ -38,7 +38,8 @@ class HTMLMediaElement;
 class LoadableTextTrack;
 
 class HTMLTrackElement final : public HTMLElement, public ActiveDOMObject, public TextTrackClient {
-    WTF_MAKE_ISO_ALLOCATED(HTMLTrackElement);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(HTMLTrackElement);
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(HTMLTrackElement);
 public:
     static Ref<HTMLTrackElement> create(const QualifiedName&, Document&);
 
@@ -48,6 +49,8 @@ public:
 
     using HTMLElement::scriptExecutionContext;
 
+    USING_CAN_MAKE_WEAKPTR(HTMLElement);
+
     const AtomString& kind();
     void setKind(const AtomString&);
 
