@@ -32,39 +32,38 @@
 #include "config.h"
 #include "HiddenInputType.h"
 
+#include "DOMFormData.h"
+#include "ElementInlines.h"
 #include "FormController.h"
-#include "FormDataList.h"
 #include "HTMLInputElement.h"
 #include "HTMLNames.h"
 #include "InputTypeNames.h"
 #include "RenderElement.h"
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(HiddenInputType);
+
 using namespace HTMLNames;
 
-const AtomicString& HiddenInputType::formControlType() const
+const AtomString& HiddenInputType::formControlType() const
 {
     return InputTypeNames::hidden();
 }
 
 FormControlState HiddenInputType::saveFormControlState() const
 {
-    // valueAttributeWasUpdatedAfterParsing() never be true for form
-    // controls create by createElement() or cloneNode(). It's ok for
-    // now because we restore values only to form controls created by
-    // parsing.
-    return element().valueAttributeWasUpdatedAfterParsing() ? FormControlState(element().value()) : FormControlState();
+    // valueAttributeWasUpdatedAfterParsing() never be true for form controls create by createElement() or cloneNode().
+    // It's OK for now because we restore values only to form controls created by parsing.
+    ASSERT(element());
+    return element()->valueAttributeWasUpdatedAfterParsing() ? FormControlState { { AtomString { element()->value() } } } : FormControlState { };
 }
 
 void HiddenInputType::restoreFormControlState(const FormControlState& state)
 {
-    element().setAttribute(valueAttr, state[0]);
-}
-
-bool HiddenInputType::supportsValidation() const
-{
-    return false;
+    ASSERT(element());
+    element()->setAttributeWithoutSynchronization(valueAttr, AtomString { state[0] });
 }
 
 RenderPtr<RenderElement> HiddenInputType::createInputRenderer(RenderStyle&&)
@@ -73,8 +72,9 @@ RenderPtr<RenderElement> HiddenInputType::createInputRenderer(RenderStyle&&)
     return nullptr;
 }
 
-void HiddenInputType::accessKeyAction(bool)
+bool HiddenInputType::accessKeyAction(bool)
 {
+    return false;
 }
 
 bool HiddenInputType::rendererIsNeeded()
@@ -87,23 +87,25 @@ bool HiddenInputType::storesValueSeparateFromAttribute()
     return false;
 }
 
-void HiddenInputType::setValue(const String& sanitizedValue, bool, TextFieldEventBehavior)
+void HiddenInputType::setValue(const String& sanitizedValue, bool, TextFieldEventBehavior, TextControlSetValueSelection)
 {
-    element().setAttribute(valueAttr, sanitizedValue);
+    ASSERT(element());
+    element()->setAttributeWithoutSynchronization(valueAttr, AtomString { sanitizedValue });
 }
 
-bool HiddenInputType::isHiddenType() const
+bool HiddenInputType::appendFormData(DOMFormData& formData) const
 {
-    return true;
-}
+    ASSERT(element());
+    auto name = element()->name();
 
-bool HiddenInputType::appendFormData(FormDataList& encoding, bool isMultipartForm) const
-{
-    if (equalIgnoringASCIICase(element().name(), "_charset_")) {
-        encoding.appendData(element().name(), String(encoding.encoding().name()));
+    if (equalIgnoringASCIICase(name, "_charset_"_s)) {
+        formData.append(name, String::fromLatin1(formData.encoding().name()));
         return true;
     }
-    return InputType::appendFormData(encoding, isMultipartForm);
+    InputType::appendFormData(formData);
+    if (auto& dirname = element()->attributeWithoutSynchronization(dirnameAttr); !dirname.isNull())
+        formData.append(dirname, element()->directionForFormData());
+    return true;
 }
 
 bool HiddenInputType::shouldRespectHeightAndWidthAttributes()
@@ -111,4 +113,9 @@ bool HiddenInputType::shouldRespectHeightAndWidthAttributes()
     return true;
 }
 
+bool HiddenInputType::dirAutoUsesValue() const
+{
+    return true;
+}
+
 } // namespace WebCore
