@@ -192,7 +192,7 @@ ExceptionOr<Ref<TextTrackCue>> TextTrackCue::create(Document& document, double s
         if (result.hasException())
             return result.releaseException();
     }
-    cueFragment.cloneChildNodes(document, fragment);
+    cueFragment.cloneChildNodes(document, nullptr, fragment);
 
     OptionSet<RequiredNodes> nodeTypes = { };
     for (Node* node = fragment->firstChild(); node; node = node->nextSibling()) {
@@ -452,7 +452,7 @@ RefPtr<DocumentFragment> TextTrackCue::getCueAsHTML()
         return nullptr;
 
     auto clonedFragment = DocumentFragment::create(*document);
-    m_cueNode->cloneChildNodes(*document, clonedFragment);
+    m_cueNode->cloneChildNodes(*document, nullptr, clonedFragment);
 
     for (Node* node = clonedFragment->firstChild(); node; node = node->nextSibling())
         removeUserAgentPartAttributes(*node);
@@ -513,7 +513,7 @@ void TextTrackCue::rebuildDisplayTree()
 
     m_displayTree->removeChildren();
     auto clonedFragment = DocumentFragment::create(*document);
-    m_cueNode->cloneChildNodes(*document, clonedFragment);
+    m_cueNode->cloneChildNodes(*document, nullptr, clonedFragment);
     m_displayTree->appendChild(clonedFragment);
 
     if (m_fontSize) {
