@@ -26,25 +26,30 @@
 #include "config.h"
 #include "CookieJar.h"
 
+#include "Cookie.h"
 #include "CookieRequestHeaderFieldProxy.h"
-#include "Document.h"
+#include "CookieStoreGetOptions.h"
+#include "DocumentInlines.h"
 #include "DocumentLoader.h"
 #include "FrameDestructionObserverInlines.h"
 #include "FrameLoader.h"
 #include "HTTPCookieAcceptPolicy.h"
 #include "LocalFrame.h"
+#include "LocalFrameLoaderClient.h"
 #include "NetworkStorageSession.h"
 #include "NetworkingContext.h"
 #include "Page.h"
 #include "PlatformStrategies.h"
 #include "StorageSessionProvider.h"
+#include <optional>
+#include <wtf/CompletionHandler.h>
 #include <wtf/SystemTracing.h>
 
 namespace WebCore {
 
 static ShouldRelaxThirdPartyCookieBlocking shouldRelaxThirdPartyCookieBlocking(const Document& document)
 {
-    if (auto* page = document.page())
+    if (RefPtr page = document.page())
         return page->shouldRelaxThirdPartyCookieBlocking();
     return ShouldRelaxThirdPartyCookieBlocking::No;
 }
@@ -61,7 +66,13 @@ IncludeSecureCookies CookieJar::shouldIncludeSecureCookies(const Document& docum
 
 SameSiteInfo CookieJar::sameSiteInfo(const Document& document, IsForDOMCookieAccess isAccessForDOM)
 {
-    if (auto* loader = document.loader())
+    RefPtr frame = document.frame();
+    if (frame && frame->loader().client().isRemoteWorkerFrameLoaderClient()) {
+        RegistrableDomain domain(document.securityOrigin().data());
+        return { domain.matches(document.firstPartyForCookies()), false, true };
+    }
+
+    if (RefPtr loader = document.loader())
         return SameSiteInfo::create(loader->request(), isAccessForDOM);
     return { };
 }
@@ -82,10 +93,10 @@ String CookieJar::cookies(Document& document, const URL& url) const
     auto pageID = document.pageID();
     std::optional<FrameIdentifier> frameID;
     if (auto* frame = document.frame())
-        frameID = frame->loader().frameID();
+        frameID = frame->frameID();
 
     std::pair<String, bool> result;
-    if (auto* session = m_storageSessionProvider->storageSession())
+    if (CheckedPtr session = protectedStorageSessionProvider()->storageSession())
         result = session->cookiesForDOM(document.firstPartyForCookies(), sameSiteInfo(document, IsForDOMCookieAccess::Yes), url, frameID, pageID, includeSecureCookies, ApplyTrackingPrevention::Yes, shouldRelaxThirdPartyCookieBlocking(document));
     else
         ASSERT_NOT_REACHED();
@@ -103,7 +114,7 @@ CookieRequestHeaderFieldProxy CookieJar::cookieRequestHeaderFieldProxy(const Doc
     auto pageID = document.pageID();
     std::optional<FrameIdentifier> frameID;
     if (auto* frame = document.frame())
-        frameID = frame->loader().frameID();
+        frameID = frame->frameID();
 
     return { document.firstPartyForCookies(), sameSiteInfo(document), url, frameID, pageID, shouldIncludeSecureCookies(document, url) };
 }
@@ -113,26 +124,40 @@ void CookieJar::setCookies(Document& document, const URL& url, const String& coo
     auto pageID = document.pageID();
     std::optional<FrameIdentifier> frameID;
     if (auto* frame = document.frame())
-        frameID = frame->loader().frameID();
+        frameID = frame->frameID();
 
-    if (auto* session = m_storageSessionProvider->storageSession())
-        session->setCookiesFromDOM(document.firstPartyForCookies(), sameSiteInfo(document, IsForDOMCookieAccess::Yes), url, frameID, pageID, ApplyTrackingPrevention::Yes, cookieString, shouldRelaxThirdPartyCookieBlocking(document));
+    if (CheckedPtr session = protectedStorageSessionProvider()->storageSession())
+        session->setCookiesFromDOM(document.firstPartyForCookies(), sameSiteInfo(document, IsForDOMCookieAccess::Yes), url, frameID, pageID, ApplyTrackingPrevention::Yes, RequiresScriptTelemetry::No, cookieString, shouldRelaxThirdPartyCookieBlocking(document));
     else
         ASSERT_NOT_REACHED();
 }
 
-bool CookieJar::cookiesEnabled(const Document&) const
+bool CookieJar::cookiesEnabled(Document& document)
 {
-    if (auto* session = m_storageSessionProvider->storageSession())
-        return session->cookieAcceptPolicy() != HTTPCookieAcceptPolicy::Never;
+    auto cookieURL = document.cookieURL();
+    if (cookieURL.isEmpty())
+        return false;
+
+    auto pageID = document.pageID();
+    std::optional<FrameIdentifier> frameID;
+    if (auto* frame = document.frame())
+        frameID = frame->frameID();
+
+    if (CheckedPtr session = protectedStorageSessionProvider()->storageSession())
+        return session->cookiesEnabled(document.firstPartyForCookies(), cookieURL, frameID, pageID, shouldRelaxThirdPartyCookieBlocking(document));
 
     ASSERT_NOT_REACHED();
     return false;
 }
 
+void CookieJar::remoteCookiesEnabled(const Document&, CompletionHandler<void(bool)>&& completionHandler) const
+{
+    completionHandler(false);
+}
+
 std::pair<String, SecureCookiesAccessed> CookieJar::cookieRequestHeaderFieldValue(const URL& firstParty, const SameSiteInfo& sameSiteInfo, const URL& url, std::optional<FrameIdentifier> frameID, std::optional<PageIdentifier> pageID, IncludeSecureCookies includeSecureCookies) const
 {
-    if (auto* session = m_storageSessionProvider->storageSession()) {
+    if (CheckedPtr session = protectedStorageSessionProvider()->storageSession()) {
         std::pair<String, bool> result = session->cookieRequestHeaderFieldValue(firstParty, sameSiteInfo, url, frameID, pageID, includeSecureCookies, ApplyTrackingPrevention::Yes, ShouldRelaxThirdPartyCookieBlocking::No);
         return { result.first, result.second ? SecureCookiesAccessed::Yes : SecureCookiesAccessed::No };
     }
@@ -146,7 +171,7 @@ String CookieJar::cookieRequestHeaderFieldValue(Document& document, const URL& u
     auto pageID = document.pageID();
     std::optional<FrameIdentifier> frameID;
     if (auto* frame = document.frame())
-        frameID = frame->loader().frameID();
+        frameID = frame->frameID();
 
     auto result = cookieRequestHeaderFieldValue(document.firstPartyForCookies(), sameSiteInfo(document), url, frameID, pageID, shouldIncludeSecureCookies(document, url));
     if (result.second == SecureCookiesAccessed::Yes)
@@ -154,36 +179,61 @@ String CookieJar::cookieRequestHeaderFieldValue(Document& document, const URL& u
     return result.first;
 }
 
-bool CookieJar::getRawCookies(const Document& document, const URL& url, Vector<Cookie>& cookies) const
+bool CookieJar::getRawCookies(Document& document, const URL& url, Vector<Cookie>& cookies) const
 {
     auto pageID = document.pageID();
     std::optional<FrameIdentifier> frameID;
     if (auto* frame = document.frame())
-        frameID = frame->loader().frameID();
+        frameID = frame->frameID();
 
-    if (auto* session = m_storageSessionProvider->storageSession())
+    if (CheckedPtr session = protectedStorageSessionProvider()->storageSession())
         return session->getRawCookies(document.firstPartyForCookies(), sameSiteInfo(document), url, frameID, pageID, ApplyTrackingPrevention::Yes, shouldRelaxThirdPartyCookieBlocking(document), cookies);
 
     ASSERT_NOT_REACHED();
     return false;
 }
 
-void CookieJar::setRawCookie(const Document&, const Cookie& cookie)
+void CookieJar::setRawCookie(const Document&, const Cookie& cookie, ShouldPartitionCookie)
 {
-    if (auto* session = m_storageSessionProvider->storageSession())
+    if (CheckedPtr session = protectedStorageSessionProvider()->storageSession())
         session->setCookie(cookie);
     else
         ASSERT_NOT_REACHED();
 }
 
-void CookieJar::deleteCookie(const Document&, const URL& url, const String& cookieName, CompletionHandler<void()>&& completionHandler)
+void CookieJar::deleteCookie(const Document& document, const URL& url, const String& cookieName, CompletionHandler<void()>&& completionHandler)
 {
-    if (auto* session = m_storageSessionProvider->storageSession())
-        session->deleteCookie(url, cookieName, WTFMove(completionHandler));
+    if (CheckedPtr session = protectedStorageSessionProvider()->storageSession())
+        session->deleteCookie(document.firstPartyForCookies(), url, cookieName, WTFMove(completionHandler));
     else {
         ASSERT_NOT_REACHED();
         completionHandler();
     }
 }
 
+void CookieJar::getCookiesAsync(Document&, const URL&, const CookieStoreGetOptions&, CompletionHandler<void(std::optional<Vector<Cookie>>&&)>&& completionHandler) const
+{
+    completionHandler(std::nullopt);
 }
+
+void CookieJar::setCookieAsync(Document&, const URL&, const Cookie&, CompletionHandler<void(bool)>&& completionHandler) const
+{
+    completionHandler(false);
+}
+
+Ref<StorageSessionProvider> CookieJar::protectedStorageSessionProvider() const
+{
+    return m_storageSessionProvider;
+}
+
+#if HAVE(COOKIE_CHANGE_LISTENER_API)
+void CookieJar::addChangeListener(const WebCore::Document&, const CookieChangeListener&)
+{
+}
+
+void CookieJar::removeChangeListener(const String&, const CookieChangeListener&)
+{
+}
+#endif
+
+} // namespace WebCore
