@@ -29,7 +29,7 @@
 #include "config.h"
 #include "NavigationAction.h"
 
-#include "Document.h"
+#include "DocumentInlines.h"
 #include "FrameLoader.h"
 #include "HistoryItem.h"
 #include "LocalFrame.h"
@@ -68,7 +68,7 @@ NavigationAction& NavigationAction::operator=(NavigationAction&&) = default;
 
 static bool shouldTreatAsSameOriginNavigation(const Document& document, const URL& url)
 {
-    return url.protocolIsAbout() || url.protocolIsData() || (url.protocolIsBlob() && document.securityOrigin().canRequest(url, OriginAccessPatternsForWebProcess::singleton()));
+    return url.protocolIsAbout() || url.protocolIsData() || (url.protocolIsBlob() && document.protectedSecurityOrigin()->canRequest(url, OriginAccessPatternsForWebProcess::singleton()));
 }
 
 static std::optional<NavigationAction::UIEventWithKeyStateData> keyStateDataForFirstEventWithKeyState(Event* event)
@@ -137,12 +137,12 @@ NavigationAction NavigationAction::copyWithShouldOpenExternalURLsPolicy(ShouldOp
 
 void NavigationAction::setTargetBackForwardItem(HistoryItem& item)
 {
-    m_targetBackForwardItemIdentifier = item.identifier();
+    m_targetBackForwardItemIdentifier = item.itemID();
 }
 
 void NavigationAction::setSourceBackForwardItem(HistoryItem* item)
 {
-    m_sourceBackForwardItemIdentifier = item ? std::make_optional(item->identifier()) : std::nullopt;
+    m_sourceBackForwardItemIdentifier = item ? std::make_optional(item->itemID()) : std::nullopt;
 }
 
 }
