@@ -40,8 +40,8 @@
 #include <wtf/URL.h>
 #include <wtf/UUID.h>
 #include <wtf/text/CString.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/StringBuilder.h>
-#include <wtf/text/StringConcatenateNumbers.h>
 
 namespace WebCore {
 
@@ -87,8 +87,8 @@ static unsigned urlHostHash(const URL& url)
 {
     StringView host = url.host();
     if (host.is8Bit())
-        return AlreadyHashed::avoidDeletedValue(StringHasher::computeHashAndMaskTop8Bits(host.characters8(), host.length()));
-    return AlreadyHashed::avoidDeletedValue(StringHasher::computeHashAndMaskTop8Bits(host.characters16(), host.length()));
+        return AlreadyHashed::avoidDeletedValue(StringHasher::computeHashAndMaskTop8Bits(host.span8()));
+    return AlreadyHashed::avoidDeletedValue(StringHasher::computeHashAndMaskTop8Bits(host.span16()));
 }
 
 ApplicationCacheGroup* ApplicationCacheStorage::loadCacheGroup(const URL& manifestURL)
@@ -567,7 +567,7 @@ void ApplicationCacheStorage::verifySchemaVersion()
     SQLiteTransaction setDatabaseVersion(m_database);
     setDatabaseVersion.begin();
 
-    auto statement = m_database.prepareStatementSlow(makeString("PRAGMA user_version=", schemaVersion));
+    auto statement = m_database.prepareStatementSlow(makeString("PRAGMA user_version="_s, schemaVersion));
     if (!statement)
         return;
     
@@ -815,7 +815,7 @@ bool ApplicationCacheStorage::store(ApplicationCacheResource* resource, unsigned
     } else {
         if (resource->data().size()) {
             auto contiguousData = resource->data().makeContiguous();
-            dataStatement->bindBlob(1, contiguousData->dataAsSpanForContiguousData());
+            dataStatement->bindBlob(1, contiguousData->span());
         }
     }
     
@@ -1054,15 +1054,15 @@ bool ApplicationCacheStorage::storeNewestCache(ApplicationCacheGroup& group)
 }
 
 template<typename CharacterType>
-static inline void parseHeader(const CharacterType* header, unsigned headerLength, ResourceResponse& response)
+static inline void parseHeader(std::span<const CharacterType> header, ResourceResponse& response)
 {
-    ASSERT(WTF::find(header, headerLength, ':') != notFound);
-    unsigned colonPosition = WTF::find(header, headerLength, ':');
+    auto colonPosition = WTF::find(header, ':');
+    ASSERT(colonPosition != notFound);
 
     // Save memory by putting the header names into atom strings so each is stored only once,
     // even though the setHTTPHeaderField function does not require an atom string.
-    AtomString headerName { header, colonPosition };
-    String headerValue { header + colonPosition + 1, headerLength - colonPosition - 1 };
+    AtomString headerName { header.first(colonPosition) };
+    String headerValue(header.subspan(colonPosition + 1));
 
     response.setHTTPHeaderField(headerName, headerValue);
 }
@@ -1075,18 +1075,18 @@ static inline void parseHeaders(const String& headers, ResourceResponse& respons
         ASSERT(startPos != endPos);
 
         if (headers.is8Bit())
-            parseHeader(headers.characters8() + startPos, endPos - startPos, response);
+            parseHeader(headers.span8().subspan(startPos, endPos - startPos), response);
         else
-            parseHeader(headers.characters16() + startPos, endPos - startPos, response);
+            parseHeader(headers.span16().subspan(startPos, endPos - startPos), response);
         
         startPos = endPos + 1;
     }
     
     if (startPos != headers.length()) {
         if (headers.is8Bit())
-            parseHeader(headers.characters8(), headers.length(), response);
+            parseHeader(headers.span8(), response);
         else
-            parseHeader(headers.characters16(), headers.length(), response);
+            parseHeader(headers.span16(), response);
     }
 }
     
@@ -1279,7 +1279,7 @@ bool ApplicationCacheStorage::writeDataToUniqueFileInDirectory(FragmentedSharedB
     String fullPath;
     
     do {
-        path = makeString(UUID::createVersion4(), fileExtension);
+        path = makeString(WTF::UUID::createVersion4(), fileExtension);
         if (path.isEmpty())
             return false;
         
@@ -1291,8 +1291,8 @@ bool ApplicationCacheStorage::writeDataToUniqueFileInDirectory(FragmentedSharedB
         return false;
     
     int64_t writtenBytes = 0;
-    data.forEachSegment([&](auto& segment) {
-        writtenBytes += FileSystem::writeToFile(handle, segment.data(), segment.size());
+    data.forEachSegment([&](auto segment) {
+        writtenBytes += FileSystem::writeToFile(handle, segment);
     });
     FileSystem::closeFile(handle);
     
@@ -1457,7 +1457,7 @@ long long ApplicationCacheStorage::flatFileAreaSize()
     return totalSize;
 }
 
-HashSet<SecurityOriginData> ApplicationCacheStorage::originsWithCache()
+UncheckedKeyHashSet<SecurityOriginData> ApplicationCacheStorage::originsWithCache()
 {
     auto urls = manifestURLs();
     if (!urls)
@@ -1465,7 +1465,7 @@ HashSet<SecurityOriginData> ApplicationCacheStorage::originsWithCache()
 
     // Multiple manifest URLs might share the same SecurityOrigin, so we might be creating extra, wasted origins here.
     // The current schema doesn't allow for a more efficient way of building this list.
-    HashSet<SecurityOriginData> origins;
+    UncheckedKeyHashSet<SecurityOriginData> origins;
     for (auto& url : *urls)
         origins.add(SecurityOriginData::fromURL(url));
     return origins;
