@@ -27,8 +27,9 @@
 
 #include "FloatPoint.h"
 #include "FloatSize.h"
+#include "ProcessQualified.h"
 #include "RectEdges.h"
-#include <wtf/EnumTraits.h>
+#include "ScrollingNodeID.h"
 
 namespace WTF {
 class TextStream;
@@ -38,6 +39,8 @@ namespace WebCore {
 
 class IntPoint;
 
+enum class ScrollBehavior : uint8_t;
+
 // scrollPosition is in content coordinates (0,0 is at scrollOrigin), so may have negative components.
 using ScrollPosition = IntPoint;
 // scrollOffset() is the value used by scrollbars (min is 0,0), and should never have negative components.
@@ -54,7 +57,7 @@ enum class OverscrollBehavior : uint8_t {
     None
 };
 
-enum ScrollDirection : uint8_t {
+enum class ScrollDirection : uint8_t {
     ScrollUp,
     ScrollDown,
     ScrollLeft,
@@ -86,53 +89,53 @@ inline ScrollDirection logicalToPhysical(ScrollLogicalDirection direction, bool
     case ScrollBlockDirectionBackward: {
         if (isVertical) {
             if (!isFlipped)
-                return ScrollUp;
-            return ScrollDown;
+                return ScrollDirection::ScrollUp;
+            return ScrollDirection::ScrollDown;
         } else {
             if (!isFlipped)
-                return ScrollLeft;
-            return ScrollRight;
+                return ScrollDirection::ScrollLeft;
+            return ScrollDirection::ScrollRight;
         }
         break;
     }
     case ScrollBlockDirectionForward: {
         if (isVertical) {
             if (!isFlipped)
-                return ScrollDown;
-            return ScrollUp;
+                return ScrollDirection::ScrollDown;
+            return ScrollDirection::ScrollUp;
         } else {
             if (!isFlipped)
-                return ScrollRight;
-            return ScrollLeft;
+                return ScrollDirection::ScrollRight;
+            return ScrollDirection::ScrollLeft;
         }
         break;
     }
     case ScrollInlineDirectionBackward: {
         if (isVertical) {
             if (!isFlipped)
-                return ScrollLeft;
-            return ScrollRight;
+                return ScrollDirection::ScrollLeft;
+            return ScrollDirection::ScrollRight;
         } else {
             if (!isFlipped)
-                return ScrollUp;
-            return ScrollDown;
+                return ScrollDirection::ScrollUp;
+            return ScrollDirection::ScrollDown;
         }
         break;
     }
     case ScrollInlineDirectionForward: {
         if (isVertical) {
             if (!isFlipped)
-                return ScrollRight;
-            return ScrollLeft;
+                return ScrollDirection::ScrollRight;
+            return ScrollDirection::ScrollLeft;
         } else {
             if (!isFlipped)
-                return ScrollDown;
-            return ScrollUp;
+                return ScrollDirection::ScrollDown;
+            return ScrollDirection::ScrollUp;
         }
         break;
     }
     }
-    return ScrollUp;
+    return ScrollDirection::ScrollUp;
 }
 
 enum class ScrollGranularity : uint8_t {
@@ -148,6 +151,13 @@ enum class ScrollElasticity : uint8_t {
     Allowed
 };
 
+// Determines if rubber-banding should be enabled if the content size is less than the scroll view size.
+enum class RubberBandingBehavior : uint8_t {
+    Always,
+    Never,
+    BasedOnSize
+};
+
 enum class ScrollbarOrientation : uint8_t {
     Horizontal,
     Vertical
@@ -159,11 +169,6 @@ enum class ScrollbarMode : uint8_t {
     AlwaysOn
 };
 
-enum class ScrollbarControlSize : uint8_t {
-    Regular,
-    Small
-};
-
 enum class ScrollbarExpansionState : uint8_t {
     Regular,
     Expanded
@@ -189,10 +194,10 @@ enum class ScrollEventAxis : uint8_t {
 inline constexpr ScrollEventAxis axisFromDirection(ScrollDirection direction)
 {
     switch (direction) {
-    case ScrollUp: return ScrollEventAxis::Vertical;
-    case ScrollDown: return ScrollEventAxis::Vertical;
-    case ScrollLeft: return ScrollEventAxis::Horizontal;
-    case ScrollRight: return ScrollEventAxis::Horizontal;
+    case ScrollDirection::ScrollUp: return ScrollEventAxis::Vertical;
+    case ScrollDirection::ScrollDown: return ScrollEventAxis::Vertical;
+    case ScrollDirection::ScrollLeft: return ScrollEventAxis::Horizontal;
+    case ScrollDirection::ScrollRight: return ScrollEventAxis::Horizontal;
     }
     ASSERT_NOT_REACHED();
     return ScrollEventAxis::Vertical;
@@ -262,6 +267,11 @@ inline BoxSide boxSideForDirection(ScrollDirection direction)
     return BoxSide::Top;
 }
 
+enum class OverlayScrollbarSizeRelevancy : bool {
+    IgnoreOverlayScrollbarSize,
+    IncludeOverlayScrollbarSize
+};
+
 enum ScrollbarControlStateMask {
     ActiveScrollbarState = 1,
     EnabledScrollbarState = 1 << 1,
@@ -312,7 +322,7 @@ enum class ScrollClamping : bool {
     Clamped
 };
 
-enum ScrollBehaviorForFixedElements : bool {
+enum class ScrollBehaviorForFixedElements : bool {
     StickToDocumentBounds,
     StickToViewportBounds
 };
@@ -348,33 +358,22 @@ enum class ScrollSnapPointSelectionMethod : uint8_t {
 
 using ScrollbarControlState = unsigned;
 using ScrollbarControlPartMask = unsigned;
-using ScrollingNodeID = uint64_t;
-
-WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollType);
-WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollClamping);
-WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollBehaviorForFixedElements);
-WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollElasticity);
-WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollbarMode);
-WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, OverflowAnchor);
-WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollDirection);
-WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollGranularity);
-WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, NativeScrollbarVisibility);
-WTF::TextStream& operator<<(WTF::TextStream&, ScrollbarWidth);
 
 struct ScrollPositionChangeOptions {
     ScrollType type;
     ScrollClamping clamping = ScrollClamping::Clamped;
     ScrollIsAnimated animated = ScrollIsAnimated::No;
     ScrollSnapPointSelectionMethod snapPointSelectionMethod = ScrollSnapPointSelectionMethod::Closest;
+    std::optional<FloatSize> originalScrollDelta = std::nullopt;
 
     static ScrollPositionChangeOptions createProgrammatic()
     {
         return { ScrollType::Programmatic };
     }
 
-    static ScrollPositionChangeOptions createProgrammaticWithOptions(ScrollClamping clamping, ScrollIsAnimated animated, ScrollSnapPointSelectionMethod snapPointSelectionMethod)
+    static ScrollPositionChangeOptions createProgrammaticWithOptions(ScrollClamping clamping, ScrollIsAnimated animated, ScrollSnapPointSelectionMethod snapPointSelectionMethod, std::optional<FloatSize> originalScrollDelta = std::nullopt)
     {
-        return { ScrollType::Programmatic, clamping, animated, snapPointSelectionMethod };
+        return { ScrollType::Programmatic, clamping, animated, snapPointSelectionMethod, originalScrollDelta };
     }
 
     static ScrollPositionChangeOptions createUser()
@@ -388,43 +387,20 @@ struct ScrollPositionChangeOptions {
     }
 };
 
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollType);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollClamping);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollBehaviorForFixedElements);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollBehavior);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollElasticity);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, RubberBandingBehavior);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollbarMode);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, OverflowAnchor);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollDirection);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollGranularity);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, NativeScrollbarVisibility);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollPositionChangeOptions);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollSnapPointSelectionMethod);
+WEBCORE_EXPORT WTF::TextStream& operator<<(WTF::TextStream&, ScrollbarWidth);
+
 } // namespace WebCore
 
-namespace WTF {
-
-template<> struct EnumTraits<WebCore::ScrollIsAnimated> {
-    using values = EnumValues<
-        WebCore::ScrollIsAnimated,
-        WebCore::ScrollIsAnimated::No,
-        WebCore::ScrollIsAnimated::Yes
-    >;
-};
-
-template<> struct EnumTraits<WebCore::ScrollGranularity> {
-    using values = EnumValues<
-        WebCore::ScrollGranularity,
-        WebCore::ScrollGranularity::Line,
-        WebCore::ScrollGranularity::Page,
-        WebCore::ScrollGranularity::Document,
-        WebCore::ScrollGranularity::Pixel
-    >;
-};
-
-template<> struct EnumTraits<WebCore::ScrollDirection> {
-    using values = EnumValues<
-        WebCore::ScrollDirection,
-        WebCore::ScrollDirection::ScrollUp,
-        WebCore::ScrollDirection::ScrollDown,
-        WebCore::ScrollDirection::ScrollLeft,
-        WebCore::ScrollDirection::ScrollRight
-    >;
-};
-
-template<> struct EnumTraits<WebCore::ScrollbarOrientation> {
-    using values = EnumValues<
-        WebCore::ScrollbarOrientation,
-        WebCore::ScrollbarOrientation::Horizontal,
-        WebCore::ScrollbarOrientation::Vertical
-    >;
-};
-} // namespace WTF
