@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2006 Samuel Weinig (sam.weinig@gmail.com)
- * Copyright (C) 2004, 2005, 2006, 2013 Apple Inc.  All rights reserved.
+ * Copyright (C) 2004-2023 Apple Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,60 +31,40 @@
 #include "FloatRect.h"
 #include "FloatSize.h"
 #include "GraphicsTypes.h"
+#include "ImageAdapter.h"
 #include "ImageOrientation.h"
+#include "ImagePaintingOptions.h"
 #include "ImageTypes.h"
 #include "NativeImage.h"
 #include "Timer.h"
-#include <wtf/Optional.h>
-#include <wtf/RefCounted.h>
+#include <wtf/RefCountedAndCanMakeWeakPtr.h>
 #include <wtf/RefPtr.h>
 #include <wtf/RetainPtr.h>
 #include <wtf/TypeCasts.h>
 #include <wtf/text/WTFString.h>
 
-#if USE(APPKIT)
-OBJC_CLASS NSImage;
-#endif
-
-#if USE(CG)
-struct CGContext;
-#endif
-
-#if PLATFORM(WIN)
-typedef struct tagSIZE SIZE;
-typedef SIZE* LPSIZE;
-typedef struct HBITMAP__ *HBITMAP;
-#endif
-
-#if PLATFORM(GTK)
-typedef struct _GdkPixbuf GdkPixbuf;
-#endif
-
 namespace WebCore {
 
 class AffineTransform;
 class FloatPoint;
 class FloatSize;
 class GraphicsContext;
-class GraphicsContextImpl;
-class SharedBuffer;
-struct ImagePaintingOptions;
+class FragmentedSharedBuffer;
+class ShareableBitmap;
 struct Length;
 
 // This class gets notified when an image creates or destroys decoded frames and when it advances animation frames.
 class ImageObserver;
 
-class Image : public RefCounted<Image> {
+class Image : public RefCountedAndCanMakeWeakPtr<Image> {
+    friend class CachedSubimage;
     friend class GraphicsContext;
-    friend class GraphicsContextImpl;
 public:
     virtual ~Image();
     
-    WEBCORE_EXPORT static Ref<Image> loadPlatformResource(const char* name);
     WEBCORE_EXPORT static RefPtr<Image> create(ImageObserver&);
+    WEBCORE_EXPORT static std::optional<Ref<Image>> create(RefPtr<ShareableBitmap>&&);
     WEBCORE_EXPORT static bool supportsType(const String&);
-    static bool isPDFResource(const String& mimeType, const URL&);
-    static bool isPostScriptResource(const String& mimeType, const URL&);
 
     virtual bool isBitmapImage() const { return false; }
     virtual bool isGeneratedImage() const { return false; }
@@ -92,15 +72,22 @@ public:
     virtual bool isNamedImageGeneratedImage() const { return false; }
     virtual bool isGradientImage() const { return false; }
     virtual bool isSVGImage() const { return false; }
+    virtual bool isSVGImageForContainer() const { return false; }
+    virtual bool isSVGResourceImage() const { return false; }
     virtual bool isPDFDocumentImage() const { return false; }
     virtual bool isCustomPaintImage() const { return false; }
 
+    bool drawsSVGImage() const { return isSVGImage() || isSVGImageForContainer(); }
+
+    virtual unsigned frameCount() const { return 1; }
+
     virtual bool currentFrameKnownToBeOpaque() const = 0;
     virtual bool isAnimated() const { return false; }
 
-    // Derived classes should override this if they can assure that 
-    // the image contains only resources from its own security origin.
-    virtual bool hasSingleSecurityOrigin() const { return false; }
+    // Derived classes should override this if their rendering could leak
+    // cross-origin data (outside of the resource itself, which undergoes
+    // a CORS cross-origin check).
+    virtual bool renderingTaintsOrigin() const { return false; }
 
     WEBCORE_EXPORT static Image& nullImage();
     bool isNull() const { return size().isEmpty(); }
@@ -111,22 +98,29 @@ public:
     virtual bool hasRelativeHeight() const { return false; }
     virtual void computeIntrinsicDimensions(Length& intrinsicWidth, Length& intrinsicHeight, FloatSize& intrinsicRatio);
 
-    virtual FloatSize size(ImageOrientation = ImageOrientation::FromImage) const = 0;
+    virtual FloatSize size(ImageOrientation = ImageOrientation::Orientation::FromImage) const = 0;
+    virtual FloatSize sourceSize(ImageOrientation orientation = ImageOrientation::Orientation::FromImage) const { return size(orientation); }
+    virtual bool hasDensityCorrectedSize() const { return false; }
     FloatRect rect() const { return FloatRect(FloatPoint(), size()); }
     float width() const { return size().width(); }
     float height() const { return size().height(); }
-    virtual Optional<IntPoint> hotSpot() const { return WTF::nullopt; }
+    virtual std::optional<IntPoint> hotSpot() const { return std::nullopt; }
+    virtual ImageOrientation orientation() const { return ImageOrientation::Orientation::FromImage; }
 
-    WEBCORE_EXPORT EncodedDataStatus setData(RefPtr<SharedBuffer>&& data, bool allDataReceived);
+    WEBCORE_EXPORT EncodedDataStatus setData(RefPtr<FragmentedSharedBuffer>&& data, bool allDataReceived);
     virtual EncodedDataStatus dataChanged(bool /*allDataReceived*/) { return EncodedDataStatus::Unknown; }
 
     virtual String uti() const { return String(); } // null string if unknown
     virtual String filenameExtension() const { return String(); } // null string if unknown
+    virtual String accessibilityDescription() const { return String(); } // null string if unknown
 
-    virtual void destroyDecodedData(bool destroyAll = true) = 0;
+    virtual void destroyDecodedData(bool /*destroyAll*/ = true) { }
 
-    SharedBuffer* data() { return m_encodedImageData.get(); }
-    const SharedBuffer* data() const { return m_encodedImageData.get(); }
+    FragmentedSharedBuffer* data() { return m_encodedImageData.get(); }
+    const FragmentedSharedBuffer* data() const { return m_encodedImageData.get(); }
+
+    virtual DestinationColorSpace colorSpace();
+    virtual Headroom headroom() const { return Headroom::None; }
 
     // Animation begins whenever someone draws the image, so startAnimation() is not normally called.
     // It will automatically pause once all observers no longer want to render the image anywhere.
@@ -136,67 +130,69 @@ public:
     virtual void resetAnimation() {}
     virtual bool isAnimating() const { return false; }
     bool animationPending() const { return m_animationStartTimer && m_animationStartTimer->isActive(); }
+    std::optional<bool> allowsAnimation() const { return m_allowsAnimation; }
+    void setAllowsAnimation(std::optional<bool> allowsAnimation) { m_allowsAnimation = allowsAnimation; }
+    static bool systemAllowsAnimationControls() { return gSystemAllowsAnimationControls; }
+    WEBCORE_EXPORT static void setSystemAllowsAnimationControls(bool allowsControls);
 
     // Typically the CachedImage that owns us.
-    ImageObserver* imageObserver() const { return m_imageObserver; }
-    void setImageObserver(ImageObserver* observer) { m_imageObserver = observer; }
+    RefPtr<ImageObserver> imageObserver() const;
+    void setImageObserver(RefPtr<ImageObserver>&&);
+
+    WEBCORE_EXPORT ImageAdapter& adapter();
+    void invalidateAdapter();
+
     URL sourceURL() const;
-    String mimeType() const;
+    WEBCORE_EXPORT String mimeType() const;
     long long expectedContentLength() const;
 
     enum TileRule { StretchTile, RoundTile, SpaceTile, RepeatTile };
 
-    virtual NativeImagePtr nativeImage(const GraphicsContext* = nullptr) { return nullptr; }
-    virtual NativeImagePtr nativeImageOfSize(const IntSize&, const GraphicsContext* = nullptr) { return nullptr; }
-    virtual NativeImagePtr nativeImageForCurrentFrame(const GraphicsContext* = nullptr) { return nullptr; }
+    virtual RefPtr<NativeImage> nativeImage(const DestinationColorSpace& = DestinationColorSpace::SRGB()) { return nullptr; }
+    virtual RefPtr<NativeImage> nativeImageAtIndex(unsigned) { return nativeImage(); }
+    virtual RefPtr<NativeImage> currentNativeImage() { return nativeImage(); }
+    virtual RefPtr<NativeImage> currentPreTransformedNativeImage(ImageOrientation = ImageOrientation::Orientation::FromImage) { return currentNativeImage(); }
 
-    // Accessors for native image formats.
+    virtual void drawPattern(GraphicsContext&, const FloatRect& destRect, const FloatRect& srcRect, const AffineTransform& patternTransform, const FloatPoint& phase, const FloatSize& spacing, ImagePaintingOptions = { });
 
-#if USE(APPKIT)
-    virtual NSImage *nsImage() { return nullptr; }
-    virtual RetainPtr<NSImage> snapshotNSImage() { return nullptr; }
+#if ASSERT_ENABLED
+    virtual bool hasSolidColor() { return false; }
+#endif
+#if ENABLE(QUICKLOOK_FULLSCREEN)
+    virtual bool shouldUseQuickLookForFullscreen() const { return false; }
 #endif
 
-#if PLATFORM(COCOA)
-    virtual CFDataRef tiffRepresentation() { return nullptr; }
-#endif
-
-#if PLATFORM(WIN)
-    virtual bool getHBITMAP(HBITMAP) { return false; }
-    virtual bool getHBITMAPOfSize(HBITMAP, const IntSize*) { return false; }
-#endif
-
-#if PLATFORM(GTK)
-    virtual GdkPixbuf* getGdkPixbuf() { return nullptr; }
-#endif
-
-    virtual void drawPattern(GraphicsContext&, const FloatRect& destRect, const FloatRect& srcRect, const AffineTransform& patternTransform, const FloatPoint& phase, const FloatSize& spacing, const ImagePaintingOptions& = { });
-
-#if !ASSERT_DISABLED
-    virtual bool notSolidColor() { return true; }
+#if ENABLE(SPATIAL_IMAGE_DETECTION)
+    virtual bool isSpatial() const { return false; }
 #endif
 
     virtual void dump(WTF::TextStream&) const;
 
+    WEBCORE_EXPORT RefPtr<ShareableBitmap> toShareableBitmap() const;
+
 protected:
-    Image(ImageObserver* = nullptr);
+    WEBCORE_EXPORT Image(ImageObserver* = nullptr);
 
     static void fillWithSolidColor(GraphicsContext&, const FloatRect& dstRect, const Color&, CompositeOperator);
 
-#if PLATFORM(WIN)
-    virtual void drawFrameMatchingSourceSize(GraphicsContext&, const FloatRect& dstRect, const IntSize& srcSize, CompositeOperator) { }
-#endif
-    virtual ImageDrawResult draw(GraphicsContext&, const FloatRect& dstRect, const FloatRect& srcRect, const ImagePaintingOptions& = { }) = 0;
-    ImageDrawResult drawTiled(GraphicsContext&, const FloatRect& dstRect, const FloatPoint& srcPoint, const FloatSize& tileSize, const FloatSize& spacing, const ImagePaintingOptions& = { });
-    ImageDrawResult drawTiled(GraphicsContext&, const FloatRect& dstRect, const FloatRect& srcRect, const FloatSize& tileScaleFactor, TileRule hRule, TileRule vRule, const ImagePaintingOptions& = { });
+    virtual bool shouldDrawFromCachedSubimage(GraphicsContext&) const { return false; }
+    virtual bool mustDrawFromCachedSubimage(GraphicsContext&) const { return false; }
+    virtual ImageDrawResult draw(GraphicsContext&, const FloatRect& dstRect, const FloatRect& srcRect, ImagePaintingOptions = { }) = 0;
+    ImageDrawResult drawTiled(GraphicsContext&, const FloatRect& dstRect, const FloatPoint& srcPoint, const FloatSize& tileSize, const FloatSize& spacing, ImagePaintingOptions = { });
+    ImageDrawResult drawTiled(GraphicsContext&, const FloatRect& dstRect, const FloatRect& srcRect, const FloatSize& tileScaleFactor, TileRule hRule, TileRule vRule, ImagePaintingOptions = { });
 
     // Supporting tiled drawing
-    virtual Color singlePixelSolidColor() const { return Color(); }
+    virtual std::optional<Color> singlePixelSolidColor() const { return std::nullopt; }
 
 private:
-    RefPtr<SharedBuffer> m_encodedImageData;
-    ImageObserver* m_imageObserver;
+    RefPtr<FragmentedSharedBuffer> m_encodedImageData;
+    WeakPtr<ImageObserver> m_imageObserver;
+    std::unique_ptr<ImageAdapter> m_adapter;
+
+    // A value of true or false will override the default Page::imageAnimationEnabled state.
+    std::optional<bool> m_allowsAnimation { std::nullopt };
     std::unique_ptr<Timer> m_animationStartTimer;
+    static bool gSystemAllowsAnimationControls;
 };
 
 WTF::TextStream& operator<<(WTF::TextStream&, const Image&);
@@ -207,4 +203,3 @@ WTF::TextStream& operator<<(WTF::TextStream&, const Image&);
 SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::ToClassName) \
     static bool isType(const WebCore::Image& image) { return image.is##ToClassName(); } \
 SPECIALIZE_TYPE_TRAITS_END()
-
