@@ -174,6 +174,11 @@ void PathStream::add(PathRoundedRect roundedRect)
     segments().append(roundedRect);
 }
 
+void PathStream::add(PathContinuousRoundedRect continuousRoundedRect)
+{
+    segments().append(continuousRoundedRect);
+}
+
 void PathStream::add(PathCloseSubpath)
 {
     segments().append(PathCloseSubpath { });
@@ -246,6 +251,11 @@ std::optional<PathRoundedRect> PathStream::singleRoundedRect() const
     return singleDataType<PathRoundedRect>();
 }
 
+std::optional<PathContinuousRoundedRect> PathStream::singleContinuousRoundedRect() const
+{
+    return singleDataType<PathContinuousRoundedRect>();
+}
+
 std::optional<PathArc> PathStream::singleArc() const
 {
     return singleDataType<PathArc>();
