@@ -32,8 +32,8 @@
 
 namespace WebCore {
 
-class Frame;
 class Element;
+class LocalFrame;
 class RenderBox;
 class RenderScrollbarPart;
 class RenderStyle;
@@ -41,34 +41,34 @@ class RenderStyle;
 class RenderScrollbar final : public Scrollbar {
 public:
     friend class Scrollbar;
-    static Ref<Scrollbar> createCustomScrollbar(ScrollableArea&, ScrollbarOrientation, Element*, Frame* owningFrame = nullptr);
+    static Ref<Scrollbar> createCustomScrollbar(ScrollableArea&, ScrollbarOrientation, Element*, LocalFrame* owningFrame = nullptr);
     virtual ~RenderScrollbar();
 
     RenderBox* owningRenderer() const;
 
     void paintPart(GraphicsContext&, ScrollbarPart, const IntRect&);
 
-    IntRect buttonRect(ScrollbarPart);
-    IntRect trackRect(int startLength, int endLength);
-    IntRect trackPieceRectWithMargins(ScrollbarPart, const IntRect&);
+    IntRect buttonRect(ScrollbarPart) const;
+    IntRect trackRect(int startLength, int endLength) const;
+    IntRect trackPieceRectWithMargins(ScrollbarPart, const IntRect&) const;
 
-    int minimumThumbLength();
+    int minimumThumbLength() const;
 
-    float opacity();
+    float opacity() const;
     
-    bool isHiddenByStyle() const final;
+    bool isHiddenByStyle() const override;
 
     std::unique_ptr<RenderStyle> getScrollbarPseudoStyle(ScrollbarPart, PseudoId) const;
 
 private:
-    RenderScrollbar(ScrollableArea&, ScrollbarOrientation, Element*, Frame*);
+    RenderScrollbar(ScrollableArea&, ScrollbarOrientation, Element*, LocalFrame*);
 
     bool isOverlayScrollbar() const override { return false; }
 
     void setParent(ScrollView*) override;
     void setEnabled(bool) override;
 
-    void paint(GraphicsContext&, const IntRect& damageRect, Widget::SecurityOriginPaintPolicy, EventRegionContext*) override;
+    void paint(GraphicsContext&, const IntRect& damageRect, Widget::SecurityOriginPaintPolicy, RegionContext*) override;
 
     void setHoveredPart(ScrollbarPart) override;
     void setPressedPart(ScrollbarPart) override;
@@ -85,8 +85,8 @@ private:
     // FrameView which this Element pointer can in no way keep alive. See webkit bug 80610.
     RefPtr<Element> m_ownerElement;
 
-    Frame* m_owningFrame;
-    HashMap<unsigned, RenderPtr<RenderScrollbarPart>> m_parts;
+    WeakPtr<LocalFrame> m_owningFrame;
+    UncheckedKeyHashMap<unsigned, RenderPtr<RenderScrollbarPart>> m_parts;
 };
 
 } // namespace WebCore
