@@ -23,6 +23,8 @@
 #include "StyleInheritedData.h"
 
 #include "RenderStyleInlines.h"
+#include "RenderStyleDifference.h"
+#include "StyleFontData.h"
 
 namespace WebCore {
 
@@ -35,6 +37,7 @@ StyleInheritedData::StyleInheritedData()
 #if ENABLE(TEXT_AUTOSIZING)
     , specifiedLineHeight(RenderStyle::initialLineHeight())
 #endif
+    , fontData(StyleFontData::create())
     , color(RenderStyle::initialColor())
     , visitedLinkColor(RenderStyle::initialColor())
 {
@@ -48,7 +51,7 @@ inline StyleInheritedData::StyleInheritedData(const StyleInheritedData& o)
 #if ENABLE(TEXT_AUTOSIZING)
     , specifiedLineHeight(o.specifiedLineHeight)
 #endif
-    , fontCascade(o.fontCascade)
+    , fontData(o.fontData)
     , color(o.color)
     , visitedLinkColor(o.visitedLinkColor)
 {
@@ -79,7 +82,7 @@ bool StyleInheritedData::nonFastPathInheritedEqual(const StyleInheritedData& oth
 #if ENABLE(TEXT_AUTOSIZING)
         && specifiedLineHeight == other.specifiedLineHeight
 #endif
-        && fontCascade == other.fontCascade
+        && fontData == other.fontData
         && horizontalBorderSpacing == other.horizontalBorderSpacing
         && verticalBorderSpacing == other.verticalBorderSpacing;
 }
@@ -90,4 +93,22 @@ void StyleInheritedData::fastPathInheritFrom(const StyleInheritedData& inheritPa
     visitedLinkColor = inheritParent.visitedLinkColor;
 }
 
+#if !LOG_DISABLED
+void StyleInheritedData::dumpDifferences(TextStream& ts, const StyleInheritedData& other) const
+{
+    fontData->dumpDifferences(ts, *other.fontData);
+
+    LOG_IF_DIFFERENT(horizontalBorderSpacing);
+    LOG_IF_DIFFERENT(verticalBorderSpacing);
+    LOG_IF_DIFFERENT(lineHeight);
+
+#if ENABLE(TEXT_AUTOSIZING)
+    LOG_IF_DIFFERENT(specifiedLineHeight);
+#endif
+
+    LOG_IF_DIFFERENT(color);
+    LOG_IF_DIFFERENT(visitedLinkColor);
+}
+#endif
+
 } // namespace WebCore
