@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,79 +25,79 @@
 
 #pragma once
 
-#include "ExceptionCode.h"
 #include "SVGAngleValue.h"
-#include "SVGPropertyTearOff.h"
+#include "SVGValueProperty.h"
 
 namespace WebCore {
 
-class SVGAngle : public SVGPropertyTearOff<SVGAngleValue> {
+class SVGAngle : public SVGValueProperty<SVGAngleValue> {
+    using Base = SVGValueProperty<SVGAngleValue>;
+    using Base::Base;
+    using Base::m_value;
+
 public:
-    static Ref<SVGAngle> create(SVGAnimatedProperty& animatedProperty, SVGPropertyRole role, SVGAngleValue& value)
+    static Ref<SVGAngle> create(const SVGAngleValue& value = { })
     {
-        return adoptRef(*new SVGAngle(animatedProperty, role, value));
+        return adoptRef(*new SVGAngle(value));
     }
 
-    static Ref<SVGAngle> create(const SVGAngleValue& initialValue = { })
+    static Ref<SVGAngle> create(SVGPropertyOwner* owner, SVGPropertyAccess access, const SVGAngleValue& value = { })
     {
-        return adoptRef(*new SVGAngle(initialValue));
+        return adoptRef(*new SVGAngle(owner, access, value));
     }
 
-    static Ref<SVGAngle> create(const SVGAngleValue* initialValue)
+    template<typename T>
+    static ExceptionOr<Ref<SVGAngle>> create(ExceptionOr<T>&& value)
     {
-        return adoptRef(*new SVGAngle(initialValue));
+        if (value.hasException())
+            return value.releaseException();
+        return adoptRef(*new SVGAngle(value.releaseReturnValue()));
     }
 
-    template<typename T> static ExceptionOr<Ref<SVGAngle>> create(ExceptionOr<T>&& initialValue)
+    SVGAngleValue::Type unitType() const
     {
-        if (initialValue.hasException())
-            return initialValue.releaseException();
-        return create(initialValue.releaseReturnValue());
-    }
-
-    SVGAngleValue::Type unitType()
-    {
-        return propertyReference().unitType();
+        // Per spec https://svgwg.org/svg2-draft/types.html#__svg__SVGAngle__SVG_ANGLETYPE_UNKNOWN
+        if (m_value.unitType() > SVGAngleValue::Type::SVG_ANGLETYPE_GRAD)
+            return SVGAngleValue::Type::SVG_ANGLETYPE_UNKNOWN;
+        return m_value.unitType();
     }
 
     ExceptionOr<void> setValueForBindings(float value)
     {
         if (isReadOnly())
-            return Exception { NO_MODIFICATION_ALLOWED_ERR };
+            return Exception { ExceptionCode::NoModificationAllowedError };
 
-        propertyReference().setValue(value);
+        m_value.setValue(value);
         commitChange();
-
         return { };
     }
     
-    float valueForBindings()
+    float valueForBindings() const
     {
-        return propertyReference().value();
+        return m_value.value();
     }
 
     ExceptionOr<void> setValueInSpecifiedUnits(float valueInSpecifiedUnits)
     {
         if (isReadOnly())
-            return Exception { NO_MODIFICATION_ALLOWED_ERR };
+            return Exception { ExceptionCode::NoModificationAllowedError };
 
-        propertyReference().setValueInSpecifiedUnits(valueInSpecifiedUnits);
+        m_value.setValueInSpecifiedUnits(valueInSpecifiedUnits);
         commitChange();
-        
         return { };
     }
     
-    float valueInSpecifiedUnits()
+    float valueInSpecifiedUnits() const
     {
-        return propertyReference().valueInSpecifiedUnits();
+        return m_value.valueInSpecifiedUnits();
     }
 
     ExceptionOr<void> setValueAsString(const String& value)
     {
         if (isReadOnly())
-            return Exception { NO_MODIFICATION_ALLOWED_ERR };
+            return Exception { ExceptionCode::NoModificationAllowedError };
 
-        auto result = propertyReference().setValueAsString(value);
+        auto result = m_value.setValueAsString(value);
         if (result.hasException())
             return result;
         
@@ -105,17 +105,17 @@ public:
         return result;
     }
 
-    String valueAsString()
+    String valueAsString() const override
     {
-        return propertyReference().valueAsString();
+        return m_value.valueAsString();
     }
 
     ExceptionOr<void> newValueSpecifiedUnits(unsigned short unitType, float valueInSpecifiedUnits)
     {
         if (isReadOnly())
-            return Exception { NO_MODIFICATION_ALLOWED_ERR };
+            return Exception { ExceptionCode::NoModificationAllowedError };
 
-        auto result = propertyReference().newValueSpecifiedUnits(unitType, valueInSpecifiedUnits);
+        auto result = m_value.newValueSpecifiedUnits(unitType, valueInSpecifiedUnits);
         if (result.hasException())
             return result;
         
@@ -126,31 +126,15 @@ public:
     ExceptionOr<void> convertToSpecifiedUnits(unsigned short unitType)
     {
         if (isReadOnly())
-            return Exception { NO_MODIFICATION_ALLOWED_ERR };
+            return Exception { ExceptionCode::NoModificationAllowedError };
 
-        auto result = propertyReference().convertToSpecifiedUnits(unitType);
+        auto result = m_value.convertToSpecifiedUnits(unitType);
         if (result.hasException())
             return result;
         
         commitChange();
         return result;
     }
-
-private:
-    SVGAngle(SVGAnimatedProperty& animatedProperty, SVGPropertyRole role, SVGAngleValue& value)
-        : SVGPropertyTearOff<SVGAngleValue>(&animatedProperty, role, value)
-    {
-    }
-
-    explicit SVGAngle(const SVGAngleValue& initialValue)
-        : SVGPropertyTearOff<SVGAngleValue>(initialValue)
-    {
-    }
-
-    explicit SVGAngle(const SVGAngleValue* initialValue)
-        : SVGPropertyTearOff<SVGAngleValue>(initialValue)
-    {
-    }
 };
 
 } // namespace WebCore
