@@ -43,7 +43,9 @@ bool SVGPathParser::parse(SVGPathSource& source, SVGPathConsumer& consumer, Path
 bool SVGPathParser::parseToByteStream(SVGPathSource& source, SVGPathByteStream& byteStream, PathParsingMode mode, bool checkForInitialMoveTo)
 {
     SVGPathByteStreamBuilder builder(byteStream);
-    return parse(source, builder, mode, checkForInitialMoveTo);
+    auto result = parse(source, builder, mode, checkForInitialMoveTo);
+    byteStream.shrinkToFit();
+    return result;
 }
 
 bool SVGPathParser::parseToString(SVGPathSource& source, String& result, PathParsingMode mode, bool checkForInitialMoveTo)
@@ -68,12 +70,12 @@ void SVGPathParser::parseClosePathSegment()
     if (m_pathParsingMode == NormalizedParsing)
         m_currentPoint = m_subPathPoint;
     m_closePath = true;
-    m_consumer.closePath();
+    m_consumer->closePath();
 }
 
 bool SVGPathParser::parseMoveToSegment()
 {
-    auto result = m_source.parseMoveToSegment();
+    auto result = m_source->parseMoveToSegment(m_currentPoint);
     if (!result)
         return false;
 
@@ -83,16 +85,16 @@ bool SVGPathParser::parseMoveToSegment()
         else
             m_currentPoint = result->targetPoint;
         m_subPathPoint = m_currentPoint;
-        m_consumer.moveTo(m_currentPoint, m_closePath, AbsoluteCoordinates);
+        m_consumer->moveTo(m_currentPoint, m_closePath, AbsoluteCoordinates);
     } else
-        m_consumer.moveTo(result->targetPoint, m_closePath, m_mode);
+        m_consumer->moveTo(result->targetPoint, m_closePath, m_mode);
     m_closePath = false;
     return true;
 }
 
 bool SVGPathParser::parseLineToSegment()
 {
-    auto result = m_source.parseLineToSegment();
+    auto result = m_source->parseLineToSegment(m_currentPoint);
     if (!result)
         return false;
 
@@ -101,15 +103,15 @@ bool SVGPathParser::parseLineToSegment()
             m_currentPoint += result->targetPoint;
         else
             m_currentPoint = result->targetPoint;
-        m_consumer.lineTo(m_currentPoint, AbsoluteCoordinates);
+        m_consumer->lineTo(m_currentPoint, AbsoluteCoordinates);
     } else
-        m_consumer.lineTo(result->targetPoint, m_mode);
+        m_consumer->lineTo(result->targetPoint, m_mode);
     return true;
 }
 
 bool SVGPathParser::parseLineToHorizontalSegment()
 {
-    auto result = m_source.parseLineToHorizontalSegment();
+    auto result = m_source->parseLineToHorizontalSegment(m_currentPoint);
     if (!result)
         return false;
 
@@ -118,15 +120,15 @@ bool SVGPathParser::parseLineToHorizontalSegment()
             m_currentPoint.move(result->x, 0);
         else
             m_currentPoint.setX(result->x);
-        m_consumer.lineTo(m_currentPoint, AbsoluteCoordinates);
+        m_consumer->lineTo(m_currentPoint, AbsoluteCoordinates);
     } else
-        m_consumer.lineToHorizontal(result->x, m_mode);
+        m_consumer->lineToHorizontal(result->x, m_mode);
     return true;
 }
 
 bool SVGPathParser::parseLineToVerticalSegment()
 {
-    auto result = m_source.parseLineToVerticalSegment();
+    auto result = m_source->parseLineToVerticalSegment(m_currentPoint);
     if (!result)
         return false;
 
@@ -135,15 +137,15 @@ bool SVGPathParser::parseLineToVerticalSegment()
             m_currentPoint.move(0, result->y);
         else
             m_currentPoint.setY(result->y);
-        m_consumer.lineTo(m_currentPoint, AbsoluteCoordinates);
+        m_consumer->lineTo(m_currentPoint, AbsoluteCoordinates);
     } else
-        m_consumer.lineToVertical(result->y, m_mode);
+        m_consumer->lineToVertical(result->y, m_mode);
     return true;
 }
 
 bool SVGPathParser::parseCurveToCubicSegment()
 {
-    auto result = m_source.parseCurveToCubicSegment();
+    auto result = m_source->parseCurveToCubicSegment(m_currentPoint);
     if (!result)
         return false;
 
@@ -153,25 +155,25 @@ bool SVGPathParser::parseCurveToCubicSegment()
             result->point2 += m_currentPoint;
             result->targetPoint += m_currentPoint;
         }
-        m_consumer.curveToCubic(result->point1, result->point2, result->targetPoint, AbsoluteCoordinates);
+        m_consumer->curveToCubic(result->point1, result->point2, result->targetPoint, AbsoluteCoordinates);
 
         m_controlPoint = result->point2;
         m_currentPoint = result->targetPoint;
     } else
-        m_consumer.curveToCubic(result->point1, result->point2, result->targetPoint, m_mode);
+        m_consumer->curveToCubic(result->point1, result->point2, result->targetPoint, m_mode);
     return true;
 }
 
 bool SVGPathParser::parseCurveToCubicSmoothSegment()
 {
-    auto result = m_source.parseCurveToCubicSmoothSegment();
+    auto result = m_source->parseCurveToCubicSmoothSegment(m_currentPoint);
     if (!result)
         return false;
 
-    if (m_lastCommand != PathSegCurveToCubicAbs
-        && m_lastCommand != PathSegCurveToCubicRel
-        && m_lastCommand != PathSegCurveToCubicSmoothAbs
-        && m_lastCommand != PathSegCurveToCubicSmoothRel)
+    if (m_lastCommand != SVGPathSegType::CurveToCubicAbs
+        && m_lastCommand != SVGPathSegType::CurveToCubicRel
+        && m_lastCommand != SVGPathSegType::CurveToCubicSmoothAbs
+        && m_lastCommand != SVGPathSegType::CurveToCubicSmoothRel)
         m_controlPoint = m_currentPoint;
 
     if (m_pathParsingMode == NormalizedParsing) {
@@ -183,18 +185,18 @@ bool SVGPathParser::parseCurveToCubicSmoothSegment()
             result->targetPoint += m_currentPoint;
         }
 
-        m_consumer.curveToCubic(point1, result->point2, result->targetPoint, AbsoluteCoordinates);
+        m_consumer->curveToCubic(point1, result->point2, result->targetPoint, AbsoluteCoordinates);
 
         m_controlPoint = result->point2;
         m_currentPoint = result->targetPoint;
     } else
-        m_consumer.curveToCubicSmooth(result->point2, result->targetPoint, m_mode);
+        m_consumer->curveToCubicSmooth(result->point2, result->targetPoint, m_mode);
     return true;
 }
 
 bool SVGPathParser::parseCurveToQuadraticSegment()
 {
-    auto result = m_source.parseCurveToQuadraticSegment();
+    auto result = m_source->parseCurveToQuadraticSegment(m_currentPoint);
     if (!result)
         return false;
 
@@ -212,26 +214,26 @@ bool SVGPathParser::parseCurveToQuadraticSegment()
         point1.scale(gOneOverThree);
         point2.scale(gOneOverThree);
 
-        m_consumer.curveToCubic(point1, point2, result->targetPoint, AbsoluteCoordinates);
+        m_consumer->curveToCubic(point1, point2, result->targetPoint, AbsoluteCoordinates);
 
         if (m_mode == RelativeCoordinates)
             m_controlPoint += m_currentPoint;
         m_currentPoint = result->targetPoint;
     } else
-        m_consumer.curveToQuadratic(result->point1, result->targetPoint, m_mode);
+        m_consumer->curveToQuadratic(result->point1, result->targetPoint, m_mode);
     return true;
 }
 
 bool SVGPathParser::parseCurveToQuadraticSmoothSegment()
 {
-    auto result = m_source.parseCurveToQuadraticSmoothSegment();
+    auto result = m_source->parseCurveToQuadraticSmoothSegment(m_currentPoint);
     if (!result)
         return false;
 
-    if (m_lastCommand != PathSegCurveToQuadraticAbs
-        && m_lastCommand != PathSegCurveToQuadraticRel
-        && m_lastCommand != PathSegCurveToQuadraticSmoothAbs
-        && m_lastCommand != PathSegCurveToQuadraticSmoothRel)
+    if (m_lastCommand != SVGPathSegType::CurveToQuadraticAbs
+        && m_lastCommand != SVGPathSegType::CurveToQuadraticRel
+        && m_lastCommand != SVGPathSegType::CurveToQuadraticSmoothAbs
+        && m_lastCommand != SVGPathSegType::CurveToQuadraticSmoothRel)
         m_controlPoint = m_currentPoint;
 
     if (m_pathParsingMode == NormalizedParsing) {
@@ -247,18 +249,18 @@ bool SVGPathParser::parseCurveToQuadraticSmoothSegment()
         point1.scale(gOneOverThree);
         point2.scale(gOneOverThree);
 
-        m_consumer.curveToCubic(point1, point2, result->targetPoint, AbsoluteCoordinates);
+        m_consumer->curveToCubic(point1, point2, result->targetPoint, AbsoluteCoordinates);
 
         m_controlPoint = cubicPoint;
         m_currentPoint = result->targetPoint;
     } else
-        m_consumer.curveToQuadraticSmooth(result->targetPoint, m_mode);
+        m_consumer->curveToQuadraticSmooth(result->targetPoint, m_mode);
     return true;
 }
 
 bool SVGPathParser::parseArcToSegment()
 {
-    auto result = m_source.parseArcToSegment();
+    auto result = m_source->parseArcToSegment(m_currentPoint);
     if (!result)
         return false;
 
@@ -266,10 +268,10 @@ bool SVGPathParser::parseArcToSegment()
     // http://www.w3.org/TR/SVG/implnote.html#ArcOutOfRangeParameters
     // If the current point and target point for the arc are identical, it should be treated as a zero length
     // path. This ensures continuity in animations.
-    result->rx = std::abs(result->rx);
-    result->ry = std::abs(result->ry);
     bool arcIsZeroLength = false;
     if (m_pathParsingMode == NormalizedParsing) {
+        result->rx = std::abs(result->rx);
+        result->ry = std::abs(result->ry);
         if (m_mode == RelativeCoordinates)
             arcIsZeroLength = result->targetPoint == FloatPoint::zero();
         else
@@ -281,9 +283,9 @@ bool SVGPathParser::parseArcToSegment()
                 m_currentPoint += result->targetPoint;
             else
                 m_currentPoint = result->targetPoint;
-            m_consumer.lineTo(m_currentPoint, AbsoluteCoordinates);
+            m_consumer->lineTo(m_currentPoint, AbsoluteCoordinates);
         } else
-            m_consumer.lineTo(result->targetPoint, m_mode);
+            m_consumer->lineTo(result->targetPoint, m_mode);
         return true;
     }
 
@@ -294,121 +296,121 @@ bool SVGPathParser::parseArcToSegment()
         m_currentPoint = result->targetPoint;
         return decomposeArcToCubic(result->angle, result->rx, result->ry, point1, result->targetPoint, result->largeArc, result->sweep);
     }
-    m_consumer.arcTo(result->rx, result->ry, result->angle, result->largeArc, result->sweep, result->targetPoint, m_mode);
+    m_consumer->arcTo(result->rx, result->ry, result->angle, result->largeArc, result->sweep, result->targetPoint, m_mode);
     return true;
 }
 
 bool SVGPathParser::parsePathData(bool checkForInitialMoveTo)
 {
     // Skip any leading spaces.
-    if (!m_source.moveToNextToken())
+    if (!m_source->moveToNextToken())
         return true;
 
-    auto parsedCommand = m_source.parseSVGSegmentType();
+    auto parsedCommand = m_source->parseSVGSegmentType();
     if (!parsedCommand)
         return false;
 
     auto command = *parsedCommand;
 
     // Path must start with moveto.
-    if (checkForInitialMoveTo && command != PathSegMoveToAbs && command != PathSegMoveToRel)
+    if (checkForInitialMoveTo && command != SVGPathSegType::MoveToAbs && command != SVGPathSegType::MoveToRel)
         return false;
 
     while (true) {
         // Skip spaces between command and first coordinate.
-        m_source.moveToNextToken();
+        m_source->moveToNextToken();
         m_mode = AbsoluteCoordinates;
         switch (command) {
-        case PathSegMoveToRel:
+        case SVGPathSegType::MoveToRel:
             m_mode = RelativeCoordinates;
             FALLTHROUGH;
-        case PathSegMoveToAbs:
+        case SVGPathSegType::MoveToAbs:
             if (!parseMoveToSegment())
                 return false;
             break;
-        case PathSegLineToRel:
+        case SVGPathSegType::LineToRel:
             m_mode = RelativeCoordinates;
             FALLTHROUGH;
-        case PathSegLineToAbs:
+        case SVGPathSegType::LineToAbs:
             if (!parseLineToSegment())
                 return false;
             break;
-        case PathSegLineToHorizontalRel:
+        case SVGPathSegType::LineToHorizontalRel:
             m_mode = RelativeCoordinates;
             FALLTHROUGH;
-        case PathSegLineToHorizontalAbs:
+        case SVGPathSegType::LineToHorizontalAbs:
             if (!parseLineToHorizontalSegment())
                 return false;
             break;
-        case PathSegLineToVerticalRel:
+        case SVGPathSegType::LineToVerticalRel:
             m_mode = RelativeCoordinates;
             FALLTHROUGH;
-        case PathSegLineToVerticalAbs:
+        case SVGPathSegType::LineToVerticalAbs:
             if (!parseLineToVerticalSegment())
                 return false;
             break;
-        case PathSegClosePath:
+        case SVGPathSegType::ClosePath:
             parseClosePathSegment();
             break;
-        case PathSegCurveToCubicRel:
+        case SVGPathSegType::CurveToCubicRel:
             m_mode = RelativeCoordinates;
             FALLTHROUGH;
-        case PathSegCurveToCubicAbs:
+        case SVGPathSegType::CurveToCubicAbs:
             if (!parseCurveToCubicSegment())
                 return false;
             break;
-        case PathSegCurveToCubicSmoothRel:
+        case SVGPathSegType::CurveToCubicSmoothRel:
             m_mode = RelativeCoordinates;
             FALLTHROUGH;
-        case PathSegCurveToCubicSmoothAbs:
+        case SVGPathSegType::CurveToCubicSmoothAbs:
             if (!parseCurveToCubicSmoothSegment())
                 return false;
             break;
-        case PathSegCurveToQuadraticRel:
+        case SVGPathSegType::CurveToQuadraticRel:
             m_mode = RelativeCoordinates;
             FALLTHROUGH;
-        case PathSegCurveToQuadraticAbs:
+        case SVGPathSegType::CurveToQuadraticAbs:
             if (!parseCurveToQuadraticSegment())
                 return false;
             break;
-        case PathSegCurveToQuadraticSmoothRel:
+        case SVGPathSegType::CurveToQuadraticSmoothRel:
             m_mode = RelativeCoordinates;
             FALLTHROUGH;
-        case PathSegCurveToQuadraticSmoothAbs:
+        case SVGPathSegType::CurveToQuadraticSmoothAbs:
             if (!parseCurveToQuadraticSmoothSegment())
                 return false;
             break;
-        case PathSegArcRel:
+        case SVGPathSegType::ArcRel:
             m_mode = RelativeCoordinates;
             FALLTHROUGH;
-        case PathSegArcAbs:
+        case SVGPathSegType::ArcAbs:
             if (!parseArcToSegment())
                 return false;
             break;
         default:
             return false;
         }
-        if (!m_consumer.continueConsuming())
+        if (!m_consumer->continueConsuming())
             return true;
 
         m_lastCommand = command;
 
-        if (!m_source.hasMoreData())
+        if (!m_source->hasMoreData())
             return true;
 
-        command = m_source.nextCommand(command);
+        command = m_source->nextCommand(command);
 
-        if (m_lastCommand != PathSegCurveToCubicAbs
-            && m_lastCommand != PathSegCurveToCubicRel
-            && m_lastCommand != PathSegCurveToCubicSmoothAbs
-            && m_lastCommand != PathSegCurveToCubicSmoothRel
-            && m_lastCommand != PathSegCurveToQuadraticAbs
-            && m_lastCommand != PathSegCurveToQuadraticRel
-            && m_lastCommand != PathSegCurveToQuadraticSmoothAbs
-            && m_lastCommand != PathSegCurveToQuadraticSmoothRel)
+        if (m_lastCommand != SVGPathSegType::CurveToCubicAbs
+            && m_lastCommand != SVGPathSegType::CurveToCubicRel
+            && m_lastCommand != SVGPathSegType::CurveToCubicSmoothAbs
+            && m_lastCommand != SVGPathSegType::CurveToCubicSmoothRel
+            && m_lastCommand != SVGPathSegType::CurveToQuadraticAbs
+            && m_lastCommand != SVGPathSegType::CurveToQuadraticRel
+            && m_lastCommand != SVGPathSegType::CurveToQuadraticSmoothAbs
+            && m_lastCommand != SVGPathSegType::CurveToQuadraticSmoothRel)
             m_controlPoint = m_currentPoint;
 
-        m_consumer.incrementPathSegmentCount();
+        m_consumer->incrementPathSegmentCount();
     }
 
     return false;
@@ -474,7 +476,7 @@ bool SVGPathParser::decomposeArcToCubic(float angle, float rx, float ry, FloatPo
 
     // Some results of atan2 on some platform implementations are not exact enough. So that we get more
     // cubic curves than expected here. Adding 0.001f reduces the count of sgements to the correct count.
-    int segments = ceilf(fabsf(thetaArc / (piOverTwoFloat + 0.001f)));
+    int segments = ceilf(std::abs(thetaArc / (piOverTwoFloat + 0.001f)));
     for (int i = 0; i < segments; ++i) {
         float startTheta = theta1 + i * thetaArc / segments;
         float endTheta = theta1 + (i + 1) * thetaArc / segments;
@@ -494,7 +496,7 @@ bool SVGPathParser::decomposeArcToCubic(float angle, float rx, float ry, FloatPo
         point2 = targetPoint;
         point2.move(t * sinEndTheta, -t * cosEndTheta);
 
-        m_consumer.curveToCubic(pointTransform.mapPoint(point1), pointTransform.mapPoint(point2),
+        m_consumer->curveToCubic(pointTransform.mapPoint(point1), pointTransform.mapPoint(point2),
                                  pointTransform.mapPoint(targetPoint), AbsoluteCoordinates);
     }
     return true;
