@@ -28,29 +28,28 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef WorkerObjectProxy_h
-#define WorkerObjectProxy_h
+#pragma once
 
+#include "MessageWithMessagePorts.h"
 #include "WorkerReportingProxy.h"
-#include "MessagePort.h"
+
 #include <memory>
+#include <wtf/Function.h>
 
 namespace WebCore {
 
-    class MessagePortChannel;
+class Worker;
 
-    // A proxy to talk to the worker object.
-    class WorkerObjectProxy : public WorkerReportingProxy {
-    public:
-        virtual void postMessageToWorkerObject(RefPtr<SerializedScriptValue>&&, std::unique_ptr<MessagePortChannelArray>) = 0;
+// A proxy to talk to the worker object.
+class WorkerObjectProxy : public WorkerReportingProxy {
+    WTF_MAKE_FAST_ALLOCATED;
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(WorkerObjectProxy);
+public:
+    virtual void postMessageToWorkerObject(MessageWithMessagePorts&&) = 0;
+    virtual void postTaskToWorkerObject(Function<void(Worker&)>&&) { }
 
-        virtual void confirmMessageFromWorkerObject(bool hasPendingActivity) = 0;
-        virtual void reportPendingActivity(bool hasPendingActivity) = 0;
-
-        // No need to notify the parent page context when dedicated workers are closing.
-        void workerGlobalScopeClosed() override { }
-    };
+    // No need to notify the parent page context when dedicated workers are closing.
+    void workerGlobalScopeClosed() override { }
+};
 
 } // namespace WebCore
-
-#endif // WorkerObjectProxy_h
