@@ -29,10 +29,13 @@
 
 #include <wtf/NeverDestroyed.h>
 #include <wtf/StdLibExtras.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 namespace XPath {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(Expression);
+
 EvaluationContext& Expression::evaluationContext()
 {
     static NeverDestroyed<EvaluationContext> context;
@@ -49,11 +52,11 @@ Expression::Expression()
 void Expression::setSubexpressions(Vector<std::unique_ptr<Expression>> subexpressions)
 {
     ASSERT(m_subexpressions.isEmpty());
-    m_subexpressions = WTF::move(subexpressions);
-    for (unsigned i = 0; i < m_subexpressions.size(); ++i) {
-        m_isContextNodeSensitive |= m_subexpressions[i]->m_isContextNodeSensitive;
-        m_isContextPositionSensitive |= m_subexpressions[i]->m_isContextPositionSensitive;
-        m_isContextSizeSensitive |= m_subexpressions[i]->m_isContextSizeSensitive;
+    m_subexpressions = WTFMove(subexpressions);
+    for (auto& subexpression : m_subexpressions) {
+        m_isContextNodeSensitive |= subexpression->m_isContextNodeSensitive;
+        m_isContextPositionSensitive |= subexpression->m_isContextPositionSensitive;
+        m_isContextSizeSensitive |= subexpression->m_isContextSizeSensitive;
     }
 }
 
