@@ -39,11 +39,15 @@
 #include <wtf/NeverDestroyed.h>
 #include <wtf/RobinHoodHashMap.h>
 #include <wtf/SetForScope.h>
+#include <wtf/TZoneMallocInlines.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/StringBuilder.h>
 
 namespace WebCore {
 namespace XPath {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(Function);
+
 static inline bool isWhitespace(UChar c)
 {
     return c == ' ' || c == '\n' || c == '\r' || c == '\t';
@@ -67,6 +71,7 @@ private:
 };
 
 class FunLast final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunLast);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Number; }
 public:
@@ -74,6 +79,7 @@ public:
 };
 
 class FunPosition final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunPosition);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Number; }
 public:
@@ -81,16 +87,19 @@ public:
 };
 
 class FunCount final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunCount);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Number; }
 };
 
 class FunId final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunId);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::NodeSet; }
 };
 
 class FunLocalName final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunLocalName);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 public:
@@ -98,6 +107,7 @@ public:
 };
 
 class FunNamespaceURI final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunNamespaceURI);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 public:
@@ -105,6 +115,7 @@ public:
 };
 
 class FunName final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunName);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 public:
@@ -112,6 +123,7 @@ public:
 };
 
 class FunString final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunString);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 public:
@@ -119,36 +131,43 @@ public:
 };
 
 class FunConcat final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunConcat);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 };
 
 class FunStartsWith final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunStartsWith);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Boolean; }
 };
 
 class FunContains final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunContains);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Boolean; }
 };
 
 class FunSubstringBefore final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunSubstringBefore);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 };
 
 class FunSubstringAfter final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunSubstringAfter);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 };
 
 class FunSubstring final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunSubstring);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 };
 
 class FunStringLength final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunStringLength);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Number; }
 public:
@@ -156,6 +175,7 @@ public:
 };
 
 class FunNormalizeSpace final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunNormalizeSpace);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 public:
@@ -163,31 +183,37 @@ public:
 };
 
 class FunTranslate final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunTranslate);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::String; }
 };
 
 class FunBoolean final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunBoolean);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Boolean; }
 };
 
 class FunNot : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunNot);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Boolean; }
 };
 
 class FunTrue final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunTrue);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Boolean; }
 };
 
 class FunFalse final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunFalse);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Boolean; }
 };
 
 class FunLang final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunLang);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Boolean; }
 public:
@@ -195,6 +221,7 @@ public:
 };
 
 class FunNumber final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunNumber);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Number; }
 public:
@@ -202,21 +229,25 @@ public:
 };
 
 class FunSum final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunSum);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Number; }
 };
 
 class FunFloor final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunFloor);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Number; }
 };
 
 class FunCeiling final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunCeiling);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Number; }
 };
 
 class FunRound final : public Function {
+    WTF_MAKE_TZONE_ALLOCATED_INLINE(FunRound);
     Value evaluate() const override;
     Value::Type resultType() const override { return Value::Type::Number; }
 public:
@@ -512,48 +543,60 @@ Value FunSubstringAfter::evaluate() const
     return s1.substring(i + s2.length());
 }
 
+// Computes the 1-based start and end (exclusive) string indices for
+// substring. This is all the positions [1, maxLen (inclusive)] where
+// start <= position < start + len
+static std::pair<unsigned, unsigned> computeSubstringStartEnd(double start, double len, double maxLen)
+{
+    ASSERT(std::isfinite(maxLen));
+    const double end = start + len;
+    if (std::isnan(start) || std::isnan(end))
+        return std::make_pair(1, 1);
+
+    // Neither start nor end are NaN, but may still be +/- Inf
+    const double clampedStart = std::clamp<double>(start, 1, maxLen + 1);
+    const double clampedEnd = std::clamp<double>(end, clampedStart, maxLen + 1);
+    return std::make_pair(static_cast<unsigned>(clampedStart), static_cast<unsigned>(clampedEnd));
+}
+
+// substring(string, number pos, number? len)
+//
+// Characters in string are indexed from 1. Numbers are doubles and
+// substring is specified to work with IEEE-754 infinity, NaN, and
+// XPath's bespoke rounding function, round.
+//
+// <https://www.w3.org/TR/xpath/#function-substring>
 Value FunSubstring::evaluate() const
 {
     EvaluationContext clonedContext1(Expression::evaluationContext());
     EvaluationContext clonedContext2(Expression::evaluationContext());
+    EvaluationContext clonedContext3(Expression::evaluationContext());
 
-    String s;
-    double doublePos;
+    String sourceString;
+    double pos;
+    double len;
 
     {
         SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext1);
-        s = argument(0).evaluate().toString();
+        sourceString = argument(0).evaluate().toString();
     }
     {
         SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext2);
-        doublePos = argument(1).evaluate().toNumber();
+        pos = FunRound::round(argument(1).evaluate().toNumber());
     }
 
-    if (std::isnan(doublePos))
-        return emptyString();
-    long pos = static_cast<long>(FunRound::round(doublePos));
-    bool haveLength = argumentCount() == 3;
-    long len = -1;
-    if (haveLength) {
-        double doubleLen = argument(2).evaluate().toNumber();
-        if (std::isnan(doubleLen))
-            return emptyString();
-        len = static_cast<long>(FunRound::round(doubleLen));
+    if (argumentCount() == 3) {
+        SetForScope<EvaluationContext> contextForScope(Expression::evaluationContext(), clonedContext3);
+        len = FunRound::round(argument(2).evaluate().toNumber());
+    } else {
+        len = std::numeric_limits<double>::infinity();
     }
 
-    if (pos > long(s.length())) 
+    const auto bounds = computeSubstringStartEnd(pos, len, sourceString.length());
+    if (bounds.second <= bounds.first)
         return emptyString();
 
-    if (pos < 1) {
-        if (haveLength) {
-            len -= 1 - pos;
-            if (len < 1)
-                return emptyString();
-        }
-        pos = 1;
-    }
-
-    return s.substring(pos - 1, len);
+    return sourceString.substring(bounds.first - 1, bounds.second - bounds.first);
 }
 
 Value FunStringLength::evaluate() const
