@@ -31,9 +31,15 @@
 #include "Document.h"
 #include "XPathPredicate.h"
 #include "XPathStep.h"
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 namespace XPath {
+
+WTF_MAKE_TZONE_ALLOCATED_IMPL(Filter);
+WTF_MAKE_TZONE_ALLOCATED_IMPL(LocationPath);
+WTF_MAKE_TZONE_ALLOCATED_IMPL(Path);
+
         
 Filter::Filter(std::unique_ptr<Expression> expression, Vector<std::unique_ptr<Expression>> predicates)
     : m_expression(WTFMove(expression)), m_predicates(WTFMove(predicates))
@@ -89,7 +95,7 @@ Value LocationPath::evaluate() const
     // logical treatment of where you would expect the "root" to be.
     Node* context = evaluationContext.node.get();
     if (m_isAbsolute && !context->isDocumentNode())
-        context = context->rootNode();
+        context = &context->rootNode();
 
     NodeSet nodes;
     nodes.append(context);
@@ -105,7 +111,7 @@ void LocationPath::evaluate(NodeSet& nodes) const
 
     for (auto& step : m_steps) {
         NodeSet newNodes;
-        HashSet<Node*> newNodesSet;
+        HashSet<RefPtr<Node>> newNodesSet;
 
         bool needToCheckForDuplicateNodes = !nodes.subtreesAreDisjoint() || (step->axis() != Step::ChildAxis && step->axis() != Step::SelfAxis
             && step->axis() != Step::DescendantAxis && step->axis() != Step::DescendantOrSelfAxis && step->axis() != Step::AttributeAxis);
