@@ -26,8 +26,9 @@
 
 #pragma once
 
+#include <wtf/TZoneMalloc.h>
 #include <wtf/Vector.h>
-#include <wtf/text/AtomicString.h>
+#include <wtf/text/AtomString.h>
 
 namespace WebCore {
 
@@ -39,7 +40,7 @@ class Expression;
 class NodeSet;
 
 class Step {
-    WTF_MAKE_FAST_ALLOCATED;
+    WTF_MAKE_TZONE_ALLOCATED(Step);
 public:
     enum Axis {
         AncestorAxis, AncestorOrSelfAxis, AttributeAxis,
@@ -50,34 +51,13 @@ public:
     };
 
     class NodeTest {
-        WTF_MAKE_FAST_ALLOCATED;
+        WTF_MAKE_TZONE_ALLOCATED(NodeTest);
     public:
         enum Kind { TextNodeTest, CommentNodeTest, ProcessingInstructionNodeTest, AnyNodeTest, NameTest };
 
         explicit NodeTest(Kind kind) : m_kind(kind) { }
-        NodeTest(Kind kind, const AtomicString& data) : m_kind(kind), m_data(data) { }
-        NodeTest(Kind kind, const AtomicString& data, const AtomicString& namespaceURI) : m_kind(kind), m_data(data), m_namespaceURI(namespaceURI) { }
-
-#if COMPILER(MSVC)
-        NodeTest(const NodeTest&);
-        void operator=(const NodeTest&);
-
-        NodeTest(NodeTest&& other)
-            : m_kind(other.m_kind)
-            , m_data(WTFMove(other.m_data))
-            , m_namespaceURI(WTFMove(other.m_namespaceURI))
-            , m_mergedPredicates(WTFMove(other.m_mergedPredicates))
-        {
-        }
-        NodeTest& operator=(NodeTest&& other)
-        {
-            m_kind = other.m_kind;
-            m_data = WTFMove(other.m_data);
-            m_namespaceURI = WTFMove(other.m_namespaceURI);
-            m_mergedPredicates = WTFMove(other.m_mergedPredicates);
-            return *this;
-        }
-#endif
+        NodeTest(Kind kind, const AtomString& data) : m_kind(kind), m_data(data) { }
+        NodeTest(Kind kind, const AtomString& data, const AtomString& namespaceURI) : m_kind(kind), m_data(data), m_namespaceURI(namespaceURI) { }
 
     private:
         friend class Step;
@@ -86,8 +66,8 @@ public:
         friend bool nodeMatches(Node&, Axis, const NodeTest&);
 
         Kind m_kind;
-        AtomicString m_data;
-        AtomicString m_namespaceURI;
+        AtomString m_data;
+        AtomString m_namespaceURI;
         Vector<std::unique_ptr<Expression>> m_mergedPredicates;
     };
 
