@@ -41,12 +41,14 @@
 #include <wtf/UniqueRef.h>
 #include <wtf/text/WTFString.h>
 
-ALLOW_COMMA_BEGIN
+#if !PLATFORM(COCOA)
 
+WTF_IGNORE_WARNINGS_IN_THIRD_PARTY_CODE_BEGIN
 #include <webrtc/p2p/base/basic_packet_socket_factory.h>
 #include <webrtc/rtc_base/third_party/sigslot/sigslot.h>
+WTF_IGNORE_WARNINGS_IN_THIRD_PARTY_CODE_END
 
-ALLOW_COMMA_END
+#endif
 
 namespace IPC {
 class Connection;
@@ -75,7 +77,9 @@ struct SocketComparator {
     }
 };
 
-class NetworkRTCProvider : private FunctionDispatcher, private IPC::MessageReceiver, public ThreadSafeRefCounted<NetworkRTCProvider, WTF::DestructionThread::MainRunLoop> {
+class NetworkRTCProvider : private FunctionDispatcher, private IPC::MessageReceiver, public ThreadSafeRefCounted<NetworkRTCProvider, WTF::DestructionThread::MainRunLoop>, public CanMakeThreadSafeCheckedPtr<NetworkRTCProvider> {
+    WTF_MAKE_FAST_ALLOCATED;
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(NetworkRTCProvider);
 public:
     static Ref<NetworkRTCProvider> create(NetworkConnectionToWebProcess& connection)
     {
@@ -85,7 +89,10 @@ public:
     }
     ~NetworkRTCProvider();
 
-    void didReceiveNetworkRTCMonitorMessage(IPC::Connection& connection, IPC::Decoder& decoder) { m_rtcMonitor.didReceiveMessage(connection, decoder); }
+    void ref() const final { ThreadSafeRefCounted::ref(); }
+    void deref() const final { ThreadSafeRefCounted::deref(); }
+
+    void didReceiveNetworkRTCMonitorMessage(IPC::Connection& connection, IPC::Decoder& decoder) { protectedRTCMonitor()->didReceiveMessage(connection, decoder); }
 
     class Socket {
     public:
@@ -107,38 +114,37 @@ public:
 
     void callOnRTCNetworkThread(Function<void()>&&);
 
-    void authorizeListeningSockets() { m_isListeningSocketAuthorized = true; }
-
     IPC::Connection& connection() { return m_ipcConnection.get(); }
+    Ref<IPC::Connection> protectedConnection() { return m_ipcConnection.get(); }
 
     void closeSocket(WebCore::LibWebRTCSocketIdentifier);
-    void doSocketTaskOnRTCNetworkThread(WebCore::LibWebRTCSocketIdentifier, Function<void(Socket&)>&&);
 
 #if PLATFORM(COCOA)
     const std::optional<audit_token_t>& sourceApplicationAuditToken() const { return m_sourceApplicationAuditToken; }
     const char* applicationBundleIdentifier() const { return m_applicationBundleIdentifier.data(); }
 #endif
 
-    static rtc::Thread& rtcNetworkThread();
-
 private:
     explicit NetworkRTCProvider(NetworkConnectionToWebProcess&);
     void startListeningForIPC();
 
     void createUDPSocket(WebCore::LibWebRTCSocketIdentifier, const RTCNetwork::SocketAddress&, uint16_t, uint16_t, WebPageProxyIdentifier, bool isFirstParty, bool isRelayDisabled, WebCore::RegistrableDomain&&);
     void createClientTCPSocket(WebCore::LibWebRTCSocketIdentifier, const RTCNetwork::SocketAddress&, const RTCNetwork::SocketAddress&, String&& userAgent, int, WebPageProxyIdentifier, bool isFirstParty, bool isRelayDisabled, WebCore::RegistrableDomain&&);
-    void wrapNewTCPConnection(WebCore::LibWebRTCSocketIdentifier identifier, WebCore::LibWebRTCSocketIdentifier newConnectionSocketIdentifier);
     void sendToSocket(WebCore::LibWebRTCSocketIdentifier, std::span<const uint8_t>, RTCNetwork::SocketAddress&&, RTCPacketOptions&&);
     void setSocketOption(WebCore::LibWebRTCSocketIdentifier, int option, int value);
-    void setPlatformTCPSocketsEnabled(bool enabled) { m_platformTCPSocketsEnabled = enabled; }
-    void setPlatformUDPSocketsEnabled(bool enabled) { m_platformUDPSocketsEnabled = enabled; }
 
     void createResolver(LibWebRTCResolverIdentifier, String&&);
     void stopResolver(LibWebRTCResolverIdentifier);
+    void getInterfaceName(URL&&, WebPageProxyIdentifier, bool isFirstParty, bool isRelayDisabled, WebCore::RegistrableDomain&&, CompletionHandler<void(String&&)>&&);
 
     void addSocket(WebCore::LibWebRTCSocketIdentifier, std::unique_ptr<Socket>&&);
 
+#if PLATFORM(COCOA)
+    const String& attributedBundleIdentifierFromPageIdentifier(WebPageProxyIdentifier);
+#else
+    static rtc::Thread& rtcNetworkThread();
     void createSocket(WebCore::LibWebRTCSocketIdentifier, std::unique_ptr<rtc::AsyncPacketSocket>&&, Socket::Type, Ref<IPC::Connection>&&);
+#endif
 
     // FunctionDispatcher
     void dispatch(Function<void()>&&) final;
@@ -146,12 +152,15 @@ private:
     // IPC::MessageReceiver
     void didReceiveMessage(IPC::Connection&, IPC::Decoder&) final;
 
-    static rtc::ProxyInfo proxyInfoFromSession(const RTCNetwork::SocketAddress&, NetworkSession&);
+    void signalSocketIsClosed(WebCore::LibWebRTCSocketIdentifier);
+
+    void assertIsRTCNetworkThread();
+
+    Ref<NetworkRTCMonitor> protectedRTCMonitor();
 
 #if PLATFORM(COCOA)
-    const String& attributedBundleIdentifierFromPageIdentifier(WebPageProxyIdentifier);
+    Ref<WorkQueue> protectedRTCNetworkThreadQueue();
 #endif
-    void signalSocketIsClosed(WebCore::LibWebRTCSocketIdentifier);
 
     static constexpr size_t maxSockets { 256 };
 
@@ -162,20 +171,16 @@ private:
 
     NetworkRTCMonitor m_rtcMonitor;
 
-    rtc::Thread& m_rtcNetworkThread;
-    UniqueRef<rtc::BasicPacketSocketFactory> m_packetSocketFactory;
-
-    HashMap<WebCore::LibWebRTCSocketIdentifier, std::unique_ptr<rtc::AsyncPacketSocket>> m_pendingIncomingSockets;
-    bool m_isListeningSocketAuthorized { true };
-    bool m_platformTCPSocketsEnabled { false };
-    bool m_platformUDPSocketsEnabled { false };
-
 #if PLATFORM(COCOA)
     HashMap<WebPageProxyIdentifier, String> m_attributedBundleIdentifiers;
     std::optional<audit_token_t> m_sourceApplicationAuditToken;
     CString m_applicationBundleIdentifier;
+    Ref<WorkQueue> m_rtcNetworkThreadQueue;
 #endif
 
+#if !PLATFORM(COCOA)
+    UniqueRef<rtc::BasicPacketSocketFactory> m_packetSocketFactory;
+#endif
 };
 
 } // namespace WebKit
