@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2,1 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -66,7 +66,7 @@ static void webkit_frame_class_init(WebKitFrameClass*)
 
 static CString getURL(WebFrame* webFrame)
 {
-    auto* documentLoader = webFrame->coreLocalFrame()->loader().provisionalDocumentLoader();
+    RefPtr documentLoader = webFrame->coreLocalFrame()->loader().provisionalDocumentLoader();
     if (!documentLoader)
         documentLoader = webFrame->coreLocalFrame()->loader().documentLoader();
 
@@ -102,8 +102,8 @@ GRefPtr<JSCValue> webkitFrameGetJSCValueForElementInWorld(WebKitFrame* frame, El
 
 Vector<GRefPtr<JSCValue>> webkitFrameGetJSCValuesForElementsInWorld(WebKitFrame* frame, const Vector<RefPtr<Element>>& elements, WebKitScriptWorld* world)
 {
-    auto* wkWorld = webkitScriptWorldGetInjectedBundleScriptWorld(world);
-    auto jsContext = jscContextGetOrCreate(frame->priv->webFrame->jsContextForWorld(wkWorld));
+    RefPtr wkWorld = webkitScriptWorldGetInjectedBundleScriptWorld(world);
+    auto jsContext = jscContextGetOrCreate(frame->priv->webFrame->jsContextForWorld(wkWorld.get()));
     auto* globalObject = frame->priv->webFrame->coreLocalFrame()->script().globalObject(wkWorld->coreWorld());
     return elements.map([&jsContext, globalObject](auto& element) -> GRefPtr<JSCValue> {
         JSValueRef jsValue = nullptr;
