@@ -29,54 +29,84 @@
 #if ENABLE(DRAG_SUPPORT)
 
 #include "ArgumentCodersGtk.h"
-#include "ShareableBitmap.h"
+#include "MessageSenderInlines.h"
 #include "WebPage.h"
 #include "WebPageProxyMessages.h"
-#include "WebSelectionData.h"
-#include <WebCore/CairoOperations.h>
 #include <WebCore/DataTransfer.h>
 #include <WebCore/DragData.h>
 #include <WebCore/GraphicsContext.h>
 #include <WebCore/Pasteboard.h>
-#include <WebCore/PlatformContextCairo.h>
+#include <WebCore/SelectionData.h>
+#include <WebCore/ShareableBitmap.h>
+
+#if USE(CAIRO)
+#include <WebCore/CairoOperations.h>
 #include <cairo.h>
+#endif
 
 namespace WebKit {
 using namespace WebCore;
 
+#if USE(CAIRO)
 static RefPtr<ShareableBitmap> convertCairoSurfaceToShareableBitmap(cairo_surface_t* surface)
 {
     if (!surface)
         return nullptr;
 
     IntSize imageSize(cairo_image_surface_get_width(surface), cairo_image_surface_get_height(surface));
-    RefPtr<ShareableBitmap> bitmap = ShareableBitmap::createShareable(imageSize, { });
+    auto bitmap = ShareableBitmap::create({ imageSize });
     auto graphicsContext = bitmap->createGraphicsContext();
 
     ASSERT(graphicsContext->hasPlatformContext());
     auto& state = graphicsContext->state();
-    Cairo::drawSurface(*graphicsContext->platformContext(), surface, IntRect(IntPoint(), imageSize), IntRect(IntPoint(), imageSize), state.imageInterpolationQuality, state.alpha, Cairo::ShadowState(state));
+    Cairo::drawSurface(*graphicsContext->platformContext(), surface, IntRect(IntPoint(), imageSize), IntRect(IntPoint(), imageSize), state.imageInterpolationQuality(), state.alpha(), Cairo::ShadowState(state));
     return bitmap;
 }
+#endif
+
+#if USE(SKIA)
+static RefPtr<ShareableBitmap> convertSkiaImageToShareableBitmap(SkImage* image)
+{
+    if (!image)
+        return nullptr;
+
+    IntSize imageSize(image->width(), image->height());
+    RefPtr bitmap = ShareableBitmap::create({ imageSize });
+    auto graphicsContext = bitmap->createGraphicsContext();
+
+    ASSERT(graphicsContext->hasPlatformContext());
+    graphicsContext->platformContext()->drawImage(image, 0, 0);
+
+    return bitmap;
+}
+#endif
 
 void WebDragClient::didConcludeEditDrag()
 {
 }
 
-void WebDragClient::startDrag(DragItem item, DataTransfer& dataTransfer, Frame&)
+void WebDragClient::startDrag(DragItem dragItem, DataTransfer& dataTransfer, Frame&)
 {
-    auto& dragImage = item.image;
-    RefPtr<ShareableBitmap> bitmap = convertCairoSurfaceToShareableBitmap(dragImage.get().get());
-    ShareableBitmap::Handle handle;
+    std::optional<ShareableBitmap::Handle> handle;
+    auto* dragSurface = dragItem.image.get().get();
+    RefPtr<ShareableBitmap> bitmap;
 
-    // If we have a bitmap, but cannot create a handle to it, we fail early.
-    if (bitmap && !bitmap->createHandle(handle))
-        return;
+#if USE(CAIRO)
+    bitmap = convertCairoSurfaceToShareableBitmap(dragSurface);
+#elif USE(SKIA)
+    bitmap = convertSkiaImageToShareableBitmap(dragSurface);
+#endif
+
+    if (bitmap) {
+        handle = bitmap->createHandle();
+
+        // If we have a bitmap, but cannot create a handle to it, we fail early.
+        if (!handle)
+            return;
+    }
 
     m_page->willStartDrag();
-
-    WebSelectionData selection(dataTransfer.pasteboard().selectionData());
-    m_page->send(Messages::WebPageProxy::StartDrag(selection, dataTransfer.sourceOperation(), handle));
+    m_page->send(Messages::WebPageProxy::StartDrag(dataTransfer.pasteboard().selectionData(), dataTransfer.sourceOperationMask(), WTFMove(handle), dataTransfer.dragLocation()));
 }
 
 }; // namespace WebKit.
