@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2022 Igalia S.L.
+ * Copyright (C) 2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,6 +31,7 @@
 #include "Encoder.h"
 #include "StreamConnectionEncoder.h"
 #include "Test.h"
+#include <wtf/StdLibExtras.h>
 
 namespace TestWebKitAPI {
 
@@ -64,16 +66,16 @@ public:
     void SetUp() override
     {
         m_encoder = makeUnique<IPC::Encoder>(EncoderDecoderTest::name(), 0);
-        ASSERT_EQ(m_encoder->bufferSize(), headerSize());
+        ASSERT_EQ(m_encoder->span().size(), headerSize());
     }
 
     IPC::Encoder& encoder() const { return *m_encoder; }
     size_t headerSize() const { return 16; }
-    size_t encoderSize() const { return m_encoder->bufferSize(); }
+    size_t encoderSize() const { return m_encoder->span().size(); }
 
     std::unique_ptr<IPC::Decoder> createDecoder() const
     {
-        return IPC::Decoder::create(m_encoder->buffer(), m_encoder->bufferSize(), { });
+        return IPC::Decoder::create(m_encoder->span(), { });
     }
 
 private:
@@ -95,7 +97,7 @@ public:
 
     std::unique_ptr<IPC::Decoder> createDecoder() const
     {
-        auto decoder = makeUnique<IPC::Decoder>(m_impl->buffer.data(), m_impl->buffer.size(), 0);
+        auto decoder = makeUnique<IPC::Decoder>(std::span { m_impl->buffer.data(), m_impl->buffer.size() }, 0);
         return decoder;
     }
 
@@ -105,7 +107,7 @@ private:
 
         Impl()
             : buffer(1024, static_cast<uint8_t>(0))
-            , encoder(EncoderDecoderTest::name(), buffer.data(), buffer.size())
+            , encoder(EncoderDecoderTest::name(), buffer.mutableSpan())
         { }
 
         Vector<uint8_t> buffer;
@@ -153,6 +155,25 @@ struct EncodingCounter {
     CounterValues& m_counterValues;
 };
 
+} // namespace TestWebKitAPI
+
+namespace IPC {
+
+template<> struct ArgumentCoder<TestWebKitAPI::EncodingCounter> {
+    static void encode(Encoder& encoder, const TestWebKitAPI::EncodingCounter& counter)
+    {
+        counter.encode(encoder);
+    }
+    static void encode(Encoder& encoder, TestWebKitAPI::EncodingCounter&& counter)
+    {
+        WTFMove(counter).encode(encoder);
+    }
+};
+
+} // namespace IPC
+
+namespace TestWebKitAPI {
+
 enum class EncodingCounterTestType {
     LValue,
     RValue,
@@ -349,6 +370,26 @@ struct DecodingMoveCounter {
     unsigned moveCounter { 0 };
 };
 
+} // namespace TestWebKitAPI
+
+namespace IPC {
+
+template<> struct ArgumentCoder<TestWebKitAPI::DecodingMoveCounter> {
+    template<typename Encoder>
+    static void encode(Encoder& encoder, TestWebKitAPI::DecodingMoveCounter&& counter)
+    {
+        WTFMove(counter).encode(encoder);
+    }
+    static std::optional<TestWebKitAPI::DecodingMoveCounter> decode(Decoder& decoder)
+    {
+        return TestWebKitAPI::DecodingMoveCounter::decode(decoder);
+    }
+};
+
+} // namespace IPC
+
+namespace TestWebKitAPI {
+
 TYPED_TEST_P(ArgumentCoderDecodingMoveCounterTest, DecodeDirectly)
 {
     TestFixture::encoder() << DecodingMoveCounter { };
@@ -546,25 +587,25 @@ TYPED_TEST_P(ArgumentCoderSpanTest, SimpleSpan)
 {
     std::array<uint8_t, 16> data8 { 0, 0, 0, 1, 1, 2, 3, 5, 8, 13, 21, 34, 55, 89, 144, 233 };
     std::array<uint32_t, 16> data32 { 0, 0, 0, 1, 1, 2, 3, 5, 8, 13, 21, 34, 55, 89, 144, 233 };
-    TestFixture::encoder() << Span<const uint8_t> { data8 } << Span<const uint32_t> { data32 } << Span<const uint64_t> { };
+    TestFixture::encoder() << std::span<const uint8_t> { data8 } << std::span<const uint32_t> { data32 } << std::span<const uint64_t> { };
 
     auto decoder = TestFixture::createDecoder();
     {
-        auto span = decoder->template decode<Span<const uint8_t>>();
+        auto span = decoder->template decode<std::span<const uint8_t>>();
         ASSERT_TRUE(!!span);
         ASSERT_EQ(data8.size(), span->size());
         ASSERT_EQ(data8.size() * sizeof(uint8_t), span->size_bytes());
-        ASSERT_EQ(memcmp(data8.data(), span->data(), span->size_bytes()), 0);
+        ASSERT_TRUE(equalSpans(std::span { data8 }, *span));
     }
     {
-        auto span = decoder->template decode<Span<const uint32_t>>();
+        auto span = decoder->template decode<std::span<const uint32_t>>();
         ASSERT_TRUE(!!span);
         ASSERT_EQ(data32.size(), span->size());
         ASSERT_EQ(data32.size() * sizeof(uint32_t), span->size_bytes());
-        ASSERT_EQ(memcmp(data32.data(), span->data(), span->size_bytes()), 0);
+        ASSERT_TRUE(equalSpans(std::span { data32 }, *span));
     }
     {
-        auto span = decoder->template decode<Span<const uint64_t>>();
+        auto span = decoder->template decode<std::span<const uint64_t>>();
         ASSERT_TRUE(!!span);
         ASSERT_EQ(span->data(), nullptr);
         ASSERT_EQ(span->size(), 0u);
@@ -603,25 +644,25 @@ TYPED_TEST_P(ArgumentCoderSpanTest, AlignedSpan)
     {
         // Span over the array data is encoded. Encoded data now includes the header, the previous byte, the span size, and array data.
         std::array<AlignedStructure, 2> alignedData { AlignedStructure { }, AlignedStructure { } };
-        encoder << Span<const AlignedStructure> { alignedData };
+        encoder << std::span<const AlignedStructure> { alignedData };
         ASSERT_EQ(TestFixture::encoderSize(), calculateEncodedSize(TestFixture::headerSize(),
             EncodedValue<uint8_t, 1> { }, EncodedValue<uint64_t, 1> { }, EncodedValue<AlignedStructure, 2> { }));
     }
 
     auto decoder = TestFixture::createDecoder();
-    ASSERT_EQ(decoder->currentBufferPosition(), TestFixture::headerSize());
+    ASSERT_EQ(decoder->currentBufferOffset(), TestFixture::headerSize());
     {
         auto byte = decoder->template decode<uint8_t>();
         ASSERT_TRUE(!!byte);
         ASSERT_EQ(*byte, 42);
-        ASSERT_EQ(decoder->currentBufferPosition(), calculateEncodedSize(TestFixture::headerSize(), EncodedValue<uint8_t, 1> { }));
+        ASSERT_EQ(decoder->currentBufferOffset(), calculateEncodedSize(TestFixture::headerSize(), EncodedValue<uint8_t, 1> { }));
     }
     {
-        auto alignedData = decoder->template decode<Span<const AlignedStructure>>();
+        auto alignedData = decoder->template decode<std::span<const AlignedStructure>>();
         ASSERT_TRUE(!!alignedData);
         ASSERT_NE(alignedData->data(), nullptr);
         ASSERT_EQ(alignedData->size(), 2u);
-        ASSERT_EQ(decoder->currentBufferPosition(), calculateEncodedSize(TestFixture::headerSize(),
+        ASSERT_EQ(decoder->currentBufferOffset(), calculateEncodedSize(TestFixture::headerSize(),
             EncodedValue<uint8_t, 1> { }, EncodedValue<uint64_t, 1> { }, EncodedValue<AlignedStructure, 2> { }));
     }
 }
@@ -633,7 +674,7 @@ TYPED_TEST_P(ArgumentCoderSpanTest, AlignedEmptySpan)
     auto& encoder = TestFixture::encoder();
     {
         // Only data about the empty span that's encoded is the 64-bit size value, and nothing more.
-        encoder << Span<const AlignedStructure> { };
+        encoder << std::span<const AlignedStructure> { };
         ASSERT_EQ(TestFixture::encoderSize(), calculateEncodedSize(TestFixture::headerSize(), EncodedValue<uint64_t, 1> { }));
     }
     {
@@ -643,20 +684,20 @@ TYPED_TEST_P(ArgumentCoderSpanTest, AlignedEmptySpan)
     }
 
     auto decoder = TestFixture::createDecoder();
-    ASSERT_EQ(decoder->currentBufferPosition(), TestFixture::headerSize());
+    ASSERT_EQ(decoder->currentBufferOffset(), TestFixture::headerSize());
     {
         // A valid but empty span should be decoded, meaning a null data pointer and 0 size.
-        auto alignedData = decoder->template decode<Span<const AlignedStructure>>();
+        auto alignedData = decoder->template decode<std::span<const AlignedStructure>>();
         ASSERT_TRUE(!!alignedData);
         ASSERT_EQ(alignedData->data(), nullptr);
         ASSERT_EQ(alignedData->size(), 0u);
-        ASSERT_EQ(decoder->currentBufferPosition(), calculateEncodedSize(TestFixture::headerSize(), EncodedValue<uint64_t, 1> { }));
+        ASSERT_EQ(decoder->currentBufferOffset(), calculateEncodedSize(TestFixture::headerSize(), EncodedValue<uint64_t, 1> { }));
     }
     {
         auto byte = decoder->template decode<uint8_t>();
         ASSERT_TRUE(!!byte);
         ASSERT_EQ(*byte, 42);
-        ASSERT_EQ(decoder->currentBufferPosition(), calculateEncodedSize(TestFixture::headerSize(), EncodedValue<uint64_t, 1> { }, EncodedValue<uint8_t, 1> { }));
+        ASSERT_EQ(decoder->currentBufferOffset(), calculateEncodedSize(TestFixture::headerSize(), EncodedValue<uint64_t, 1> { }, EncodedValue<uint8_t, 1> { }));
     }
 }
 
@@ -664,4 +705,19 @@ REGISTER_TYPED_TEST_SUITE_P(ArgumentCoderSpanTest,
     SimpleSpan, AlignedSpan, AlignedEmptySpan);
 INSTANTIATE_TYPED_TEST_SUITE_P(ArgumentCoderTest, ArgumentCoderSpanTest, EncoderTypes, EncoderTypeNames);
 
+template<typename T> class ArgumentCoderVectorTest : public ArgumentCoderEncoderDecoderTest<T> { };
+TYPED_TEST_SUITE_P(ArgumentCoderVectorTest);
+
+TYPED_TEST_P(ArgumentCoderVectorTest, VectorTooBig)
+{
+    std::array<uint8_t, 9> bytes { 255, 255, 255, 255, 255, 255, 255, 255, 255 };
+    TestFixture::encoder() << std::span<uint8_t, 9>(bytes);
+    auto optionalVector = TestFixture::createDecoder()->template decode<Vector<String>>();
+    ASSERT_FALSE(optionalVector);
+}
+
+REGISTER_TYPED_TEST_SUITE_P(ArgumentCoderVectorTest,
+    VectorTooBig);
+INSTANTIATE_TYPED_TEST_SUITE_P(ArgumentCoderTest, ArgumentCoderVectorTest, EncoderTypes, EncoderTypeNames);
+
 } // namespace TestWebKitAPI
