@@ -68,12 +68,20 @@ struct MockTestMessageWithAsyncReply1 {
     using Promise = WTF::NativePromise<uint64_t, IPC::Error>;
 };
 
-class MockConnectionClient final : public IPC::Connection::Client {
+class MockConnectionClient final : public IPC::Connection::Client, public RefCounted<MockConnectionClient> {
+    WTF_MAKE_FAST_ALLOCATED;
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(MockConnectionClient);
 public:
-    ~MockConnectionClient()
+    static Ref<MockConnectionClient> create()
     {
+        return adoptRef(*new MockConnectionClient);
     }
 
+    ~MockConnectionClient() = default;
+
+    void ref() const final { RefCounted::ref(); }
+    void deref() const final { RefCounted::deref(); }
+
     Vector<MessageInfo> takeMessages()
     {
         Vector<MessageInfo> result;
@@ -138,12 +146,14 @@ public:
         m_didClose = true;
     }
 
-    void didReceiveInvalidMessage(IPC::Connection&, IPC::MessageName message) override
+    void didReceiveInvalidMessage(IPC::Connection&, IPC::MessageName message, int32_t) override
     {
         m_didReceiveInvalidMessage = message;
     }
 
 private:
+    MockConnectionClient() = default;
+
     bool m_didClose { false };
     std::optional<IPC::MessageName> m_didReceiveInvalidMessage;
     Deque<MessageInfo> m_messages;
@@ -226,7 +236,7 @@ protected:
 
     struct {
         RefPtr<IPC::Connection> connection;
-        MockConnectionClient client;
+        Ref<MockConnectionClient> client = MockConnectionClient::create();
     } m_connections[2];
 };
 
